@@ -3,6 +3,8 @@
 Astronomical and physics constants in SI units.  See :mod:`astropy.constants`
 for a complete listing of constants defined in Astropy.
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
 import numpy as np
 
@@ -30,6 +32,10 @@ c = Constant('c', "Speed of light in vacuum", 2.99792458e8, 'm / (s)', 0.,
 G = Constant('G', "Gravitational constant", 6.67384e-11, 'm3 / (kg s2)',
              0.00080e-11, 'CODATA 2010', system='si')
 
+# Standard acceleration of gravity
+g0 = Constant('g0', "Standard acceleration of gravity", 9.80665, 'm / s2', 0.0,
+              'CODATA 2010', system='si')
+
 # Proton mass
 m_p = Constant('m_p', "Proton mass", 1.672621777e-27, 'kg', 0.000000074e-27,
                'CODATA 2010', system='si')
@@ -42,6 +48,10 @@ m_n = Constant('m_n', "Neutron mass", 1.674927351e-27, 'kg', 0.000000074e-27,
 m_e = Constant('m_e', "Electron mass", 9.10938291e-31, 'kg', 0.00000040e-31,
                'CODATA 2010', system='si')
 
+# Atomic mass
+u = Constant('u', "Atomic mass", 1.660538921e-27, 'kg', 0.000000073e-27,
+             'CODATA 2010', system='si')
+
 # Stefan-Boltzmann constant
 sigma_sb = Constant('sigma_sb', "Stefan-Boltzmann constant", 5.670373e-8,
                     'W / (K4 m2)', 0.000021e-8, 'CODATA 2010', system='si')
@@ -50,6 +60,10 @@ sigma_sb = Constant('sigma_sb', "Stefan-Boltzmann constant", 5.670373e-8,
 e = EMConstant('e', 'Electron charge', 1.602176565e-19, 'C', 0.000000035e-19,
                'CODATA 2010', system='si')
 
+# Electric constant
+eps0 = EMConstant('eps0', 'Electic constant', 8.854187817e-12, 'F/m', 0.0,
+                  'CODATA 2010', system='si')
+
 # Avogadro's number
 N_A = Constant('N_A', "Avogadro's number", 6.02214129e23, '1 / (mol)',
                0.00000027e23, 'CODATA 2010', system='si')
@@ -62,6 +76,26 @@ R = Constant('R', "Gas constant", 8.3144621, 'J / (K mol)', 0.0000075,
 Ryd = Constant('Ryd', 'Rydberg constant', 10973731.568539, '1 / (m)', 0.000055,
                'CODATA 2010', system='si')
 
+# Bohr radius
+a0 = Constant('a0', "Bohr radius", 0.52917721092e-10, 'm', 0.00000000017e-10,
+              'CODATA 2010', system='si')
+
+# Bohr magneton
+muB = Constant('muB', "Bohr magneton", 927.400968e-26, 'J/T', 0.00002e-26,
+               'CODATA 2010', system='si')
+
+# Fine structure constant
+alpha = Constant('alpha', "Fine-structure constant", 7.2973525698e-3, '',
+                 0.0000000024e-3, 'CODATA 2010', system='si')
+
+# Atmosphere
+atm = Constant('atmosphere', "Atmosphere", 101325, 'Pa', 0.0,
+               'CODATA 2010', system='si')
+
+# Magnetic constant
+mu0 = Constant('mu0', "Magnetic constant", 4.0e-7 * np.pi, 'N/A2', 0.0,
+               'CODATA 2010', system='si')
+
 # DISTANCE
 
 # Astronomical Unit
