@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
@@ -7,13 +6,17 @@ This module contains utility functions that are for internal use in
 astropy.coordinates.angles. Mainly they are conversions from one format
 of data to another.
 """
-from __future__ import unicode_literals
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
-import math
 import os
 from warnings import warn
 
-from .errors import *
+import numpy as np
+
+from .errors import (IllegalHourWarning, IllegalHourError,
+                     IllegalMinuteWarning, IllegalMinuteError,
+                     IllegalSecondWarning, IllegalSecondError)
 from ..utils import format_exception
 from .. import units as u
 
@@ -37,7 +40,8 @@ class _AngleParser(object):
 
     @classmethod
     def _get_simple_unit_names(cls):
-        simple_units = set(u.radian.find_equivalent_units())
+        simple_units = set(
+            u.radian.find_equivalent_units(include_prefix_units=True))
         simple_units.remove(u.deg)
         simple_units.remove(u.hourangle)
         simple_unit_names = set()
@@ -88,13 +92,17 @@ class _AngleParser(object):
                 t.value = -1.0
             return t
 
+        def t_SIMPLE_UNIT(t):
+            t.value = u.Unit(t.value)
+            return t
+        t_SIMPLE_UNIT.__doc__ = '|'.join(
+            '(?:{0})'.format(x) for x in cls._get_simple_unit_names())
+
         t_COLON = ':'
         t_DEGREE = r'd(eg(ree(s)?)?)?|°'
         t_HOUR = r'hour(s)?|h(r)?|ʰ'
-        t_MINUTE = r'm(in(ute(s)?)?)?|′|\''
-        t_SECOND = r's(ec(ond(s)?)?)?|″|\"'
-        t_SIMPLE_UNIT = '|'.join(
-            '({0})'.format(x) for x in cls._get_simple_unit_names())
+        t_MINUTE = r'm(in(ute(s)?)?)?|′|\'|ᵐ'
+        t_SECOND = r's(ec(ond(s)?)?)?|″|\"|ˢ'
 
         # A string containing ignored characters (spaces)
         t_ignore = ' '
@@ -116,6 +124,8 @@ class _AngleParser(object):
             '''
             angle : hms
                   | dms
+                  | arcsecond
+                  | arcminute
                   | simple
             '''
             p[0] = p[1]
@@ -178,8 +188,10 @@ class _AngleParser(object):
                 | sign UINT HOUR UINT MINUTE ufloat SECOND
                 | generic HOUR
             '''
-            if len(p) in (3, 4):
+            if len(p) == 3:
                 p[0] = (p[1], u.hourangle)
+            elif len(p) == 4:
+                p[0] = (p[1] * p[2], u.hourangle)
             elif len(p) in (5, 6):
                 p[0] = ((p[1] * p[2], p[4], 0.0), u.hourangle)
             elif len(p) in (7, 8):
@@ -194,8 +206,10 @@ class _AngleParser(object):
                 | sign UINT DEGREE UINT MINUTE ufloat SECOND
                 | generic DEGREE
             '''
-            if len(p) in (3, 4):
+            if len(p) == 3:
                 p[0] = (p[1], u.degree)
+            elif len(p) == 4:
+                p[0] = (p[1] * p[2], u.degree)
             elif len(p) in (5, 6):
                 p[0] = ((p[1] * p[2], p[4], 0.0), u.degree)
             elif len(p) in (7, 8):
@@ -211,6 +225,18 @@ class _AngleParser(object):
             else:
                 p[0] = (p[1], p[2])
 
+        def p_arcsecond(p):
+            '''
+            arcsecond : generic SECOND
+            '''
+            p[0] = (p[1], u.arcsecond)
+
+        def p_arcminute(p):
+            '''
+            arcminute : generic MINUTE
+            '''
+            p[0] = (p[1], u.arcminute)
+
         def p_error(p):
             raise ValueError
 
@@ -236,42 +262,32 @@ class _AngleParser(object):
                 raise ValueError(
                     "Syntax error parsing angle {0!r}".format(angle))
 
-        if unit is not None:
-            unit = u.Unit(unit)
-            if (found_unit is not None and
-                found_unit is not unit):
-                raise UnitsError(
-                    "Unit in string ({0}) does not match requested unit "
-                    "({1})".format(found_unit, unit))
-        else:
-            if found_unit is None:
-                raise UnitsError("No unit specified")
-            else:
-                unit = found_unit
+        if unit is None and found_unit is None:
+                raise u.UnitsError("No unit specified")
 
-        return found_angle, unit
+        return found_angle, found_unit
 
 
 def _check_hour_range(hrs):
     """
     Checks that the given value is in the range (-24, 24).
     """
-    if math.fabs(hrs) == 24.:
-        warn(str(IllegalHourError(hrs)))
-    elif not -24. < hrs < 24.:
+    if np.any(np.abs(hrs) == 24.):
+        warn(IllegalHourWarning(hrs, 'Treating as 24 hr'))
+    elif np.any(hrs < -24.) or np.any(hrs > 24.):
         raise IllegalHourError(hrs)
 
 
-def _check_minute_range(min):
+def _check_minute_range(m):
     """
     Checks that the given value is in the range [0,60].  If the value
     is equal to 60, then a warning is raised.
     """
-    if min == 60.:
-        warn(str(IllegalMinuteError(min)))
-    elif not 0. <= min < 60.:
-        # "Error: minutes not in range [0,60) ({0}).".format(min))
-        raise IllegalMinuteError(min)
+    if np.any(m == 60.):
+        warn(IllegalMinuteWarning(m, 'Treating as 0 min, +1 hr/deg'))
+    elif np.any(m < -60.) or np.any(m > 60.):
+        # "Error: minutes not in range [-60,60) ({0}).".format(min))
+        raise IllegalMinuteError(m)
 
 
 def _check_second_range(sec):
@@ -279,10 +295,10 @@ def _check_second_range(sec):
     Checks that the given value is in the range [0,60].  If the value
     is equal to 60, then a warning is raised.
     """
-    if sec == 60.:
-        warn(str(IllegalSecondError(sec)))
-    elif not 0. <= sec < 60.:
-        # "Error: seconds not in range [0,60) ({0}).".format(sec))
+    if np.any(sec == 60.):
+        warn(IllegalSecondWarning(sec, 'Treating as 0 sec, +1 min'))
+    elif np.any(sec < -60.) or np.any(sec > 60.):
+        # "Error: seconds not in range [-60,60) ({0}).".format(sec))
         raise IllegalSecondError(sec)
 
 
@@ -336,16 +352,13 @@ def degrees_to_dms(d):
     Convert a floating-point degree value into a ``(degree, arcminute,
     arcsecond)`` tuple.
     """
-    sign = math.copysign(1.0, d)
+    sign = np.copysign(1.0, d)
 
-    (df, d) = math.modf(abs(d))  # (degree fraction, degree)
-    (mf, m) = math.modf(df * 60.)  # (minute fraction, minute)
+    (df, d) = np.modf(np.abs(d))  # (degree fraction, degree)
+    (mf, m) = np.modf(df * 60.)  # (minute fraction, minute)
     s = mf * 60.
 
-    _check_minute_range(m)
-    _check_second_range(s)
-
-    return (float(sign * d), int(sign * m), sign * s)
+    return np.floor(sign * d), sign * np.floor(m), sign * s
 
 
 def dms_to_degrees(d, m, s):
@@ -357,15 +370,15 @@ def dms_to_degrees(d, m, s):
     _check_second_range(s)
 
     # determine sign
-    sign = math.copysign(1.0, d)
+    sign = np.copysign(1.0, d)
 
     # TODO: This will fail if d or m have values after the decimal
     # place
 
     try:
-        d = int(abs(d))
-        m = int(abs(m))
-        s = float(abs(s))
+        d = np.floor(np.abs(np.asarray(d)))
+        m = np.floor(np.abs(np.asarray(m)))
+        s = np.abs(s)
     except ValueError:
         raise ValueError(format_exception(
             "{func}: dms values ({1[0]},{2[1]},{3[2]}) could not be "
@@ -382,15 +395,15 @@ def hms_to_hours(h, m, s):
     check_hms_ranges(h, m, s)
 
     # determine sign
-    sign = math.copysign(1.0, h)
+    sign = np.copysign(1.0, h)
 
     # TODO: This will fail if d or m have values after the decimal
     # place
 
     try:
-        h = int(abs(h))
-        m = int(abs(m))
-        s = float(abs(s))
+        h = np.floor(np.abs(h))
+        m = np.floor(np.abs(m))
+        s = np.abs(s)
     except ValueError:
         raise ValueError(format_exception(
             "{func}: HMS values ({1[0]},{2[1]},{3[2]}) could not be "
@@ -446,15 +459,13 @@ def hours_to_hms(h):
     second)`` tuple.
     """
 
-    sign = math.copysign(1.0, h)
+    sign = np.copysign(1.0, h)
 
-    (hf, h) = math.modf(abs(h))  # (degree fraction, degree)
-    (mf, m) = math.modf(hf * 60.0)  # (minute fraction, minute)
+    (hf, h) = np.modf(np.abs(h))  # (degree fraction, degree)
+    (mf, m) = np.modf(hf * 60.0)  # (minute fraction, minute)
     s = mf * 60.0
 
-    check_hms_ranges(h, m, s)  # throws exception if out of range
-
-    return (float(sign * h), int(sign * m), sign * s)
+    return (np.floor(sign * h), sign * np.floor(m), sign * s)
 
 
 def radians_to_degrees(r):
@@ -490,174 +501,147 @@ def radians_to_dms(r):
     return degrees_to_dms(degrees)
 
 
-def hours_to_string(h, precision=5, pad=False, sep=('h', 'm', 's')):
+def sexagesimal_to_string(values, precision=None, pad=False, sep=(':',),
+                          fields=3):
+    """
+    Given an already separated tuple of sexagesimal values, returns
+    a string.
+
+    See `hours_to_string` and `degrees_to_string` for a higher-level
+    interface to this functionality.
+    """
+
+    # If the coordinates are negative, we need to take the absolute value of
+    # the (arc)minutes and (arc)seconds. We need to use np.abs because abs(-0)
+    # is -0.
+    values = (values[0], np.abs(values[1]), np.abs(values[2]))
+
+    if pad:
+        if values[0] < 0:
+            pad = 3
+        else:
+            pad = 2
+    else:
+        pad = 0
+
+    if not isinstance(sep, tuple):
+        sep = tuple(sep)
+
+    if not sep:  # empty string, False, or None, etc.
+        sep = ('', '', '')
+    elif len(sep) == 1:
+        sep = sep + (sep[0], '')
+    elif len(sep) == 2:
+        sep = sep + ('',)
+    elif len(sep) != 3:
+        raise ValueError(
+            "Invalid separator specification for converting angle to string.")
+
+    if fields < 1 or fields > 3:
+        raise ValueError(
+            "fields must be 1, 2, or 3")
+
+    # Simplify the expression based on the requested precision.  For
+    # example, if the seconds will round up to 60, we should convert
+    # it to 0 and carry upwards.  If the field is hidden (by the
+    # fields kwarg) we round up around the middle, 30.0.
+    if precision is None:
+        rounding_thresh = 60.0 - (10.0 ** -5)
+    else:
+        rounding_thresh = 60.0 - (10.0 ** -precision)
+
+    values = list(values)
+    if fields == 3 and values[2] >= rounding_thresh:
+        values[2] = 0.0
+        values[1] += 1.0
+    elif fields < 3 and values[2] >= 30.0:
+        values[1] += 1.0
+
+    if fields >= 2 and int(values[1]) >= 60.0:
+        values[1] = 0.0
+        values[0] += 1.0
+    elif fields < 2 and int(values[1]) >= 30.0:
+        values[0] += 1.0
+
+    literal = []
+    last_value = ''
+    literal.append('{0:0{pad}.0f}{sep[0]}')
+    if fields >= 2:
+        literal.append('{1:02d}{sep[1]}')
+    if fields == 3:
+        if precision is None:
+            last_value = '{0:g}'.format(abs(values[2]))
+        else:
+            last_value = '{0:.{precision}f}'.format(
+                abs(values[2]), precision=precision)
+        if len(last_value) == 1 or last_value[1] == '.':
+            last_value = '0' + last_value
+        literal.append('{last_value}{sep[2]}')
+    literal = ''.join(literal)
+    return literal.format(values[0], int(abs(values[1])), abs(values[2]),
+                          sep=sep, pad=pad,
+                          last_value=last_value)
+
+
+def hours_to_string(h, precision=5, pad=False, sep=('h', 'm', 's'),
+                    fields=3):
     """
     Takes a decimal hour value and returns a string formatted as hms with
     separator specified by the 'sep' parameter.
 
-    TODO: More detailed description here!
+    `h` must be a scalar.
     """
-
-    if pad:
-        if h < 0:
-            pad = 3
-        else:
-            pad = 2
-    else:
-        pad = 0
-
-    if not isinstance(sep, tuple):
-        # Note: This will convert 'hms' to ('h', 'm', 's'); a potentially nice
-        # shortcut
-        sep = tuple(sep)
-
-    if len(sep) == 1:
-        sep = sep + (sep[0], '')
-    elif len(sep) == 2:
-        sep = sep + ('',)
-    elif len(sep) != 3:
-        raise ValueError(
-            "Invalid separator specification for converting angle to string.")
-
-    literal = ('{0:0{pad}.0f}{sep[0]}{1:02d}{sep[1]}{2:0{width}.{precision}f}'
-               '{sep[2]}')
     h, m, s = hours_to_hms(h)
-    return literal.format(h, abs(m), abs(s), sep=sep, pad=pad,
-                          width=(precision + 3 if precision > 0 else 2),
-                          precision=precision)
+    return sexagesimal_to_string((h, m, s), precision=precision, pad=pad,
+                                 sep=sep, fields=fields)
 
 
-def degrees_to_string(d, precision=5, pad=False, sep=':'):
+def degrees_to_string(d, precision=5, pad=False, sep=':', fields=3):
     """
     Takes a decimal hour value and returns a string formatted as dms with
     separator specified by the 'sep' parameter.
+
+    `d` must be a scalar.
     """
-
-    if pad:
-        if d < 0:
-            pad = 3
-        else:
-            pad = 2
-    else:
-        pad = 0
-
-    if not isinstance(sep, tuple):
-        sep = tuple(sep)
-
-    if len(sep) == 1:
-        sep = sep + (sep[0], '')
-    elif len(sep) == 2:
-        sep = sep + ('',)
-    elif len(sep) != 3:
-        raise ValueError(
-            "Invalid separator specification for converting angle to string.")
-
-    literal = ('{0:0{pad}.0f}{sep[0]}{1:02d}{sep[1]}{2:0{width}.{precision}f}'
-               '{sep[2]}')
     d, m, s = degrees_to_dms(d)
-    return literal.format(d, abs(m), abs(s), sep=sep, pad=pad,
-                          width=(precision + 3 if precision > 0 else 2),
-                          precision=precision)
+    return sexagesimal_to_string((d, m, s), precision=precision, pad=pad,
+                                 sep=sep, fields=fields)
 
 
-#<----------Spherical angular distances------------->
-def small_angle_sphere_dist(lon1, lat1, lon2, lat2):
+def angular_separation(lon1, lat1, lon2, lat2):
     """
-    Euclidean angular distance "on a sphere" - only valid on sphere in the
-    small-angle approximation.
+    Angular separation between two points on a sphere
 
-    .. warning::
-        Do not use this unless you know small-angle is a valid approximation
-        for your problem and performance is a major conern.  In general this
-        is very wrong.
+    Parameters
+    ----------
+    lon1, lat1, lon2, lat2 : Angle, Quantity or float
+        Longitude and latitude of the two points.  Quantities should be in
+        angular units; floats in radians
 
-    Inputs must be in radians.
+    Returns
+    -------
+    angular separation : Quantity or float
+        Type depends on input; Quantity in angular units, or float in radians
+
+    Notes
+    -----
+    The angular separation is calculated using the Vincenty formula [1]_,
+    which is slighly more complex and computationally expensive than
+    some alternatives, but is stable at at all distances, including the
+    poles and antipodes.
+
+    .. [1] http://en.wikipedia.org/wiki/Great-circle_distance
     """
 
-    from math import cos
-
-    dlat = lat2 - lat1
-    dlon = (lon2 - lon1) * cos((lat1 + lat2) / 2.)
-
-    return (dlat ** 2 + dlon ** 2) ** 0.5
-
-
-def simple_sphere_dist(lon1, lat1, lon2, lat2):
-    """
-    Simple formula for angular distance on a sphere: numerically unstable
-    for small distances.
-
-    Inputs must be in radians.
-    """
-
-    # FIXME: array: use numpy functions
-    from math import acos, sin, cos
-
-    cdlon = cos(lon2 - lon1)
-    return acos(sin(lat1) * sin(lat2) + cos(lat1) * cos(-lat2) * cdlon)
-
-
-def haversine_sphere_dist(lon1, lat1, lon2, lat2):
-    """
-    Haversine formula for angular distance on a sphere: more stable at poles
-
-    Inputs must be in radians.
-    """
-
-    # FIXME: array: use numpy functions
-    from math import asin, sin, cos
-
-    sdlat = sin((lat2 - lat1) / 2)
-    sdlon = sin((lon2 - lon1) / 2)
-    coslats = cos(lat1) * cos(lat2)
-
-    return 2 * asin((sdlat ** 2 + coslats * sdlon ** 2) ** 0.5)
-
-
-def haversine_atan_sphere_dist(lon1, lat1, lon2, lat2):
-    """
-    Haversine formula for angular distance on a sphere: more stable at poles.
-    This version uses arctan instead of arcsin and thus does better with sign
-    conventions.
-
-    Inputs must be in radians.
-    """
-
-    # FIXME: array: use numpy functions
-    from math import atan2, sin, cos
-
-    sdlat = sin((lat2 - lat1) / 2)
-    sdlon = sin((lon2 - lon1) / 2)
-    coslats = cos(lat1) * cos(lat2)
-
-    numerator = sdlat ** 2 + coslats * sdlon ** 2
-
-    return 2 * atan2(numerator ** 0.5, (1 - numerator) ** 0.5)
-
-
-def vincenty_sphere_dist(lon1, lat1, lon2, lat2):
-    """
-    Vincenty formula for angular distance on a sphere: stable at poles and
-    antipodes but more complex/computationally expensive.
-
-    Note that this is the only version actually used in the `AngularSeparation`
-    classes, so the other `*_spher_dist` functions are only for possible
-    future internal use.
-
-    Inputs must be in radians.
-    """
-    #FIXME: array: use numpy functions
-    from math import atan2, sin, cos
-
-    sdlon = sin(lon2 - lon1)
-    cdlon = cos(lon2 - lon1)
-    slat1 = sin(lat1)
-    slat2 = sin(lat2)
-    clat1 = cos(lat1)
-    clat2 = cos(lat2)
+    sdlon = np.sin(lon2 - lon1)
+    cdlon = np.cos(lon2 - lon1)
+    slat1 = np.sin(lat1)
+    slat2 = np.sin(lat2)
+    clat1 = np.cos(lat1)
+    clat2 = np.cos(lat2)
 
     num1 = clat2 * sdlon
     num2 = clat1 * slat2 - slat1 * clat2 * cdlon
     denominator = slat1 * slat2 + clat1 * clat2 * cdlon
 
-    return atan2((num1 ** 2 + num2 ** 2) ** 0.5, denominator)
+    return np.arctan2(np.sqrt(num1 ** 2 + num2 ** 2), denominator)
