@@ -31,24 +31,29 @@ core.py:
 ## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import os
-import sys
 import re
 import csv
 import itertools
+import functools
 import numpy
-from contextlib import contextmanager
 
 from ...table import Table
 from ...utils.data import get_readable_fileobj
+from ...utils import OrderedDict
+from . import connect
+
+# Global dictionary mapping format arg to the corresponding Reader class
+FORMAT_CLASSES = {}
+
 
 class InconsistentTableError(ValueError):
     pass
 
 # Python 3 compatibility tweaks.  Should work back through 2.4.
 try:
-    import cStringIO as io
+    from cStringIO import StringIO
 except ImportError:
-    import io
+    from io import StringIO
 
 try:
     next = next
@@ -101,14 +106,12 @@ class Column(object):
     The key attributes of a Column object are:
 
     * **name** : column name
-    * **index** : column index (first column has index=0, second has index=1, etc)
     * **type** : column type (NoType, StrType, NumType, FloatType, IntType)
     * **str_vals** : list of column values as strings
     * **data** : list of converted column values
     """
-    def __init__(self, name, index):
+    def __init__(self, name):
         self.name = name
-        self.index = index
         self.type = NoType
         self.str_vals = []
         self.fill_values = {}
@@ -132,7 +135,7 @@ class BaseInputter(object):
         """
         try:
             if (hasattr(table, 'read') or
-                ('\n' not in table and '\r' not in table + '')):
+                    ('\n' not in table and '\r' not in table + '')):
                 with get_readable_fileobj(table) as file_obj:
                     table = file_obj.read()
             lines = table.splitlines()
@@ -246,7 +249,7 @@ class DefaultSplitter(BaseSplitter):
 
     def __init__(self):
         self.csv_writer = None
-        self.csv_writer_out = io.StringIO()
+        self.csv_writer_out = StringIO()
 
     def __call__(self, lines):
         """Return an iterator over the table ``lines``, where each iterator output
@@ -258,18 +261,22 @@ class DefaultSplitter(BaseSplitter):
         if self.process_line:
             lines = [self.process_line(x) for x in lines]
 
-        if self.delimiter == '\s':
+        # In Python 2.x the inputs to csv cannot be unicode.  In Python 3 these
+        # lines do nothing.
+        escapechar = None if self.escapechar is None else str(self.escapechar)
+        quotechar = None if self.quotechar is None else str(self.quotechar)
+        delimiter = None if self.delimiter is None else str(self.delimiter)
+
+        if delimiter == '\s':
             delimiter = ' '
-        else:
-            delimiter = self.delimiter
 
         csv_reader = csv.reader(lines,
-                                delimiter = delimiter,
-                                doublequote = self.doublequote,
-                                escapechar =self.escapechar,
-                                quotechar = self.quotechar,
-                                quoting = self.quoting,
-                                skipinitialspace = self.skipinitialspace
+                                delimiter=delimiter,
+                                doublequote=self.doublequote,
+                                escapechar=escapechar,
+                                quotechar=quotechar,
+                                quoting=self.quoting,
+                                skipinitialspace=self.skipinitialspace
                                 )
         for vals in csv_reader:
             if self.process_val:
@@ -278,19 +285,20 @@ class DefaultSplitter(BaseSplitter):
                 yield vals
 
     def join(self, vals):
-        if self.delimiter is None:
-            delimiter = ' '
-        else:
-            delimiter = self.delimiter
+
+        # In Python 2.x the inputs to csv cannot be unicode
+        escapechar = None if self.escapechar is None else str(self.escapechar)
+        quotechar = None if self.quotechar is None else str(self.quotechar)
+        delimiter = ' ' if self.delimiter is None else str(self.delimiter)
 
         if self.csv_writer is None:
             self.csv_writer = csv.writer(self.csv_writer_out,
-                                         delimiter = self.delimiter,
-                                         doublequote = self.doublequote,
-                                         escapechar = self.escapechar,
-                                         quotechar = self.quotechar,
-                                         quoting = self.quoting,
-                                         lineterminator = '',
+                                         delimiter=delimiter,
+                                         doublequote=self.doublequote,
+                                         escapechar=escapechar,
+                                         quotechar=quotechar,
+                                         quoting=self.quoting,
+                                         lineterminator='',
                                          )
         self.csv_writer_out.seek(0)
         self.csv_writer_out.truncate()
@@ -341,38 +349,33 @@ class BaseHeader(object):
     :param comment: regular expression for comment lines
     :param splitter_class: Splitter class for splitting data lines into columns
     :param names: list of names corresponding to each data column
-    :param include_names: list of names to include in output (default=None selects all names)
-    :param exclude_names: list of names to exlude from output (applied after ``include_names``)
     """
     auto_format = 'col%d'
     start_line = None
     comment = None
     splitter_class = DefaultSplitter
     names = None
-    include_names = None
-    exclude_names = None
     write_spacer_lines = ['ASCII_TABLE_WRITE_SPACER_LINE']
 
     def __init__(self):
         self.splitter = self.__class__.splitter_class()
 
     def _set_cols_from_names(self):
-        # Filter full list of non-null column names with the include/exclude lists
-        names = set(self.names)
-        if self.include_names is not None:
-            names.intersection_update(self.include_names)
-        if self.exclude_names is not None:
-            names.difference_update(self.exclude_names)
+        self.cols = [Column(name=x) for x in self.names]
 
-        self.cols = [Column(name=x, index=i) for i, x in enumerate(self.names) if x in names]
+    def update_meta(self, lines, meta):
+        """
+        Extract any table-level metadata, e.g. keywords, comments, column metadata, from
+        the table ``lines`` and update the OrderedDict ``meta`` in place.  This base
+        method does nothing.
+        """
+        pass
 
     def get_cols(self, lines):
         """Initialize the header Column objects from the table ``lines``.
 
         Based on the previously set Header attributes find or create the column names.
-        Sets ``self.cols`` with the list of Columns.  This list only includes the actual
-        requested columns after filtering by the include_names and exclude_names
-        attributes.  See ``self.names`` for the full list.
+        Sets ``self.cols`` with the list of Columns.
 
         :param lines: list of table lines
         :returns: None
@@ -381,22 +384,20 @@ class BaseHeader(object):
         start_line = _get_line_index(self.start_line, self.process_lines(lines))
         if start_line is None:
             # No header line so auto-generate names from n_data_cols
-            if self.names is None:
-                # Get the data values from the first line of table data to determine n_data_cols
-                try:
-                    first_data_vals = next(self.data.get_str_vals())
-                except StopIteration:
-                    raise InconsistentTableError('No data lines found so cannot autogenerate '
-                                                 'column names')
-                n_data_cols = len(first_data_vals)
-                self.names = [self.auto_format % i for i in range(1, n_data_cols+1)]
+            # Get the data values from the first line of table data to determine n_data_cols
+            try:
+                first_data_vals = next(self.data.get_str_vals())
+            except StopIteration:
+                raise InconsistentTableError('No data lines found so cannot autogenerate '
+                                             'column names')
+            n_data_cols = len(first_data_vals)
+            self.names = [self.auto_format % i for i in range(1, n_data_cols+1)]
 
-        elif self.names is None:
-            # No column names supplied so read them from header line in table.
+        else:
             for i, line in enumerate(self.process_lines(lines)):
                 if i == start_line:
                     break
-            else: # No header line matching
+            else:  # No header line matching
                 raise ValueError('No header line found in table')
 
             self.names = next(self.splitter([line]))
@@ -424,22 +425,6 @@ class BaseHeader(object):
         """Return the column names of the table"""
         return tuple(col.name for col in self.cols)
 
-    def _get_n_data_cols(self):
-        """Return the number of expected data columns from data splitting.
-        This is either explicitly set (typically for fixedwidth splitters)
-        or set to self.names otherwise.
-        """
-        if not hasattr(self, '_n_data_cols'):
-            self._n_data_cols = len(self.names)
-        return self._n_data_cols
-
-    def _set_n_data_cols(self, val):
-        """Return the number of expected data columns from data splitting.
-        """
-        self._n_data_cols = val
-
-    n_data_cols = property(_get_n_data_cols, _set_n_data_cols)
-
     def get_type_map_key(self, col):
         return col.raw_type
 
@@ -449,7 +434,7 @@ class BaseHeader(object):
             return self.col_type_map[type_map_key.lower()]
         except KeyError:
             raise ValueError('Unknown data type ""%s"" for column "%s"' % (
-                    col.raw_type, col.name))
+                col.raw_type, col.name))
 
 
 class BaseData(object):
@@ -523,7 +508,13 @@ class BaseData(object):
 
         """
         if self.fill_values:
-            #if input is only one <fill_spec>, then make it a list
+            # when we write tables the columns may be astropy.table.Columns
+            # which don't carry a fill_values by default
+            for col in cols:
+                if ~hasattr(col, 'fill_values'):
+                    col.fill_values = {}
+
+            # if input is only one <fill_spec>, then make it a list
             try:
                 self.fill_values[0] + ''
                 self.fill_values = [self.fill_values]
@@ -561,6 +552,14 @@ class BaseData(object):
                     col.str_vals[i] = col.fill_values[str_val]
                     col.mask[i] = True
 
+    def _replace_vals(self, cols):
+        """Replace string values in col.str_vals"""
+        if self.fill_values:
+            for col in (col for col in cols if col.fill_values):
+                for i, str_val in ((i, x) for i, x in enumerate(col.str_vals)
+                                   if x in col.fill_values):
+                    col.str_vals[i] = col.fill_values[str_val]
+
     def write(self, lines):
         if hasattr(self.start_line, '__call__'):
             raise TypeError('Start_line attribute cannot be callable for write()')
@@ -570,27 +569,22 @@ class BaseData(object):
         while len(lines) < data_start_line:
             lines.append(itertools.cycle(self.write_spacer_lines))
 
-        with self._set_col_formats(self.cols, self.formats):
-            col_str_iters = [col.iter_str_vals() for col in self.cols]
-            for vals in izip(*col_str_iters):
-                lines.append(self.splitter.join(vals))
+        self._set_fill_values(self.cols)
+        self._set_col_formats()
+        for col in self.cols:
+            col.str_vals = list(col.iter_str_vals())
+        self._replace_vals(self.cols)
+        col_str_iters = [col.str_vals for col in self.cols]
+        for vals in zip(*col_str_iters):
+            lines.append(self.splitter.join(vals))
 
-    @contextmanager
-    def _set_col_formats(self, cols, formats):
+    def _set_col_formats(self):
         """
-        Context manager to save the internal column formats in `cols` and
-        override with any custom `formats`.
         """
-        orig_formats = [col.format for col in cols]
-        for col in cols:
-            if col.name in formats:
-                col.format = formats[col.name]
+        for col in self.cols:
+            if col.name in self.formats:
+                col.format = self.formats[col.name]
 
-        yield  # execute the nested context manager block
-
-        # Restore the original column format values
-        for col, orig_format in izip(cols, orig_formats):
-            col.format = orig_format
 
 def convert_numpy(numpy_type):
     """Return a tuple ``(converter_func, converter_type)``.  The converter
@@ -617,6 +611,7 @@ def convert_numpy(numpy_type):
         return numpy.array(vals, numpy_type)
     return converter, converter_type
 
+
 class BaseOutputter(object):
     """Output table as a dict of column objects keyed on column name.  The
     table data are stored as plain python lists within the column objects.
@@ -671,24 +666,107 @@ class TableOutputter(BaseOutputter):
                           convert_numpy(numpy.float),
                           convert_numpy(numpy.str)]
 
-    def __call__(self, cols):
+    def __call__(self, cols, meta):
         self._convert_vals(cols)
 
         # If there are any values that were filled and tagged with a mask bit then this
         # will be a masked table.  Otherwise use a plain table.
         masked = any(hasattr(col, 'mask') and numpy.any(col.mask) for col in cols)
 
-        out = Table([x.data for x in cols], names=[x.name for x in cols], masked=masked)
+        out = Table([x.data for x in cols], names=[x.name for x in cols], masked=masked,
+                    meta=meta['table'])
         for col, out_col in zip(cols, out.columns.values()):
             if masked and hasattr(col, 'mask'):
                 out_col.data.mask = col.mask
-            for attr in ('format', 'units', 'description'):
+            for attr in ('format', 'unit', 'description'):
                 if hasattr(col, attr):
                     setattr(out_col, attr, getattr(col, attr))
 
         return out
 
 
+class MetaBaseReader(type):
+    def __init__(cls, name, bases, dct):
+        super(MetaBaseReader, cls).__init__(name, bases, dct)
+
+        format = dct.get('_format_name')
+        if format is None:
+            return
+
+        FORMAT_CLASSES[format] = cls
+
+        io_formats = ['ascii.' + format] + dct.get('_io_registry_format_aliases', [])
+
+        if dct.get('_io_registry_suffix'):
+            func = functools.partial(connect.io_identify, dct['_io_registry_suffix'])
+            connect.io_registry.register_identifier(io_formats[0], Table, func)
+
+        for io_format in io_formats:
+            func = functools.partial(connect.io_read, io_format)
+            connect.io_registry.register_reader(io_format, Table, func)
+
+            if dct.get('_io_registry_can_write', True):
+                func = functools.partial(connect.io_write, io_format)
+                connect.io_registry.register_writer(io_format, Table, func)
+
+
+def _is_number(x):
+    try:
+        x = float(x)
+        return True
+    except ValueError:
+        pass
+    return False
+
+
+def _apply_include_exclude_names(table, names, include_names, exclude_names, strict_names):
+    """Apply names, include_names and exclude_names to a table.
+
+    :param table: input table (Reader object, NumPy struct array, list of lists, etc)
+    :param names: list of names to override those in table (default=None uses existing names)
+    :param include_names: list of names to include in output (default=None selects all names)
+    :param exclude_names: list of names to exlude from output (applied after ``include_names``)
+    :param strict_names: apply strict checks on column names
+    """
+    # Check column names.  This must be done before applying the names transformation
+    # so that guessing will fail appropriately if `names` is supplied.  For instance
+    # if the basic reader is given a table with no column header row.
+    if strict_names:
+        # Impose strict requirements on column names (normally used in guessing)
+        bads = [" ", ",", "|", "\t", "'", '"']
+        for name in table.colnames:
+            if (_is_number(name) or
+                    len(name) == 0 or
+                    name[0] in bads or
+                    name[-1] in bads):
+                raise ValueError('Column name {0!r} does not meet strict name requirements'
+                                 .format(name))
+
+    if names is not None:
+        # Rename table column names to those passed by user
+        if len(names) != len(table.colnames):
+            raise ValueError('Length of names argument ({0}) does not match number'
+                             ' of table columns ({1})'.format(len(names), len(table.colnames)))
+
+        # Temporarily rename with names that are not in `names` or `table.colnames`.
+        # This ensures that rename succeeds regardless of existing names.
+        xxxs = 'x' * max(len(name) for name in list(names) + list(table.colnames))
+        for ii, colname in enumerate(table.colnames):
+            table.rename_column(colname, xxxs + str(ii))
+
+        for ii, name in enumerate(names):
+            table.rename_column(xxxs + str(ii), name)
+
+    names = set(table.colnames)
+    if include_names is not None:
+        names.intersection_update(include_names)
+    if exclude_names is not None:
+        names.difference_update(exclude_names)
+    if names != set(table.colnames):
+        remove_names = set(table.colnames) - set(names)
+        table.remove_columns(remove_names)
+
+
 class BaseReader(object):
     """Class providing methods to read and write an ASCII table using the specified
     header, data, inputter, and outputter instances.
@@ -702,18 +780,30 @@ class BaseReader(object):
     The default behavior is to raise an InconsistentTableError.
 
     """
+    __metaclass__ = MetaBaseReader
+
+    names = None
+    include_names = None
+    exclude_names = None
+    strict_names = False
+
     def __init__(self):
         self.header = BaseHeader()
         self.data = BaseData()
         self.inputter = BaseInputter()
         self.outputter = TableOutputter()
-        self.meta = {}                  # Placeholder for storing table metadata
         # Data and Header instances benefit from a little cross-coupling.  Header may need to
         # know about number of data columns for auto-column name generation and Data may
         # need to know about header (e.g. for fixed-width tables where widths are spec'd in header.
         self.data.header = self.header
         self.header.data = self.data
 
+        # Metadata, consisting of table-level meta and column-level meta.  The latter
+        # could include information about column type, description, formatting, etc,
+        # depending on the table meta format.
+        self.meta = OrderedDict(table=OrderedDict(),
+                                cols=OrderedDict())
+
     def read(self, table):
         """Read the ``table`` and return the results in a format determined by
         the ``outputter`` attribute.
@@ -743,23 +833,31 @@ class BaseReader(object):
         self.data.header = self.header
         self.header.data = self.data
 
+        # Get a list of the lines (rows) in the table
         self.lines = self.inputter.get_lines(table)
+
+        # Set self.data.data_lines to a slice of lines contain the data rows
         self.data.get_data_lines(self.lines)
+
+        # Extract table meta values (e.g. keywords, comments, etc).  Updates self.meta.
+        self.header.update_meta(self.lines, self.meta)
+
+        # Get the table column definitions
         self.header.get_cols(self.lines)
-        cols = self.header.cols         # header.cols corresponds to *output* columns requested
-        n_data_cols = self.header.n_data_cols # number of data cols expected from splitter
+
+        cols = self.header.cols
         self.data.splitter.cols = cols
 
         for i, str_vals in enumerate(self.data.get_str_vals()):
-            if len(str_vals) != n_data_cols:
-                str_vals = self.inconsistent_handler(str_vals, n_data_cols)
+            if len(str_vals) != len(cols):
+                str_vals = self.inconsistent_handler(str_vals, len(cols))
 
-                #if str_vals is None, we skip this row
+                # if str_vals is None, we skip this row
                 if str_vals is None:
                     continue
 
-                #otherwise, we raise an error only if it is still inconsistent
-                if len(str_vals) != n_data_cols:
+                # otherwise, we raise an error only if it is still inconsistent
+                if len(str_vals) != len(cols):
                     errmsg = ('Number of header columns (%d) inconsistent with '
                               'data columns (%d) at data line %d\n'
                               'Header values: %s\n'
@@ -767,13 +865,16 @@ class BaseReader(object):
                                                    [x.name for x in cols], str_vals))
                     raise InconsistentTableError(errmsg)
 
-            for col in cols:
-                col.str_vals.append(str_vals[col.index])
+            for j, col in enumerate(cols):
+                col.str_vals.append(str_vals[j])
 
         self.data.masks(cols)
-        table = self.outputter(cols)
+        table = self.outputter(cols, self.meta)
         self.cols = self.header.cols
 
+        _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names,
+                                     self.strict_names)
+
         return table
 
     def inconsistent_handler(self, str_vals, ncols):
@@ -793,7 +894,7 @@ class BaseReader(object):
             raised in read().  Can also be None, in which case the row will be
             skipped.
         """
-        #an empty list will always trigger an InconsistentTableError in read()
+        # an empty list will always trigger an InconsistentTableError in read()
         return str_vals
 
     @property
@@ -814,9 +915,13 @@ class BaseReader(object):
         :param table: input table data (astropy.table.Table object)
         :returns: list of strings corresponding to ASCII table
         """
+
+        _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names,
+                                     self.strict_names)
+
         # link information about the columns to the writer object (i.e. self)
-        self.header.cols = table.cols
-        self.data.cols = table.cols
+        self.header.cols = table.columns.values()
+        self.data.cols = table.columns.values()
 
         # Write header and data to lines list
         lines = []
@@ -825,6 +930,7 @@ class BaseReader(object):
 
         return lines
 
+
 class ContinuationLinesInputter(BaseInputter):
     """Inputter where lines ending in ``continuation_char`` are joined
     with the subsequent line.  Example::
@@ -882,9 +988,10 @@ extra_reader_pars = ('Reader', 'Inputter', 'Outputter',
                      'delimiter', 'comment', 'quotechar', 'header_start',
                      'data_start', 'data_end', 'converters',
                      'data_Splitter', 'header_Splitter',
-                     'names', 'include_names', 'exclude_names',
+                     'names', 'include_names', 'exclude_names', 'strict_names',
                      'fill_values', 'fill_include_names', 'fill_exclude_names')
 
+
 def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
     """Initialize a table reader allowing for common customizations.  See ui.get_reader()
     for param docs.  This routine is for internal (package) use only and is useful
@@ -923,11 +1030,17 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
     if 'header_Splitter' in kwargs:
         reader.header.splitter = kwargs['header_Splitter']()
     if 'names' in kwargs:
-        reader.header.names = kwargs['names']
+        reader.names = kwargs['names']
     if 'include_names' in kwargs:
-        reader.header.include_names = kwargs['include_names']
+        reader.include_names = kwargs['include_names']
     if 'exclude_names' in kwargs:
-        reader.header.exclude_names = kwargs['exclude_names']
+        reader.exclude_names = kwargs['exclude_names']
+    # Strict names is normally set only within the guessing process to
+    # indicate that column names cannot be numeric or have certain
+    # characters at the beginning or end.  It gets used in
+    # core._apply_include_exclude_names().
+    if 'strict_names' in kwargs:
+        reader.strict_names = kwargs['strict_names']
     if 'fill_values' in kwargs:
         reader.data.fill_values = kwargs['fill_values']
     if 'fill_include_names' in kwargs:
@@ -937,12 +1050,13 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
 
     return reader
 
-extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats', 
+extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats',
                      'strip_whitespace',
                      'names', 'include_names', 'exclude_names',
                      'fill_values', 'fill_include_names',
                      'fill_exclude_names')
 
+
 def _get_writer(Writer, **kwargs):
     """Initialize a table writer allowing for common customizations. This
     routine is for internal (package) use only and is useful because it depends
@@ -975,9 +1089,9 @@ def _get_writer(Writer, **kwargs):
     if 'names' in kwargs:
         writer.header.names = kwargs['names']
     if 'include_names' in kwargs:
-        writer.header.include_names = kwargs['include_names']
+        writer.include_names = kwargs['include_names']
     if 'exclude_names' in kwargs:
-        writer.header.exclude_names = kwargs['exclude_names']
+        writer.exclude_names = kwargs['exclude_names']
     if 'fill_values' in kwargs:
         writer.data.fill_values = kwargs['fill_values']
     if 'fill_include_names' in kwargs:
