@@ -3,12 +3,78 @@ import re
 
 import numpy as np
 
+from ....utils import OrderedDict
 from ....tests.helper import pytest
-from ... import ascii as asciitable
+from ... import ascii as asciitable  # TODO: delete this line, use ascii.*
+from ... import ascii
+from ....table import Table
 
-from .common import (raises, numpy_lt_1p5,
-                     assert_equal, assert_almost_equal, assert_true,
-                     setup_function, teardown_function, has_isnan)
+from .common import (raises, assert_equal, assert_almost_equal,
+                     assert_true, setup_function, teardown_function)
+
+
+def test_guess_with_names_arg():
+    """
+    Make sure reading a table with guess=True gives the expected result when
+    the names arg is specified.
+    """
+    # This is a NoHeader format table and so `names` should replace
+    # the default col0, col1 names.  It fails as a Basic format
+    # table when guessing because the column names would be '1', '2'.
+    dat = ascii.read(['1,2', '3,4'], names=('a', 'b'))
+    assert len(dat) == 2
+    assert dat.colnames == ['a', 'b']
+
+    # This is a Basic format table and the first row
+    # gives the column names 'c', 'd', which get replaced by 'a', 'b'
+    dat = ascii.read(['c,d', '3,4'], names=('a', 'b'))
+    assert len(dat) == 1
+    assert dat.colnames == ['a', 'b']
+
+    # This is also a Basic format table and the first row
+    # gives the column names 'c', 'd', which get replaced by 'a', 'b'
+    dat = ascii.read(['c d', 'e f'], names=('a', 'b'))
+    assert len(dat) == 1
+    assert dat.colnames == ['a', 'b']
+
+
+def test_guess_with_format_arg():
+    """
+    When the format or Reader is explicitly given then disable the
+    strict column name checking in guessing.
+    """
+    dat = ascii.read(['1,2', '3,4'], format='basic')
+    assert len(dat) == 1
+    assert dat.colnames == ['1', '2']
+
+    dat = ascii.read(['1,2', '3,4'], names=('a', 'b'), format='basic')
+    assert len(dat) == 1
+    assert dat.colnames == ['a', 'b']
+
+    dat = ascii.read(['1,2', '3,4'], Reader=ascii.Basic)
+    assert len(dat) == 1
+    assert dat.colnames == ['1', '2']
+
+    dat = ascii.read(['1,2', '3,4'], names=('a', 'b'), Reader=ascii.Basic)
+    assert len(dat) == 1
+    assert dat.colnames == ['a', 'b']
+
+    # For good measure check the same in the unified I/O interface
+    dat = Table.read(['1,2', '3,4'], format='ascii.basic')
+    assert len(dat) == 1
+    assert dat.colnames == ['1', '2']
+
+    dat = Table.read(['1,2', '3,4'], format='ascii.basic', names=('a', 'b'))
+    assert len(dat) == 1
+    assert dat.colnames == ['a', 'b']
+
+
+@raises(ValueError)
+def test_read_with_names_arg():
+    """
+    Test that a bad value of `names` raises an exception.
+    """
+    dat = ascii.read(['c d', 'e f'], names=('a', ), guess=False)
 
 
 def test_read_all_files():
@@ -21,7 +87,28 @@ def test_read_all_files():
             test_opts = testfile['opts'].copy()
             if 'guess' not in test_opts:
                 test_opts['guess'] = guess
-            table = asciitable.read(testfile['name'], **testfile['opts'])
+            table = asciitable.read(testfile['name'], **test_opts)
+            assert_equal(table.dtype.names, testfile['cols'])
+            for colname in table.dtype.names:
+                assert_equal(len(table[colname]), testfile['nrows'])
+
+
+def test_read_all_files_via_table():
+    for testfile in get_testfiles():
+        if testfile.get('skip'):
+            print('\n\n******** SKIPPING %s' % testfile['name'])
+            continue
+        print('\n\n******** READING %s' % testfile['name'])
+        for guess in (True, False):
+            test_opts = testfile['opts'].copy()
+            if 'guess' not in test_opts:
+                test_opts['guess'] = guess
+            if 'Reader' in test_opts:
+                format = 'ascii.{0}'.format(test_opts['Reader']._format_name)
+                del test_opts['Reader']
+            else:
+                format = 'ascii'
+            table = Table.read(testfile['name'], format=format, **test_opts)
             assert_equal(table.dtype.names, testfile['cols'])
             for colname in table.dtype.names:
                 assert_equal(len(table[colname]), testfile['nrows'])
@@ -45,7 +132,6 @@ def test_guess_all_files():
                 assert_equal(len(table[colname]), testfile['nrows'])
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_daophot_indef():
     """Test that INDEF is correctly interpreted as a missing value"""
     table = asciitable.read('t/daophot2.dat', Reader=asciitable.Daophot)
@@ -55,7 +141,6 @@ def test_daophot_indef():
         assert np.all(table[colname].mask == mask_value)
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_daophot_types():
     """
     Test specific data types which are different from what would be
@@ -110,13 +195,13 @@ def test_missing_file():
 
 
 def test_set_names():
-    names = ('c1','c2','c3', 'c4', 'c5', 'c6')
+    names = ('c1', 'c2', 'c3', 'c4', 'c5', 'c6')
     data = asciitable.read('t/simple3.txt', names=names, delimiter='|')
     assert_equal(data.dtype.names, names)
 
 
 def test_set_include_names():
-    names = ('c1','c2','c3', 'c4', 'c5', 'c6')
+    names = ('c1', 'c2', 'c3', 'c4', 'c5', 'c6')
     include_names = ('c1', 'c3')
     data = asciitable.read('t/simple3.txt', names=names, include_names=include_names,
                            delimiter='|')
@@ -153,7 +238,6 @@ def test_custom_process_lines():
     assert_equal(len(data), 3)
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_custom_process_line():
     def process_line(line):
         line_out = re.sub(r'^\|\s*', '', line.strip())
@@ -234,66 +318,59 @@ def test_comment_lines():
     assert_equal(table.comment_lines, ['# first comment', '  # second comment'])
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a','1'), **testfile['opts'])
-    assert_true((data['a'].mask==[False,True]).all())
-    assert_true((data['a']==[1,1]).all())
-    assert_true((data['b'].mask==[False,True]).all())
-    assert_true((data['b']==[2,1]).all())
+    data = asciitable.read(f, fill_values=('a', '1'), **testfile['opts'])
+    assert_true((data['a'].mask == [False, True]).all())
+    assert_true((data['a'] == [1, 1]).all())
+    assert_true((data['b'].mask == [False, True]).all())
+    assert_true((data['b'] == [2, 1]).all())
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_col():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a','1', 'b'), **testfile['opts'])
+    data = asciitable.read(f, fill_values=('a', '1', 'b'), **testfile['opts'])
     check_fill_values(data)
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_include_names():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a','1'),
+    data = asciitable.read(f, fill_values=('a', '1'),
                            fill_include_names = ['b'], **testfile['opts'])
     check_fill_values(data)
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_exclude_names():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a','1'),
+    data = asciitable.read(f, fill_values=('a', '1'),
                            fill_exclude_names = ['a'], **testfile['opts'])
     check_fill_values(data)
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def check_fill_values(data):
     """compare array column by column with expectation """
-    assert_true((data['a'].mask==[False,False]).all())
-    assert_true((data['a']==['1','a']).all())
-    assert_true((data['b'].mask==[False,True]).all())
+    assert_true((data['a'].mask == [False, False]).all())
+    assert_true((data['a'] == ['1', 'a']).all())
+    assert_true((data['b'].mask == [False, True]).all())
     # Check that masked value is "do not care" in comparison
-    assert_true((data['b']==[2, -999]).all())
+    assert_true((data['b'] == [2, -999]).all())
     data['b'].mask = False  # explicitly unmask for comparison
-    assert_true((data['b']==[2,1]).all())
+    assert_true((data['b'] == [2, 1]).all())
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_list():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=[('a','42'),('1','42','a')],
+    data = asciitable.read(f, fill_values=[('a', '42'), ('1', '42', 'a')],
                            **testfile['opts'])
     data['a'].mask = False  # explicitly unmask for comparison
-    assert_true((data['a']==[42,42]).all())
+    assert_true((data['a'] == [42, 42]).all())
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_masking_Cds():
     f = 't/cds.dat'
     testfile = get_testfiles(f)
@@ -303,7 +380,6 @@ def test_masking_Cds():
     assert_true(not data['Fit'].mask[0])
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_null_Ipac():
     f = 't/ipac.dat'
     testfile = get_testfiles(f)
@@ -315,6 +391,21 @@ def test_null_Ipac():
     assert np.all(data.mask == mask)
 
 
+def test_Ipac_meta():
+    keywords = OrderedDict((('intval', 1),
+                            ('floatval', 2.3e3),
+                            ('date', "Wed Sp 20 09:48:36 1995"),
+                            ('key_continue', 'IPAC keywords can continue across lines')))
+    comments = ['This is an example of a valid comment']
+    f = 't/ipac.dat'
+    testfile = get_testfiles(f)
+    data = asciitable.read(f, **testfile['opts'])
+    assert data.meta['keywords'].keys() == keywords.keys()
+    for data_kv, kv in zip(data.meta['keywords'].values(), keywords.values()):
+        assert data_kv['value'] == kv
+    assert data.meta['comments'] == comments
+
+
 def test_set_guess_kwarg():
     """Read a file using guess with one of the typical guess_kwargs explicitly set."""
     data = asciitable.read('t/space_delim_no_header.dat',
@@ -329,7 +420,48 @@ def test_read_rdb_wrong_type():
     table = """col1\tcol2
 N\tN
 1\tHello"""
-    dat = asciitable.read(table, Reader=asciitable.Rdb)
+    asciitable.read(table, Reader=asciitable.Rdb)
+
+
+def test_default_missing():
+    """Read a table with empty values and ensure that corresponding entries are masked"""
+    table = '\n'.join(['a,b,c,d',
+                       '1,3,,',
+                       '2, , 4.0 , ss '])
+    dat = asciitable.read(table)
+    assert dat.masked is True
+    assert dat.pformat() == [' a   b   c   d ',
+                             '--- --- --- ---',
+                             '  1   3  --  --',
+                             '  2  -- 4.0  ss']
+
+    # Single row table with a single missing element
+    table = """ a \n "" """
+    dat = asciitable.read(table)
+    assert dat.pformat() == [' a ',
+                             '---',
+                             ' --']
+    assert dat['a'].dtype.kind == 'i'
+
+    # Same test with a fixed width reader
+    table = '\n'.join([' a   b   c   d ',
+                       '--- --- --- ---',
+                       '  1   3        ',
+                       '  2     4.0  ss'])
+    dat = asciitable.read(table, Reader=asciitable.FixedWidthTwoLine)
+    assert dat.masked is True
+    assert dat.pformat() == [' a   b   c   d ',
+                             '--- --- --- ---',
+                             '  1   3  --  --',
+                             '  2  -- 4.0  ss']
+
+    dat = asciitable.read(table, Reader=asciitable.FixedWidthTwoLine, fill_values=None)
+    assert dat.masked is False
+    assert dat.pformat() == [' a   b   c   d ',
+                             '--- --- --- ---',
+                             '  1   3        ',
+                             '  2     4.0  ss']
+
 
 def get_testfiles(name=None):
     """Set up information about the columns, number of rows, and reader params to
@@ -358,7 +490,6 @@ def get_testfiles(name=None):
                   'Fit'),
          'name': 't/cds.dat',
          'nrows': 1,
-         'skip': numpy_lt_1p5,
          'opts': {'Reader': asciitable.Cds}},
         {'cols': ('a', 'b', 'c'),
          'name': 't/commented_header.dat',
@@ -401,7 +532,6 @@ def get_testfiles(name=None):
         {'cols': ('ra', 'dec', 'sai', 'v2', 'sptype'),
          'name': 't/ipac.dat',
          'nrows': 2,
-         'skip': numpy_lt_1p5,
          'opts': {'Reader': asciitable.Ipac}},
         {'cols': ('col0',
                   'objID',
@@ -423,7 +553,6 @@ def get_testfiles(name=None):
                   'detlim90',
                   'fBlim90'),
          'name': 't/nls1_stackinfo.dbout',
-         'skip': numpy_lt_1p5,
          'nrows': 58,
          'opts': {'data_start': 2, 'delimiter': '|', 'guess': False}},
         {'cols': ('Index',
@@ -439,7 +568,6 @@ def get_testfiles(name=None):
                   'AK',
                   'Fit'),
          'name': 't/no_data_cds.dat',
-         'skip': numpy_lt_1p5,
          'nrows': 0,
          'opts': {'Reader': asciitable.Cds}},
         {'cols': ('ID',
@@ -470,12 +598,10 @@ def get_testfiles(name=None):
         {'cols': ('ra', 'dec', 'sai', 'v2', 'sptype'),
          'name': 't/no_data_ipac.dat',
          'nrows': 0,
-         'skip': numpy_lt_1p5,
          'opts': {'Reader': asciitable.Ipac}},
         {'cols': ('ra', 'v2'),
          'name': 't/ipac.dat',
          'nrows': 2,
-         'skip': numpy_lt_1p5,
          'opts': {'Reader': asciitable.Ipac, 'include_names': ['ra', 'v2']}},
         {'cols': ('a', 'b', 'c'),
          'name': 't/no_data_with_header.dat',
@@ -508,7 +634,7 @@ def get_testfiles(name=None):
         {'cols': ('col1', 'col2', 'col3'),
          'name': 't/space_delim_no_header.dat',
          'nrows': 2,
-         'opts': {}},
+         'opts': {'Reader': asciitable.NoHeader}},
         {'cols': ('obsid', 'offset', 'x', 'y', 'name', 'oaa'),
          'name': 't/space_delim_blank_lines.txt',
          'nrows': 3,
@@ -533,9 +659,18 @@ def get_testfiles(name=None):
          'name': 't/latex2.tex',
          'nrows': 3,
          'opts': {'Reader': asciitable.AASTex}},
-         ]
+    ]
 
     if name is not None:
         return [x for x in testfiles if x['name'] == name][0]
     else:
         return testfiles
+
+
+def test_overlapping_names():
+    """
+    Check that the names argument list can overlap with the existing column names.
+    This tests the issue in #1991.
+    """
+    t = ascii.read(['a b', '1 2'], names=['b', 'a'])
+    assert t.colnames == ['b', 'a']
