@@ -2,8 +2,6 @@
 
 import copy
 import re
-import string
-import sys
 import warnings
 
 import numpy as np
@@ -12,12 +10,14 @@ from .util import _str_to_num, _is_int, maketrans, translate, _words_group
 from .verify import _Verify, _ErrList, VerifyError, VerifyWarning
 
 from . import ENABLE_RECORD_VALUED_KEYWORD_CARDS, STRIP_HEADER_WHITESPACE
+
+from ...extern.six import string_types
 from ...utils import deprecated
+from ...utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
 
 
 __all__ = ['Card', 'CardList', 'create_card', 'create_card_from_string',
-           'upper_key', 'createCard', 'createCardFromString', 'upperKey',
-           'Undefined']
+           'upper_key', 'Undefined']
 
 
 FIX_FP_TABLE = maketrans('de', 'DE')
@@ -28,8 +28,11 @@ CARD_LENGTH = 80
 BLANK_CARD = ' ' * CARD_LENGTH
 KEYWORD_LENGTH = 8  # The max length for FITS-standard keywords
 
+VALUE_INDICATOR = '= '  # The standard FITS value indicator
+HIERARCH_VALUE_INDICATOR = '='  # HIERARCH cards may use a shortened indicator
 
-class Undefined:
+
+class Undefined(object):
     """Undefined value."""
 
     def __init__(self):
@@ -40,7 +43,7 @@ UNDEFINED = Undefined()
 
 class CardList(list):
     """
-    .. deprecated:: 3.1
+    .. deprecated:: 0.1
         `CardList` used to provide the list-like functionality for manipulating
         a header as a list of cards.  This functionality is now subsumed into
         the `Header` class itself, so it is no longer necessary to create or
@@ -66,7 +69,7 @@ class CardList(list):
                 'functionality has been subsumed by the Header class, so '
                 'CardList objects should not be directly created.  See the '
                 'PyFITS 3.1.0 CHANGELOG for more details.',
-                DeprecationWarning)
+                AstropyDeprecationWarning)
 
         # This is necessary for now to prevent a circular import
         from .header import Header
@@ -138,13 +141,13 @@ class CardList(list):
 
         return '\n'.join(str(card) for card in self)
 
-    @deprecated('3.1', alternative=':meth:`Header.copy`', pending=False)
+    @deprecated('0.1', alternative=':meth:`Header.copy`', pending=False)
     def copy(self):
         """Make a (deep)copy of the `CardList`."""
 
         return CardList(self._header.copy())
 
-    @deprecated('3.1', alternative=':meth:`Header.keys`', pending=False)
+    @deprecated('0.1', alternative=':meth:`Header.keys`', pending=False)
     def keys(self):
         """
         Return a list of all keywords from the `CardList`.
@@ -152,19 +155,19 @@ class CardList(list):
 
         return self._header.keys()
 
-    @deprecated('3.1', alternative=':meth:`Header.values`', pending=False)
+    @deprecated('0.1', alternative=':meth:`Header.values`', pending=False)
     def values(self):
         """
         Return a list of the values of all cards in the `CardList`.
 
-        For `RecordValuedKeywordCard` objects, the value returned is
+        For ``RecordValuedKeywordCard`` objects, the value returned is
         the floating point value, exclusive of the
         ``field_specifier``.
         """
 
         return self._header.values()
 
-    @deprecated('3.1', alternative=':meth:`Header.append`', pending=False)
+    @deprecated('0.1', alternative=':meth:`Header.append`', pending=False)
     def append(self, card, useblanks=True, bottom=False):
         """
         Append a `Card` to the `CardList`.
@@ -177,26 +180,25 @@ class CardList(list):
         useblanks : bool, optional
             Use any *extra* blank cards?
 
-            If `useblanks` is `True`, and if there are blank cards
-            directly before ``END``, it will use this space first,
-            instead of appending after these blank cards, so the total
-            space will not increase.  When `useblanks` is `False`, the
-            card will be appended at the end, even if there are blank
-            cards in front of ``END``.
+            If ``useblanks`` is `True`, and if there are blank cards directly
+            before ``END``, it will use this space first, instead of appending
+            after these blank cards, so the total space will not increase.
+            When ``useblanks`` is `False`, the card will be appended at the
+            end, even if there are blank cards in front of ``END``.
 
         bottom : bool, optional
-           If `False` the card will be appended after the last
-           non-commentary card.  If `True` the card will be appended
-           after the last non-blank card.
+           If `False` the card will be appended after the last non-commentary
+           card.  If `True` the card will be appended after the last non-blank
+           card.
         """
 
         self._header.append(card, useblanks=useblanks, bottom=bottom)
 
-    @deprecated('3.1', alternative=':meth:`Header.extend`', pending=False)
+    @deprecated('0.1', alternative=':meth:`Header.extend`', pending=False)
     def extend(self, cards):
         self._header.extend(cards)
 
-    @deprecated('3.1', alternative=':meth:`Header.insert`', pending=False)
+    @deprecated('0.1', alternative=':meth:`Header.insert`', pending=False)
     def insert(self, idx, card, useblanks=True):
         """
         Insert a `Card` to the `CardList`.
@@ -211,33 +213,32 @@ class CardList(list):
             The card to be inserted.
 
         useblanks : bool, optional
-            If `useblanks` is `True`, and if there are blank cards
-            directly before ``END``, it will use this space first,
-            instead of appending after these blank cards, so the total
-            space will not increase.  When `useblanks` is `False`, the
-            card will be appended at the end, even if there are blank
-            cards in front of ``END``.
+            If ``useblanks`` is `True`, and if there are blank cards directly
+            before ``END``, it will use this space first, instead of appending
+            after these blank cards, so the total space will not increase.
+            When `useblanks` is `False`, the card will be appended at the end,
+            even if there are blank cards in front of ``END``.
         """
 
         self._header.insert(idx, card, useblanks=useblanks)
 
-    @deprecated('3.1', alternative=':meth:`Header.remove`')
+    @deprecated('0.1', alternative=':meth:`Header.remove`')
     def remove(self, card):
         del self._header[self.index(card)]
 
-    @deprecated('3.1', alternative=':meth:`Header.pop`')
+    @deprecated('0.1', alternative=':meth:`Header.pop`')
     def pop(self, index=-1):
         return self._header.pop(index)
 
-    @deprecated('3.1', alternative=':meth:`Header.index`')
+    @deprecated('0.1', alternative=':meth:`Header.index`')
     def index(self, card):
         return self._header._cards.index(card)
 
-    @deprecated('3.1', alternative=':meth:`Header.count`')
+    @deprecated('0.1', alternative=':meth:`Header.count`')
     def count(self, card):
         return self._header._cards.count(card)
 
-    @deprecated('3.1', alternative=':meth:`Header.index`', pending=False)
+    @deprecated('0.1', alternative=':meth:`Header.index`', pending=False)
     def index_of(self, key, backward=False):
         """
         Get the index of a keyword in the `CardList`.
@@ -247,7 +248,7 @@ class CardList(list):
         key : str or int
             The keyword name (a string) or the index (an integer).
 
-        backward : bool, (optional)
+        backward : bool, optional
             When `True`, search the index from the ``END``, i.e.,
             backward.
 
@@ -264,7 +265,7 @@ class CardList(list):
         else:
             raise KeyError('Illegal key data type %s' % type(key))
 
-    @deprecated('3.1', alternative='`header[<wildcard_pattern>]`')
+    @deprecated('0.1', alternative='``header[<wildcard_pattern>]``')
     def filter_list(self, key):
         """
         Construct a `CardList` that contains references to all of the cards in
@@ -278,19 +279,14 @@ class CardList(list):
 
         Returns
         -------
-        cardlist :
+        cardlist
             A `CardList` object containing references to all the
             requested cards.
         """
 
         return CardList(self._header[key])
 
-    # For API backwards-compatibility
-    @deprecated('3.0', alternative=':meth:`filter_list`', pending=False)
-    def filterList(self, key):
-        return self.filter_list(key)
-
-    @deprecated('3.1', pending=False)
+    @deprecated('0.1', pending=False)
     def count_blanks(self):
         """
         Returns how many blank cards are *directly* before the ``END``
@@ -301,12 +297,15 @@ class CardList(list):
 
 
 class Card(_Verify):
+
     length = CARD_LENGTH
+    """The length of a Card image; should always be 80 for valid FITS files."""
 
     # String for a FITS standard compliant (FSC) keyword.
     _keywd_FSC_RE = re.compile(r'^[A-Z0-9_-]{0,%d}$' % KEYWORD_LENGTH)
     # This will match any printable ASCII character excluding '='
-    _keywd_hierarch_RE = re.compile(r'^(?:HIERARCH )?(?:^[ -<>-~]+ ?)+$', re.I)
+    _keywd_hierarch_RE = re.compile(r'^(?:HIERARCH +)?(?:^[ -<>-~]+ ?)+$',
+                                    re.I)
 
     # A number sub-string, either an integer or a float in fixed or
     # scientific notation.  One for FSC and one for non-FSC (NFSC) format:
@@ -387,7 +386,7 @@ class Card(_Verify):
     _rvkc_field_specifier_val = (r'(?P<keyword>%s): (?P<val>%s)' %
                                  (_rvkc_field_specifier_s, _numr_FSC))
     _rvkc_keyword_val = r'\'(?P<rawval>%s)\'' % _rvkc_field_specifier_val
-    _rvkc_keyword_val_comm = (r' +%s *(/ *(?P<comm>[ -~]*))?$' %
+    _rvkc_keyword_val_comm = (r' *%s *(/ *(?P<comm>[ -~]*))?$' %
                               _rvkc_keyword_val)
 
     _rvkc_field_specifier_val_RE = re.compile(_rvkc_field_specifier_val + '$')
@@ -404,7 +403,11 @@ class Card(_Verify):
     # (ex "'AXIS.1: 1' / a comment")
     _rvkc_keyword_val_comm_RE = re.compile(_rvkc_keyword_val_comm)
 
-    _commentary_keywords = ['', 'COMMENT', 'HISTORY', 'END']
+    _commentary_keywords = set(['', 'COMMENT', 'HISTORY', 'END'])
+
+    # The default value indicator; may be changed if required by a convention
+    # (namely HIERARCH cards)
+    _value_indicator = VALUE_INDICATOR
 
     def __init__(self, keyword=None, value=None, comment=None, **kwargs):
         # For backwards compatibility, support the 'key' keyword argument:
@@ -414,7 +417,7 @@ class Card(_Verify):
         self._keyword = None
         self._value = None
         self._comment = None
-        self._rawvalue = None
+
         self._image = None
 
         # This attribute is set to False when creating the card from a card
@@ -431,7 +434,12 @@ class Card(_Verify):
         self._invalid = False
 
         self._field_specifier = None
-        if not self._check_if_rvkc(keyword, value):
+        # These are used primarily only by RVKCs
+        self._rawkeyword = None
+        self._rawvalue = None
+
+        if not (keyword is not None and value is not None and
+                self._check_if_rvkc(keyword, value)):
             # If _check_if_rvkc passes, it will handle setting the keyword and
             # value
             if keyword is not None:
@@ -497,11 +505,12 @@ class Card(_Verify):
                 # the old behavior makes it possible to create HEIRARCH cards
                 # that would otherwise be recognized as RVKCs
                 self._hierarch = True
+                self._value_indicator = HIERARCH_VALUE_INDICATOR
 
                 if keyword_upper[:9] == 'HIERARCH ':
                     # The user explicitly asked for a HIERARCH card, so don't
                     # bug them about it...
-                    keyword = keyword[9:]
+                    keyword = keyword[9:].strip()
                 else:
                     # We'll gladly create a HIERARCH card, but a warning is
                     # also displayed
@@ -517,12 +526,14 @@ class Card(_Verify):
             raise ValueError('Keyword name %r is not a string.' % keyword)
 
     @property
-    @deprecated('3.1', alternative='the `.keyword` attribute')
+    @deprecated('0.1', alternative='the `.keyword` attribute')
     def key(self):
         return self.keyword
 
     @property
     def value(self):
+        """The value associated with the keyword stored in this card."""
+
         if self.field_specifier:
             return float(self._value)
 
@@ -557,6 +568,11 @@ class Card(_Verify):
                                   np.complexfloating, np.bool_)):
             raise ValueError('Illegal value: %r.' % value)
 
+        if isinstance(value, float) and (np.isnan(value) or np.isinf(value)):
+            raise ValueError(
+                "Floating point %r values are not allowed in FITS headers." %
+                value)
+
         if isinstance(value, unicode):
             try:
                 # Any string value must be encodable as ASCII
@@ -566,14 +582,32 @@ class Card(_Verify):
                     'FITS header values must contain standard ASCII '
                     'characters; %r contains characters not representable in '
                     'ASCII.' % value)
+        elif isinstance(value, bytes):
+            # Allow str, but only if they can be decoded to ASCII text; note
+            # this is not even allowed on Python 3 since the `bytes` type is
+            # not included in `six.string_types`.  Presently we simply don't
+            # allow bytes to be assigned to headers, as doing so would too
+            # easily mask potential user error
+            try:
+                value.decode('ascii')
+            except UnicodeDecodeError:
+                raise ValueError(
+                    'FITS header values must contain standard ASCII '
+                    'characters; %r contains characters/bytes that do not '
+                    'represent characters in ASCII.' % value)
+        elif isinstance(value, np.bool_):
+            value = bool(value)
 
         if (STRIP_HEADER_WHITESPACE() and
             (isinstance(oldvalue, basestring) and
              isinstance(value, basestring))):
             # Ignore extra whitespace when comparing the new value to the old
             different = oldvalue.rstrip() != value.rstrip()
+        elif isinstance(oldvalue, bool) or isinstance(value, bool):
+            different = oldvalue is not value
         else:
-            different = oldvalue != value
+            different = (oldvalue != value or
+                         not isinstance(value, type(oldvalue)))
 
         if different:
             self._value = value
@@ -600,6 +634,12 @@ class Card(_Verify):
             raise AttributeError('Values cannot be deleted from record-valued '
                                  'keyword cards')
 
+    @property
+    def rawkeyword(self):
+        if self._rawkeyword is None and self.keyword:
+            self._rawkeyword = self.keyword
+        return self._rawkeyword
+
     @property
     def rawvalue(self):
         if self._rawvalue is None and self.value:
@@ -692,6 +732,11 @@ class Card(_Verify):
 
     @property
     def image(self):
+        """
+        The card "image", that is, the 80 byte character string that represents
+        this card in an actual FITS header.
+        """
+
         if self._image and not self._verified:
             self.verify('fix')
         if self._image is None or self._modified:
@@ -699,11 +744,11 @@ class Card(_Verify):
         return self._image
 
     @property
-    @deprecated('3.1', alternative='the `.image` attribute')
+    @deprecated('0.1', alternative='the `.image` attribute')
     def cardimage(self):
         return self.image
 
-    @deprecated('3.1', alternative='the `.image` attribute')
+    @deprecated('0.1', alternative='the `.image` attribute')
     def ascardimage(self, option='silentfix'):
         if not self._verified:
             self.verify(option)
@@ -734,12 +779,15 @@ class Card(_Verify):
         ----------
         key : or str
             A keyword value or a ``keyword.field-specifier`` value
-
-        Returns
-        -------
-            The converted string
         """
 
+        # Test first for the most common case: a standard FITS keyword provided
+        # in standard all-caps
+        if (len(keyword) <= KEYWORD_LENGTH and
+                cls._keywd_FSC_RE.match(keyword)):
+            return keyword
+
+        # Test if this is a record-valued keyword
         match = cls._rvkc_keyword_name_RE.match(keyword)
 
         if match:
@@ -751,6 +799,7 @@ class Card(_Verify):
             # "HIERARCH HIERARCH", but shame on you if you do that.
             return keyword[9:].strip()
         else:
+            # A normal FITS keyword, but provided in non-standard case
             return keyword.strip().upper()
 
     def _check_if_rvkc(self, *args):
@@ -770,31 +819,18 @@ class Card(_Verify):
         Examples
         --------
 
-        >>> self._check_if_rvkc('DP1', 'AXIS.1: 2')
-        >>> self._check_if_rvkc('DP1.AXIS.1', 2)
-        >>> self._check_if_rvkc('DP1     = AXIS.1: 2')
+        ::
+
+            self._check_if_rvkc('DP1', 'AXIS.1: 2')
+            self._check_if_rvkc('DP1.AXIS.1', 2)
+            self._check_if_rvkc('DP1     = AXIS.1: 2')
         """
 
         if not ENABLE_RECORD_VALUED_KEYWORD_CARDS():
             return False
 
         if len(args) == 1:
-            image = args[0]
-            eq_idx = image.find('=')
-            if eq_idx < 0 or eq_idx > 9:
-                return False
-            keyword, rest = image.split('=', 1)
-            if keyword in self._commentary_keywords:
-                return False
-            match = self._rvkc_keyword_val_comm_RE.match(rest)
-            if match:
-                field_specifier = match.group('keyword')
-                self._keyword = '.'.join((keyword.strip().upper(),
-                                          field_specifier))
-                self._field_specifier = field_specifier
-                self._value = _int_or_float(match.group('val'))
-                self._rawvalue = match.group('rawval')
-                return True
+            self._check_if_rvkc_image(*args)
         elif len(args) == 2:
             keyword, value = args
             if not isinstance(keyword, basestring):
@@ -809,48 +845,102 @@ class Card(_Verify):
                 self._field_specifier = field_specifier
                 self._value = value
                 return True
-            if isinstance(value, basestring):
+
+            # Testing for ': ' is a quick way to avoid running the full regular
+            # expression, speeding this up for the majority of cases
+            if isinstance(value, string_types) and value.find(': ') > 0:
                 match = self._rvkc_field_specifier_val_RE.match(value)
                 if match and self._keywd_FSC_RE.match(keyword):
-                    field_specifier = match.group('keyword')
-                    self._keyword = '.'.join((keyword.upper(),
-                                              field_specifier))
-                    self._field_specifier = field_specifier
-                    self._value = _int_or_float(match.group('val'))
-                    self._rawvalue = value
+                    self._init_rvkc(keyword, match.group('keyword'), value,
+                                    match.group('val'))
                     return True
 
-    def _parse_keyword(self):
-        if self._check_if_rvkc(self._image):
-            return self._keyword
+    def _check_if_rvkc_image(self, *args):
+        """
+        Implements `Card._check_if_rvkc` for the case of an unparsed card
+        image.  If given one argment this is the full intact image.  If given
+        two arguments the card has already been split between keyword and
+        value+comment at the standard value indicator '= '.
+        """
 
+        if len(args) == 1:
+            image = args[0]
+            eq_idx = image.find(VALUE_INDICATOR)
+            if eq_idx < 0 or eq_idx > 9:
+                return False
+            keyword = image[:eq_idx]
+            rest = image[eq_idx + len(VALUE_INDICATOR):]
+        else:
+            keyword, rest = args
+
+        rest = rest.lstrip()
+
+        # This test allows us to skip running the full regular expression for
+        # the majority of cards that do not contain strings or that definitely
+        # do not contain RVKC field-specifiers; it's very much a
+        # micro-optimization but it does make a measurable difference
+        if not rest or rest[0] != "'" or rest.find(': ') < 2:
+            return False
+
+        match = self._rvkc_keyword_val_comm_RE.match(rest)
+        if match:
+            self._init_rvkc(keyword, match.group('keyword'),
+                            match.group('rawval'), match.group('val'))
+            return True
+
+    def _init_rvkc(self, keyword, field_specifier, field, value):
+        """
+        Sort of addendum to Card.__init__ to set the appropriate internal
+        attributes if the card was determined to be a RVKC.
+        """
+
+        keyword_upper = keyword.upper()
+        self._keyword = '.'.join((keyword_upper, field_specifier))
+        self._rawkeyword = keyword_upper
+        self._field_specifier = field_specifier
+        self._value = _int_or_float(value)
+        self._rawvalue = field
+
+    def _parse_keyword(self):
         keyword = self._image[:KEYWORD_LENGTH].strip()
         keyword_upper = keyword.upper()
-        value_indicator = self._image.find('= ')
+        val_ind_idx = self._image.find(VALUE_INDICATOR)
 
-        special = self._commentary_keywords + ['CONTINUE']
+        special = self._commentary_keywords
 
-        if keyword_upper in special or 0 <= value_indicator <= KEYWORD_LENGTH:
+        if (0 <= val_ind_idx <= KEYWORD_LENGTH or keyword_upper in special or
+                keyword_upper == 'CONTINUE'):
             # The value indicator should appear in byte 8, but we are flexible
             # and allow this to be fixed
-            if value_indicator >= 0:
-                keyword = keyword[:value_indicator]
-                keyword_upper = keyword_upper[:value_indicator]
+            if val_ind_idx >= 0:
+                keyword = keyword[:val_ind_idx]
+                rest = self._image[val_ind_idx + len(VALUE_INDICATOR):]
+
+                # So far this looks like a standard FITS keyword; check whether
+                # the value represents a RVKC; if so then we pass things off to
+                # the RVKC parser
+                if self._check_if_rvkc_image(keyword, rest):
+                    return self._keyword
+
+                keyword_upper = keyword_upper[:val_ind_idx]
 
             if keyword_upper != keyword:
                 self._modified = True
+
             return keyword_upper
         elif (keyword_upper == 'HIERARCH' and self._image[8] == ' ' and
-              '=' in self._image):
+              HIERARCH_VALUE_INDICATOR in self._image):
             # This is valid HIERARCH card as described by the HIERARCH keyword
             # convention:
             # http://fits.gsfc.nasa.gov/registry/hierarch_keyword.html
             self._hierarch = True
-            return self._image.split('=', 1)[0][9:].rstrip()
+            self._value_indicator = HIERARCH_VALUE_INDICATOR
+            keyword = self._image.split(HIERARCH_VALUE_INDICATOR, 1)[0][9:]
+            return keyword.strip()
         else:
             warnings.warn('The following header keyword is invalid or follows '
                           'an unrecognized non-standard convention:\n%s' %
-                          self._image)
+                          self._image, AstropyUserWarning)
             self._invalid = True
             return keyword
 
@@ -957,11 +1047,11 @@ class Card(_Verify):
         else:
             image = self.image
 
-        if self.keyword in self._commentary_keywords + ['CONTINUE']:
+        if self.keyword in self._commentary_keywords.union(['CONTINUE']):
             keyword, valuecomment = image.split(' ', 1)
         else:
             try:
-                delim_index = image.index('= ')
+                delim_index = image.index(self._value_indicator)
             except ValueError:
                 delim_index = None
 
@@ -974,7 +1064,7 @@ class Card(_Verify):
                 keyword = image[:8]
                 valuecomment = image[8:]
             else:
-                keyword, valuecomment = image.split('= ', 1)
+                keyword, valuecomment = image.split(self._value_indicator, 1)
         return keyword.strip(), valuecomment.strip()
 
     def _fix_keyword(self):
@@ -1083,7 +1173,9 @@ class Card(_Verify):
             comment = self._format_comment()
 
         # equal sign string
-        delimiter = '= '
+        # by default use the standard value indicator even for HIERARCH cards;
+        # later we may abbreviate it if necessary
+        delimiter = VALUE_INDICATOR
         if is_commentary:
             delimiter = ''
 
@@ -1110,7 +1202,7 @@ class Card(_Verify):
             # longstring case (CONTINUE card)
             # try not to use CONTINUE if the string value can fit in one line.
             # Instead, just truncate the comment
-            if (isinstance(self.value, str) and
+            if (isinstance(self.value, basestring) and
                 len(value) > (self.length - 10)):
                 output = self._format_long_image()
             else:
@@ -1186,7 +1278,7 @@ class Card(_Verify):
 
         # verify the equal sign position
         if (self.keyword not in self._commentary_keywords and
-            (self._image and self._image[:8].upper() != 'HIERARCH' and
+            (self._image and self._image[:9].upper() != 'HIERARCH ' and
              self._image.find('=') != 8)):
             errs.append(self.run_option(
                 option,
@@ -1198,7 +1290,8 @@ class Card(_Verify):
         # verify the key, it is never fixable
         # always fix silently the case where "=" is before column 9,
         # since there is no way to communicate back to the _keys.
-        if self._image and self._image[:8].upper() == 'HIERARCH':
+        if ((self._image and self._image[:8].upper() == 'HIERARCH') or
+                self._hierarch):
             pass
         else:
             keyword = self.keyword
@@ -1268,34 +1361,27 @@ def create_card(key='', value='', comment=''):
     return Card(key, value, comment)
 create_card.__doc__ = Card.__init__.__doc__
 # For API backwards-compatibility
-createCard = deprecated('3.0', name='createCard',
-                        alternative=':meth:`Card.__init__`')(create_card)
-create_card = deprecated('3.1', name='create_card',
-                         alternative=':meth:`Card.__init__`',
-                         pending=True)(create_card)
+create_card = deprecated('0.1', name='create_card',
+                         alternative='``Card.__init__``',
+                         pending=False)(create_card)
 
 
 def create_card_from_string(input):
     return Card.fromstring(input)
 create_card_from_string.__doc__ = Card.fromstring.__doc__
 # For API backwards-compat
-createCardFromString = \
-    deprecated('3.0', name='createCardFromString',
-               alternative=':meth:`Card.fromstring`')(create_card_from_string)
-create_card_from_string = deprecated('3.1', name='create_card_from_string',
+create_card_from_string = deprecated('0.1', name='create_card_from_string',
                                      alternative=':meth:`Card.fromstring`',
-                                     pending=True)(create_card_from_string)
+                                     pending=False)(create_card_from_string)
 
 
 def upper_key(key):
     return Card.normalize_keyword(key)
 upper_key.__doc__ = Card.normalize_keyword.__doc__
 # For API backwards-compat
-upperKey = deprecated('3.0', name='upperKey',
-                      alternative=':meth:`Card.normalize_keyword`')(upper_key)
-upper_key = deprecated('3.1', name='upper_key',
+upper_key = deprecated('0.1', name='upper_key',
                        alternative=':meth:`Card.normalize_keyword`',
-                       pending=True)(upper_key)
+                       pending=False)(upper_key)
 
 
 def _int_or_float(s):
