@@ -8,19 +8,19 @@ import shutil
 import sys
 import warnings
 
-from numpy import memmap as Memmap
-
 from . import compressed
 from .base import _BaseHDU, _ValidHDU, _NonstandardHDU, ExtensionHDU
 from .groups import GroupsHDU
 from .image import PrimaryHDU, ImageHDU
-from .table import _TableBaseHDU
-from ..file import PYTHON_MODES, _File
-from ..header import BLOCK_SIZE, _pad_length
-from ..util import (_is_int, _tmp_name, isfile, fileobj_name, fileobj_closed,
-                    fileobj_mode, ignore_sigint, _get_array_mmap, indent)
+from ..file import _File
+from ..header import _pad_length
+from ..util import (_is_int, _tmp_name, fileobj_closed, ignore_sigint,
+                    _get_array_mmap)
 from ..verify import _Verify, _ErrList, VerifyError, VerifyWarning
 
+from ....utils import indent
+from ....utils.exceptions import AstropyUserWarning
+
 
 def fitsopen(name, mode='readonly', memmap=None, save_backup=False, **kwargs):
     """Factory function to open a FITS file and return an `HDUList` object.
@@ -35,9 +35,8 @@ def fitsopen(name, mode='readonly', memmap=None, save_backup=False, **kwargs):
         'ostream'.
 
         If `name` is a file object that is already opened, `mode` must
-        match the mode the file was opened with, copyonwrite (rb),
-        readonly (rb), update (rb+), append (ab+), ostream (w),
-        denywrite (rb)).
+        match the mode the file was opened with, readonly (rb), update (rb+),
+        append (ab+), ostream (w), denywrite (rb)).
 
     memmap : bool
         Is memory mapping to be used?
@@ -131,7 +130,7 @@ class HDUList(list, _Verify):
         ----------
         hdus : sequence of HDU objects or single HDU, optional
             The HDU object(s) to comprise the `HDUList`.  Should be
-            instances of `_BaseHDU`.
+            instances of HDU classes like `ImageHDU` or `BinTableHDU`.
 
         file : file object, optional
             The opened physical file associated with the `HDUList`.
@@ -235,10 +234,10 @@ class HDUList(list, _Verify):
         self.close()
 
     @classmethod
-    def fromfile(cls, fileobj, mode='readonly', memmap=False,
+    def fromfile(cls, fileobj, mode=None, memmap=False,
                  save_backup=False, **kwargs):
         """
-        Creates an HDUList instance from a file-like object.
+        Creates an `HDUList` instance from a file-like object.
 
         The actual implementation of :func:`fitsopen`, and generally shouldn't
         be used directly.  Use :func:`open` instead (and see its
@@ -251,7 +250,7 @@ class HDUList(list, _Verify):
     @classmethod
     def fromstring(cls, data, **kwargs):
         """
-        Creates an HDUList instance from a string or other in-memory data
+        Creates an `HDUList` instance from a string or other in-memory data
         buffer containing an entire FITS file.  Similar to
         :meth:`HDUList.fromfile`, but does not accept the mode or memmap
         arguments, as they are only relevant to reading from a file on disk.
@@ -307,7 +306,7 @@ class HDUList(list, _Verify):
             file       File object associated with the HDU
             filename   Name of associated file object
             filemode   Mode in which the file was opened (readonly,
-                       copyonwrite, update, append, denywrite, ostream)
+                       update, append, denywrite, ostream)
             resized    Flag that when `True` indicates that the data has been
                        resized since the last read/write so the returned values
                        may not be valid.
@@ -348,14 +347,14 @@ class HDUList(list, _Verify):
 
     def insert(self, index, hdu):
         """
-        Insert an HDU into the `HDUList` at the given `index`.
+        Insert an HDU into the `HDUList` at the given ``index``.
 
         Parameters
         ----------
         index : int
             Index before which to insert the new HDU.
 
-        hdu : _BaseHDU instance
+        hdu : HDU object
             The HDU object to insert
         """
 
@@ -417,7 +416,7 @@ class HDUList(list, _Verify):
 
         Parameters
         ----------
-        hdu : instance of _BaseHDU
+        hdu : HDU object
             HDU to add to the `HDUList`.
         """
 
@@ -433,7 +432,7 @@ class HDUList(list, _Verify):
                 # You passed a Primary HDU but we need an Extension HDU
                 # so create an Extension HDU from the input Primary HDU.
                 # TODO: This isn't necessarily sufficient to copy the HDU;
-                # _hdrLoc and friends need to be copied too.
+                # _header_offset and friends need to be copied too.
                 hdu = ImageHDU(hdu.data, hdu.header)
         else:
             if not isinstance(hdu, (PrimaryHDU, _NonstandardHDU)):
@@ -465,9 +464,9 @@ class HDUList(list, _Verify):
         Parameters
         ----------
         key : int, str or tuple of (string, int)
-           The key identifying the HDU.  If `key` is a tuple, it is of
-           the form (`key`, `ver`) where `ver` is an ``EXTVER`` value
-           that must match the HDU being searched for.
+           The key identifying the HDU.  If ``key`` is a tuple, it is of the
+           form ``(key, ver)`` where ``ver`` is an ``EXTVER`` value that must
+           match the HDU being searched for.
 
         Returns
         -------
@@ -495,7 +494,7 @@ class HDUList(list, _Verify):
                 name = name.strip().upper()
             # 'PRIMARY' should always work as a reference to the first HDU
             if ((name == _key or (_key == 'PRIMARY' and idx == 0)) and
-                (_ver is None or _ver == hdu._extver)):
+                (_ver is None or _ver == hdu.ver)):
                 found = idx
                 nfound += 1
 
@@ -535,7 +534,7 @@ class HDUList(list, _Verify):
 
         if self.__file.mode not in ('append', 'update', 'ostream'):
             warnings.warn("Flush for '%s' mode is not supported."
-                          % self.__file.mode)
+                          % self.__file.mode, AstropyUserWarning)
             return
 
         if self._save_backup and self.__file.mode in ('append', 'update'):
@@ -549,7 +548,7 @@ class HDUList(list, _Verify):
                     backup = filename + '.bak.' + str(idx)
                     idx += 1
                 warnings.warn('Saving a backup of %s to %s.' %
-                              (filename, backup))
+                              (filename, backup), AstropyUserWarning)
                 try:
                     shutil.copy(filename, backup)
                 except IOError as e:
@@ -581,8 +580,8 @@ class HDUList(list, _Verify):
 
     def update_extend(self):
         """
-        Make sure that if the primary header needs the keyword
-        ``EXTEND`` that it has it and it is correct.
+        Make sure that if the primary header needs the keyword ``EXTEND`` that
+        it has it and it is correct.
         """
 
         if not len(self):
@@ -613,8 +612,8 @@ class HDUList(list, _Verify):
         Parameters
         ----------
         fileobj : file path, file object or file-like object
-            File to write to.  If a file object, must be opened for
-            append (ab+).
+            File to write to.  If a file object, must be opened in a
+            writeable mode.
 
         output_verify : str
             Output verification option.  Must be one of ``"fix"``,
@@ -630,44 +629,25 @@ class HDUList(list, _Verify):
         """
 
         if (len(self) == 0):
-            warnings.warn("There is nothing to write.")
+            warnings.warn("There is nothing to write.", AstropyUserWarning)
             return
 
         self.verify(option=output_verify)
 
-        # check if the file object is closed
-        closed = fileobj_closed(fileobj)
-        fmode = fileobj_mode(fileobj) or 'ab+'
-        filename = fileobj_name(fileobj)
-
-        # check if the output file already exists
-        if (isfile(fileobj) or
-            isinstance(fileobj, (basestring, gzip.GzipFile))):
-            if (os.path.exists(filename) and os.path.getsize(filename) != 0):
-                if clobber:
-                    warnings.warn("Overwriting existing file '%s'." % filename)
-                    if not closed:
-                        fileobj.close()
-                    os.remove(filename)
-                else:
-                    raise IOError("File '%s' already exists." % filename)
-        elif (hasattr(fileobj, 'len') and fileobj.len > 0):
-            if clobber:
-                warnings.warn("Overwriting existing file '%s'." % filename)
-                name.truncate(0)
-            else:
-                raise IOError("File '%s' already exists." % filename)
-
         # make sure the EXTEND keyword is there if there is extension
         self.update_extend()
 
-        mode = 'copyonwrite'
-        for key, val in PYTHON_MODES.iteritems():
-            if val == fmode:
-                mode = key
-                break
+        # make note of whether the input file object is already open, in which
+        # case we should not close it after writing (that should be the job
+        # of the caller)
+        closed = fileobj_closed(fileobj)
 
-        hdulist = fitsopen(fileobj, mode=mode)
+        # writeto is only for writing a new file from scratch, so the most
+        # sensible mode to require is 'ostream'.  This can accept an open
+        # file object that's open to write only, or in append/update modes
+        # but only if the file doesn't exist.
+        fileobj = _File(fileobj, mode='ostream', clobber=clobber)
+        hdulist = self.fromfile(fileobj)
 
         for hdu in self:
             hdu._prewriteto(checksum=checksum)
@@ -675,6 +655,7 @@ class HDUList(list, _Verify):
                 hdu._writeto(hdulist.__file)
             finally:
                 hdu._postwriteto()
+
         hdulist.close(output_verify=output_verify, closed=closed)
 
     def close(self, output_verify='exception', verbose=False, closed=True):
@@ -711,8 +692,8 @@ class HDUList(list, _Verify):
 
         Parameters
         ----------
-        output : file, bool (optional)
-            A file-like object to write the output to.  If ``False``, does not
+        output : file, bool, optional
+            A file-like object to write the output to.  If `False`, does not
             output to a file and instead returns a list of tuples representing
             the HDU info.  Writes to ``sys.stdout`` by default.
         """
@@ -764,7 +745,7 @@ class HDUList(list, _Verify):
         return None
 
     @classmethod
-    def _readfrom(cls, fileobj=None, data=None, mode='readonly',
+    def _readfrom(cls, fileobj=None, data=None, mode=None,
                   memmap=False, save_backup=False, **kwargs):
         """
         Provides the implementations from HDUList.fromfile and
@@ -773,10 +754,20 @@ class HDUList(list, _Verify):
         """
 
         if fileobj is not None:
-            # instantiate a FITS file object (ffo)
-            ffo = _File(fileobj, mode=mode, memmap=memmap)
+            if not isinstance(fileobj, _File):
+                # instantiate a FITS file object (ffo)
+                ffo = _File(fileobj, mode=mode, memmap=memmap)
+            else:
+                ffo = fileobj
+            # The pyfits mode is determined by the _File initializer if the
+            # supplied mode was None
+            mode = ffo.mode
             hdulist = cls(file=ffo)
         else:
+            if mode is None:
+                # The default mode
+                mode = 'readonly'
+
             hdulist = cls()
             # This method is currently only called from HDUList.fromstring and
             # HDUList.fromfile.  If fileobj is None then this must be the
@@ -792,15 +783,15 @@ class HDUList(list, _Verify):
                 kwargs['disable_image_compression']):
                 compressed.COMPRESSION_ENABLED = False
 
-            if mode == 'ostream':
-                # Output stream--not interested in reading/parsing the
-                # HDUs--just writing to the output file
-                return hdulist
-
             # read all HDUs
             while True:
                 try:
                     if fileobj is not None:
+                        if ffo.writeonly:
+                            # Output stream--not interested in reading/parsing
+                            # the HDUs--just writing to the output file
+                            return hdulist
+
                         try:
                             hdu = _BaseHDU.readfrom(ffo, **kwargs)
                         except EOFError:
@@ -814,7 +805,7 @@ class HDUList(list, _Verify):
                         if not data:
                             break
                         hdu = _BaseHDU.fromstring(data)
-                        data = data[hdu._datLoc + hdu._datSpan:]
+                        data = data[hdu._data_offset + hdu._data_size:]
                     hdulist.append(hdu)
                     hdu._new = False
                     if 'checksum' in kwargs:
@@ -823,7 +814,7 @@ class HDUList(list, _Verify):
                 # corrupted HDU
                 except (VerifyError, ValueError) as err:
                     warnings.warn(
-                        'Error validating header for HDU #%d (note: PyFITS '
+                        'Error validating header for HDU #%d (note: Astropy '
                         'uses zero-based indexing).\n%s\n'
                         'There may be extra bytes after the last HDU or the '
                         'file is corrupted.' %
@@ -948,7 +939,7 @@ class HDUList(list, _Verify):
                 # Collect a list of open mmaps to the data; this well be used
                 # later.  See below.
                 mmaps = [(idx, _get_array_mmap(hdu.data), hdu.data)
-                         for idx, hdu in enumerate(self) if hdu._data_loaded]
+                         for idx, hdu in enumerate(self) if hdu._has_data]
 
             hdulist.__file.close()
             self.__file.close()
@@ -979,8 +970,7 @@ class HDUList(list, _Verify):
             for hdu in self:
                 # Need to update the _file attribute and close any open mmaps
                 # on each HDU
-                if (hdu._data_loaded and
-                    _get_array_mmap(hdu.data) is not None):
+                if hdu._has_data and _get_array_mmap(hdu.data) is not None:
                     del hdu.data
                 hdu._file = ffo
 
@@ -1043,7 +1033,7 @@ class HDUList(list, _Verify):
             for hdu in self:
                 # Header:
                 nbytes = len(str(hdu._header))
-                if nbytes != (hdu._datLoc - hdu._hdrLoc):
+                if nbytes != (hdu._data_offset - hdu._header_offset):
                     self._resize = True
                     self._truncate = False
                     if verbose:
@@ -1051,12 +1041,12 @@ class HDUList(list, _Verify):
                     break
 
                 # Data:
-                if not hdu._data_loaded or hdu.data is None:
+                if not hdu._has_data:
                     continue
 
                 nbytes = hdu.size
                 nbytes = nbytes + _pad_length(nbytes)
-                if nbytes != hdu._datSpan:
+                if nbytes != hdu._data_size:
                     self._resize = True
                     self._truncate = False
                     if verbose:
@@ -1065,7 +1055,7 @@ class HDUList(list, _Verify):
 
             if self._truncate:
                 try:
-                    self.__file.truncate(hdu._datLoc + hdu._datSpan)
+                    self.__file.truncate(hdu._data_offset + hdu._data_size)
                 except IOError:
                     self._resize = True
                 self._truncate = False
