@@ -100,6 +100,7 @@ class _ImageBaseHDU(_ValidHDU):
             self._header = header
 
         self._do_not_scale_image_data = do_not_scale_image_data
+
         self._uint = uint
         self._scale_back = scale_back
 
@@ -134,6 +135,11 @@ class _ImageBaseHDU(_ValidHDU):
         self._modified = False
 
         if data is DELAYED:
+            if (not do_not_scale_image_data and
+                    (self._bscale != 1 or self._bzero != 0)):
+                # This indicates that when the data is accessed or written out
+                # to a new file it will need to be rescaled
+                self._data_needs_rescale = True
             return
         else:
             self.data = data
@@ -190,10 +196,23 @@ class _ImageBaseHDU(_ValidHDU):
 
     @lazyproperty
     def data(self):
+        """
+        Image/array data as a `~numpy.ndarray`.
+
+        Please remember that the order of axes on an Numpy array are opposite
+        of the order specified in the FITS file.  For example for a 2D image
+        the "rows" or y-axis are the first dimension, and the "columns" or
+        x-axis are the second dimension.
+
+        If the data is scaled using the BZERO and BSCALE parameters, this
+        attribute returns the data scaled to its physical values unless the
+        file was opened with ``do_not_scale_image_data=True``.
+        """
+
         if len(self._axes) < 1:
             return
 
-        data = self._get_scaled_image_data(self._datLoc, self.shape)
+        data = self._get_scaled_image_data(self._data_offset, self.shape)
         self._update_header_scale_info(data.dtype)
 
         return data
@@ -244,7 +263,7 @@ class _ImageBaseHDU(_ValidHDU):
         """
 
         if not (self._modified or self._header._modified or
-                (self._data_loaded and self.shape != self.data.shape)):
+                (self._has_data and self.shape != self.data.shape)):
             # Not likely that anything needs updating
             return
 
@@ -262,7 +281,7 @@ class _ImageBaseHDU(_ValidHDU):
         # If the data's shape has changed (this may have happened without our
         # noticing either via a direct update to the data.shape attribute) we
         # need to update the internal self._axes
-        if self._data_loaded and self.shape != self.data.shape:
+        if self._has_data and self.shape != self.data.shape:
             self._axes = list(self.data.shape)
             self._axes.reverse()
 
@@ -302,6 +321,17 @@ class _ImageBaseHDU(_ValidHDU):
     def _update_header_scale_info(self, dtype=None):
         if (not self._do_not_scale_image_data and
                 not (self._orig_bzero == 0 and self._orig_bscale == 1)):
+
+            if dtype is None:
+                dtype = self._dtype_for_bitpix()
+
+            if (dtype is not None and dtype.kind == 'u' and
+                    (self._scale_back or self._scale_back is None)):
+                # Data is pseudo-unsigned integers, and the scale_back option
+                # was not explicitly set to False, so preserve all the scale
+                # factors
+                return
+
             for keyword in ['BSCALE', 'BZERO']:
                 try:
                     del self._header[keyword]
@@ -325,11 +355,10 @@ class _ImageBaseHDU(_ValidHDU):
         """
         Scale image data by using ``BSCALE``/``BZERO``.
 
-        Call to this method will scale `data` and update the keywords
-        of ``BSCALE`` and ``BZERO`` in `_header`.  This method should
-        only be used right before writing to the output file, as the
-        data will be scaled and is therefore not very usable after the
-        call.
+        Call to this method will scale `data` and update the keywords of
+        ``BSCALE`` and ``BZERO`` in the HDU's header.  This method should only
+        be used right before writing to the output file, as the data will be
+        scaled and is therefore not very usable after the call.
 
         Parameters
         ----------
@@ -343,10 +372,10 @@ class _ImageBaseHDU(_ValidHDU):
             ``BSCALE`` and ``BZERO`` values when the data was
             read/created. If ``"minmax"``, use the minimum and maximum
             of the data to scale.  The option will be overwritten by
-            any user specified `bscale`/`bzero` values.
+            any user specified ``bscale``/``bzero`` values.
 
         bscale, bzero : int, optional
-            User-specified ``BSCALE`` and ``BZERO`` values.
+            User-specified ``BSCALE`` and ``BZERO`` values
         """
 
         if self.data is None:
@@ -433,7 +462,7 @@ class _ImageBaseHDU(_ValidHDU):
             self.scale(self.NumCode[self._orig_bitpix])
 
         self.update_header()
-        if not inplace and not self._data_loaded:
+        if not inplace and not self._has_data:
             self._update_header_scale_info()
         return super(_ImageBaseHDU, self)._prewriteto(checksum, inplace)
 
@@ -515,6 +544,7 @@ class _ImageBaseHDU(_ValidHDU):
             bits = dtype.itemsize * 8
             data = np.array(data, dtype=dtype)
             data -= np.uint64(1 << (bits - 1))
+
             return data
 
     def _get_scaled_image_data(self, offset, shape):
@@ -590,8 +620,12 @@ class _ImageBaseHDU(_ValidHDU):
                 format = self.data.dtype.name
                 format = format[format.rfind('.')+1:]
         else:
-            # if data is not touched yet, use header info.
-            format = self.NumCode[self._bitpix]
+            if self.shape and all(self.shape):
+                # Only show the format if all the dimensions are non-zero
+                # if data is not touched yet, use header info.
+                format = self.NumCode[self._bitpix]
+            else:
+                format = ''
 
         # Display shape in FITS-order
         shape = tuple(reversed(self.shape))
@@ -603,7 +637,7 @@ class _ImageBaseHDU(_ValidHDU):
         Calculate the value for the ``DATASUM`` card in the HDU.
         """
 
-        if self._data_loaded and self.data is not None:
+        if self._has_data:
             # We have the data to be used.
             d = self.data
 
@@ -705,7 +739,7 @@ class Section(object):
 
             dims = tuple(dims)
             bitpix = self.hdu._orig_bitpix
-            offset = self.hdu._datLoc + (offset * abs(bitpix) // 8)
+            offset = self.hdu._data_offset + (offset * abs(bitpix) // 8)
             data = self.hdu._get_scaled_image_data(offset, dims)
         else:
             data = self._getdata(key)
@@ -714,15 +748,14 @@ class Section(object):
 
     def _getdata(self, keys):
         out = []
-        naxis = len(self.hdu.shape)
 
         # Determine the number of slices in the set of input keys.
         # If there is only one slice then the result is a one dimensional
         # array, otherwise the result will be a multidimensional array.
-        numSlices = 0
+        n_slices = 0
         for idx, key in enumerate(keys):
             if isinstance(key, slice):
-                numSlices = numSlices + 1
+                n_slices = n_slices + 1
 
         for idx, key in enumerate(keys):
             if isinstance(key, slice):
@@ -736,7 +769,7 @@ class Section(object):
                     key1[idx] = k
                     key1 = tuple(key1)
 
-                    if numSlices > 1:
+                    if n_slices > 1:
                         # This is not the only slice in the list of keys so
                         # we simply get the data for this section and append
                         # it to the list that is output.  The out variable will
@@ -770,8 +803,10 @@ class PrimaryHDU(_ImageBaseHDU):
     FITS primary HDU class.
     """
 
+    _default_name = 'PRIMARY'
+
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
-                 uint=False, scale_back=False):
+                 uint=False, scale_back=None):
         """
         Construct a primary HDU.
 
@@ -781,8 +816,8 @@ class PrimaryHDU(_ImageBaseHDU):
             The data in the HDU.
 
         header : Header instance, optional
-            The header to be used (as a template).  If `header` is
-            `None`, a minimal header will be provided.
+            The header to be used (as a template).  If ``header`` is `None`, a
+            minimal header will be provided.
 
         do_not_scale_image_data : bool, optional
             If `True`, image data is not scaled using BSCALE/BZERO values
@@ -807,9 +842,6 @@ class PrimaryHDU(_ImageBaseHDU):
             do_not_scale_image_data=do_not_scale_image_data, uint=uint,
             scale_back=scale_back)
 
-        self._name = 'PRIMARY'
-        self._extver = 1
-
         # insert the keywords EXTEND
         if header is None:
             dim = self._header['NAXIS']
@@ -857,7 +889,7 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
     _extension = 'IMAGE'
 
     def __init__(self, data=None, header=None, name=None,
-                 do_not_scale_image_data=False, uint=False, scale_back=False):
+                 do_not_scale_image_data=False, uint=False, scale_back=None):
         """
         Construct an image HDU.
 
