@@ -1,19 +1,29 @@
+# -*- coding: utf8 -*-
 # Licensed under a 3-clause BSD style license - see PYFITS.rst
 
 import itertools
+import sys
 import warnings
 
+from io import StringIO, BytesIO
+
 import numpy as np
 
 from ....io import fits
-from ....tests.helper import pytest
+from ....io.fits.verify import VerifyWarning
+from ....extern.six import u
+from ....tests.helper import pytest, catch_warnings
 
 from . import FitsTestCase
 from .util import ignore_warnings
 from ..card import _pad
+from ..header import _pad_length, BLOCK_SIZE
 from ..util import encode_ascii
 
 
+PY3 = sys.version_info[0] >= 3
+
+
 class TestOldApiHeaderFunctions(FitsTestCase):
     """
     Tests that specifically use attributes and methods from the old
@@ -68,12 +78,6 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         assert header.cards[1].value == 0
         assert header[''] == [0, 1, 2, 3, '', '', 4]
 
-    def test_has_key(self):
-        header = fits.Header([('A', 'B', 'C'), ('D', 'E', 'F')])
-        assert header.has_key('A')
-        assert header.has_key('D')
-        assert not header.has_key('C')
-
     def test_totxtfile(self):
         hdul = fits.open(self.data('test0.fits'))
         hdul[0].header.toTxtFile(self.temp('header.txt'))
@@ -81,9 +85,9 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         hdu.header.update('MYKEY', 'FOO', 'BAR')
         hdu.header.fromTxtFile(self.temp('header.txt'), replace=True)
         assert len(hdul[0].header.ascard) == len(hdu.header.ascard)
-        assert not hdu.header.has_key('MYKEY')
-        assert not hdu.header.has_key('EXTENSION')
-        assert hdu.header.has_key('SIMPLE')
+        assert 'MYKEY' not in hdu.header
+        assert 'EXTENSION' not in hdu.header
+        assert 'SIMPLE' in hdu.header
 
         # Write the hdu out and read it back in again--it should be recognized
         # as a PrimaryHDU
@@ -97,16 +101,16 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         # hdu.header should have MYKEY keyword, and also adds PCOUNT and
         # GCOUNT, giving it 3 more keywords in total than the original
         assert len(hdul[0].header.ascard) == len(hdu.header.ascard) - 3
-        assert hdu.header.has_key('MYKEY')
-        assert not hdu.header.has_key('EXTENSION')
-        assert hdu.header.has_key('SIMPLE')
+        assert 'MYKEY' in hdu.header
+        assert 'EXTENSION' not in hdu.header
+        assert 'SIMPLE' in hdu.header
 
         with ignore_warnings():
             hdu.writeto(self.temp('test.fits'), output_verify='ignore',
                         clobber=True)
         hdul2 = fits.open(self.temp('test.fits'))
         assert len(hdul2) == 2
-        assert hdul2[1].header.has_key('MYKEY')
+        assert 'MYKEY' in hdul2[1].header
 
     def test_update_comment(self):
         hdul = fits.open(self.data('arange.fits'))
@@ -198,17 +202,17 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         # The old header.update method won't let you append a duplicate keyword
         header.append(('D', 'G', 'H'))
 
-        assert header.ascardlist().index(header.cards['A']) == 0
-        assert header.ascardlist().index(header.cards['D']) == 1
-        assert header.ascardlist().index(header.cards[('D', 1)]) == 2
+        assert header.ascard.index(header.cards['A']) == 0
+        assert header.ascard.index(header.cards['D']) == 1
+        assert header.ascard.index(header.cards[('D', 1)]) == 2
 
         # Since the original CardList class really only works on card objects
         # the count method is mostly useless since cards didn't used to compare
         # equal sensibly
-        assert header.ascardlist().count(header.cards['A']) == 1
-        assert header.ascardlist().count(header.cards['D']) == 1
-        assert header.ascardlist().count(header.cards[('D', 1)]) == 1
-        assert header.ascardlist().count(fits.Card('A', 'B', 'C')) == 0
+        assert header.ascard.count(header.cards['A']) == 1
+        assert header.ascard.count(header.cards['D']) == 1
+        assert header.ascard.count(header.cards[('D', 1)]) == 1
+        assert header.ascard.count(fits.Card('A', 'B', 'C')) == 0
 
 
 class TestHeaderFunctions(FitsTestCase):
@@ -285,7 +289,7 @@ class TestHeaderFunctions(FitsTestCase):
     def test_keyword_too_long(self):
         """Test that long Card keywords are allowed, but with a warning"""
 
-        with warnings.catch_warnings():
+        with catch_warnings():
             warnings.simplefilter('error')
             pytest.raises(UserWarning, fits.Card, 'abcdefghi', 'long')
 
@@ -299,10 +303,10 @@ class TestHeaderFunctions(FitsTestCase):
         # keyword like 'abc+' was simply not allowed.  Now it should create a
         # HIERARCH card.
 
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             c = fits.Card('abc+', 9)
-            assert len(w) == 1
-            assert c.image == _pad('HIERARCH abc+ =                    9')
+        assert len(w) == 1
+        assert c.image == _pad('HIERARCH abc+ =                    9')
 
     def test_commentary_cards(self):
         # commentary cards
@@ -396,23 +400,23 @@ class TestHeaderFunctions(FitsTestCase):
     def test_verify_invalid_equal_sign(self):
         # verification
         c = fits.Card.fromstring('abc= a6')
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             c.verify()
-            err_text1 = ("Card 'ABC' is not FITS standard (equal sign not at "
-                         "column 8)")
-            err_text2 = ("Card 'ABC' is not FITS standard (invalid value "
-                         "string: a6")
-            assert len(w) == 4
-            assert err_text1 in str(w[1].message)
-            assert err_text2 in str(w[2].message)
+        err_text1 = ("Card 'ABC' is not FITS standard (equal sign not at "
+                     "column 8)")
+        err_text2 = ("Card 'ABC' is not FITS standard (invalid value "
+                     "string: a6")
+        assert len(w) == 4
+        assert err_text1 in str(w[1].message)
+        assert err_text2 in str(w[2].message)
 
     def test_fix_invalid_equal_sign(self):
         c = fits.Card.fromstring('abc= a6')
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             c.verify('fix')
-            fix_text = "Fixed 'ABC' card to meet the FITS standard."
-            assert len(w) == 4
-            assert fix_text in str(w[1].message)
+        fix_text = "Fixed 'ABC' card to meet the FITS standard."
+        assert len(w) == 4
+        assert fix_text in str(w[1].message)
         assert str(c) == _pad("ABC     = 'a6      '")
 
     def test_long_string_value(self):
@@ -438,8 +442,8 @@ class TestHeaderFunctions(FitsTestCase):
         h1['TEST'] = 'abcdefg' * 30
 
         h2 = fits.Header()
-        with warnings.catch_warnings(record=True) as w:
-            h2['TEST'] = u'abcdefg' * 30
+        with catch_warnings() as w:
+            h2['TEST'] = u('abcdefg' * 30)
             assert len(w) == 0
 
         assert str(h1) == str(h2)
@@ -546,14 +550,14 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_hierarch_card_creation(self):
         # Test automatic upgrade to hierarch card
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             c = fits.Card('ESO INS SLIT2 Y1FRML',
                           'ENC=OFFSET+RESOL*acos((WID-(MAX+MIN))/(MAX-MIN)')
-            assert len(w) == 1
-            assert 'HIERARCH card will be created' in str(w[0].message)
-            assert (str(c) ==
-                    "HIERARCH ESO INS SLIT2 Y1FRML= "
-                    "'ENC=OFFSET+RESOL*acos((WID-(MAX+MIN))/(MAX-MIN)'")
+        assert len(w) == 1
+        assert 'HIERARCH card will be created' in str(w[0].message)
+        assert (str(c) ==
+                "HIERARCH ESO INS SLIT2 Y1FRML= "
+                "'ENC=OFFSET+RESOL*acos((WID-(MAX+MIN))/(MAX-MIN)'")
 
         # Test manual creation of hierarch card
         c = fits.Card('hierarch abcdefghi', 10)
@@ -657,7 +661,7 @@ class TestHeaderFunctions(FitsTestCase):
         msg = 'a HIERARCH card will be created'
 
         header = fits.Header()
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings(VerifyWarning) as w:
             header.update('HIERARCH BLAH BLAH', 'TESTA')
             assert len(w) == 0
             assert 'BLAH BLAH' in header
@@ -720,7 +724,7 @@ class TestHeaderFunctions(FitsTestCase):
         msg = 'a HIERARCH card will be created'
 
         header = fits.Header()
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings(VerifyWarning) as w:
             header.update('HIERARCH BLA BLA', 'TESTA')
             assert len(w) == 0
             assert 'BLA BLA' in header
@@ -747,10 +751,11 @@ class TestHeaderFunctions(FitsTestCase):
             assert header['bla bla'], 'TESTE'
 
         header = fits.Header()
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings(VerifyWarning) as w:
             # Create a HIERARCH card containing invalid characters without
             # explicitly stating 'HIERARCH'
             header.update('BLA BLA', 'TESTA')
+            print([x.category for x in w])
             assert len(w) == 1
             assert msg in str(w[0].message)
 
@@ -1136,26 +1141,6 @@ class TestHeaderFunctions(FitsTestCase):
         assert len(hdu.header) == 5
         assert hdu.header[-1] == 'some val'
 
-    def test_header_extend_unique(self):
-        """
-        Test extending the header with and without unique=True.
-        """
-        hdu = fits.PrimaryHDU()
-        hdu2 = fits.ImageHDU()
-        hdu.header['MYKEY'] = ('some val', 'some comment')
-        hdu2.header['MYKEY'] = ('some other val', 'some other comment')
-        hdu.header.extend(hdu2.header)
-        assert len(hdu.header) == 6
-        assert hdu.header[-2] == 'some val'
-        assert hdu.header[-1] == 'some other val'
-
-        hdu = fits.PrimaryHDU()
-        hdu2 = fits.ImageHDU()
-        hdu.header['MYKEY'] = ('some val', 'some comment')
-        hdu.header.extend(hdu2.header, unique=True)
-        assert len(hdu.header) == 5
-        assert hdu.header[-1] == 'some val'
-
     def test_header_extend_update(self):
         """
         Test extending the header with and without update=True.
@@ -1279,6 +1264,43 @@ class TestHeaderFunctions(FitsTestCase):
         assert header[1] == 'H'
         assert header[-1] == ''
 
+    def test_header_insert_before_keyword(self):
+        """
+        Test that a keyword name or tuple can be used to insert new keywords.
+
+        Also tests the ``after`` keyword argument.
+
+        Regression test for https://github.com/spacetelescope/PyFITS/issues/12
+        """
+
+        header = fits.Header([
+            ('NAXIS1', 10), ('COMMENT', 'Comment 1'),
+            ('COMMENT', 'Comment 3')])
+
+        header.insert('NAXIS1', ('NAXIS', 2, 'Number of axes'))
+        assert list(header.keys())[0] == 'NAXIS'
+        assert header[0] == 2
+        assert header.comments[0] == 'Number of axes'
+
+        header.insert('NAXIS1', ('NAXIS2', 20), after=True)
+        assert list(header.keys())[1] == 'NAXIS1'
+        assert list(header.keys())[2] == 'NAXIS2'
+        assert header[2] == 20
+
+        header.insert(('COMMENT', 1), ('COMMENT', 'Comment 2'))
+        assert header['COMMENT'] == ['Comment 1', 'Comment 2', 'Comment 3']
+
+        header.insert(('COMMENT', 2), ('COMMENT', 'Comment 4'), after=True)
+        assert header['COMMENT'] == ['Comment 1', 'Comment 2', 'Comment 3',
+                                     'Comment 4']
+
+        header.insert(-1, ('TEST1', True))
+        assert list(header.keys())[-2] == 'TEST1'
+
+        header.insert(-1, ('TEST2', True), after=True)
+        assert list(header.keys())[-1] == 'TEST2'
+        assert list(header.keys())[-3] == 'TEST1'
+
     def test_header_comments(self):
         header = fits.Header([('A', 'B', 'C'), ('DEF', 'G', 'H')])
         assert (repr(header.comments) ==
@@ -1453,6 +1475,79 @@ class TestHeaderFunctions(FitsTestCase):
         pytest.raises(ValueError, header.insert, len(header), 'END')
         pytest.raises(ValueError, header.set, 'END')
 
+    def test_invalid_end_cards(self):
+        """
+        Regression test for https://trac.assembla.com/pyfits/ticket/217
+
+        This tests the case where the END card looks like a normal card like
+        'END = ' and other similar oddities.  As long as a card starts with END
+        and looks like it was intended to be the END card we allow it, but with
+        a warning.
+        """
+
+        horig = fits.PrimaryHDU(data=np.arange(100)).header
+
+        def invalid_header(end, pad):
+            # Build up a goofy invalid header
+            # Start from a seemingly normal header
+            s = horig.tostring(sep='', endcard=False, padding=False)
+            # append the bogus end card
+            s += end
+            # add additional padding if requested
+            if pad:
+                s += ' ' * _pad_length(len(s))
+
+            # This will differ between Python versions
+            if isinstance(s, bytes):
+                return BytesIO(s)
+            else:
+                return StringIO(s)
+
+        # Basic case motivated by the original issue; it's as if the END card
+        # was appened by software that doesn't know to treat it specially, and
+        # it is given an = after it
+        s = invalid_header('END =', True)
+
+        with catch_warnings() as w:
+            h = fits.Header.fromfile(s)
+            assert h == horig
+            assert len(w) == 1
+            assert str(w[0].message).startswith(
+                "Unexpected bytes trailing END keyword: ' ='")
+
+        # A case similar to the last but with more spaces between END and the
+        # =, as though the '= ' value indicator were placed like that of a
+        # normal card
+        s = invalid_header('END     = ', True)
+        with catch_warnings() as w:
+            h = fits.Header.fromfile(s)
+            assert h == horig
+            assert len(w) == 1
+            assert str(w[0].message).startswith(
+                "Unexpected bytes trailing END keyword: '     ='")
+
+        # END card with trailing gibberish
+        s = invalid_header('END$%&%^*%*', True)
+        with catch_warnings() as w:
+            h = fits.Header.fromfile(s)
+            assert h == horig
+            assert len(w) == 1
+            assert str(w[0].message).startswith(
+                "Unexpected bytes trailing END keyword: '$%&%^*%*'")
+
+        # 'END' at the very end of a truncated file without padding; the way
+        # the block reader works currently this can only happen if the 'END'
+        # is at the very end of the file.
+        s = invalid_header('END', False)
+        with catch_warnings() as w:
+            # Don't raise an exception on missing padding, but still produce a
+            # warning that the END card is incomplete
+            h = fits.Header.fromfile(s, padding=False)
+            assert h == horig
+            assert len(w) == 1
+            assert str(w[0].message).startswith(
+                "Missing padding to end of the FITS block")
+
     def test_unnecessary_move(self):
         """
         Regression test for https://trac.assembla.com/pyfits/ticket/125
@@ -1557,15 +1652,15 @@ class TestHeaderFunctions(FitsTestCase):
             f.write(encode_ascii('e'))
 
         hdul = fits.open(self.temp('test.fits'))
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             hdul.writeto(self.temp('temp.fits'), output_verify='warn')
-            assert len(w) == 5
-            # The first two warnings are just the headers to the actual warning
-            # message (HDU 0, Card 4).  I'm still not sure things like that
-            # should be output as separate warning messages, but that's
-            # something to think about...
-            msg = str(w[3].message)
-            assert '(invalid value string: 5.0022221e-07)' in msg
+        assert len(w) == 5
+        # The first two warnings are just the headers to the actual warning
+        # message (HDU 0, Card 4).  I'm still not sure things like that
+        # should be output as separate warning messages, but that's
+        # something to think about...
+        msg = str(w[3].message)
+        assert '(invalid value string: 5.0022221e-07)' in msg
 
     def test_leading_zeros(self):
         """
@@ -1767,6 +1862,28 @@ class TestHeaderFunctions(FitsTestCase):
         pytest.raises(ValueError, assign, 'FOO', (erikku, 'BAZ'))
         pytest.raises(ValueError, assign, 'FOO', (erikku, erikku))
 
+    def test_assign_non_ascii(self):
+        """
+        First regression test for
+        https://github.com/spacetelescope/PyFITS/issues/37
+
+        Although test_assign_unicode ensures that Python 2 `unicode` objects
+        and Python 3 `str` objects containing non-ASCII characters cannot be
+        assigned to headers, there is a bug that allows Python 2 `str` objects
+        of arbitrary encoding containing non-ASCII characters to be passed
+        through.
+
+        On Python 3 it should not be possible to assign bytes to a header at
+        all.
+        """
+
+        h = fits.Header()
+        if PY3:
+            pytest.raises(ValueError, h.set, 'TEST',
+                          bytes('Hello', encoding='ascii'))
+        else:
+            pytest.raises(ValueError, h.set, 'TEST', str('ñ'))
+
     def test_header_strip_whitespace(self):
         """
         Regression test for https://trac.assembla.com/pyfits/ticket/146, and
@@ -1847,15 +1964,15 @@ class TestHeaderFunctions(FitsTestCase):
 
         hdu = fits.PrimaryHDU()
         # This should work with some warnings
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             hdu.header.append(c1)
             hdu.header.append(c2)
             hdu.header.append(c3)
-            assert len(w) == 3
+        assert len(w) == 3
 
         hdu.writeto(self.temp('test.fits'))
 
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             with fits.open(self.temp('test.fits')) as hdul:
                 # Merely opening the file should blast some warnings about the
                 # invalid keywords
@@ -1901,6 +2018,109 @@ class TestHeaderFunctions(FitsTestCase):
         pytest.raises(ValueError, h.set, 'TEST', float('inf'))
         pytest.raises(ValueError, h.set, 'TEST', np.inf)
 
+    def test_update_bool(self):
+        """
+        Regression test for an issue where a value of True in a header
+        cannot be updated to a value of 1, and likewise for False/0.
+        """
+
+        h = fits.Header([('TEST', True)])
+        h['TEST'] = 1
+        assert h['TEST'] is not True
+        assert isinstance(h['TEST'], int)
+        assert h['TEST'] == 1
+
+        h['TEST'] = np.bool_(True)
+        assert h['TEST'] is True
+
+        h['TEST'] = False
+        assert h['TEST'] is False
+        h['TEST'] = np.bool_(False)
+        assert h['TEST'] is False
+
+        h['TEST'] = 0
+        assert h['TEST'] is not False
+        assert isinstance(h['TEST'], int)
+        assert h['TEST'] == 0
+
+        h['TEST'] = np.bool_(False)
+        assert h['TEST'] is False
+
+    def test_update_numeric(self):
+        """
+        Regression test for https://github.com/spacetelescope/PyFITS/issues/49
+
+        Ensure that numeric values can be upcast/downcast between int, float,
+        and complex by assigning values that compare equal to the existing
+        value but are a different type.
+        """
+
+        h = fits.Header()
+        h['TEST'] = 1
+
+        # int -> float
+        h['TEST'] = 1.0
+        assert isinstance(h['TEST'], float)
+        assert str(h).startswith('TEST    =                  1.0')
+
+        # float -> int
+        h['TEST'] = 1
+        assert isinstance(h['TEST'], int)
+        assert str(h).startswith('TEST    =                    1')
+
+        # int -> complex
+        h['TEST'] = 1.0+0.0j
+        assert isinstance(h['TEST'], complex)
+        assert str(h).startswith('TEST    =           (1.0, 0.0)')
+
+        # complex -> float
+        h['TEST'] = 1.0
+        assert isinstance(h['TEST'], float)
+        assert str(h).startswith('TEST    =                  1.0')
+
+        # float -> complex
+        h['TEST'] = 1.0+0.0j
+        assert isinstance(h['TEST'], complex)
+        assert str(h).startswith('TEST    =           (1.0, 0.0)')
+
+        # complex -> int
+        h['TEST'] = 1
+        assert isinstance(h['TEST'], int)
+        assert str(h).startswith('TEST    =                    1')
+
+        # Now the same tests but with zeros
+        h['TEST'] = 0
+
+        # int -> float
+        h['TEST'] = 0.0
+        assert isinstance(h['TEST'], float)
+        assert str(h).startswith('TEST    =                  0.0')
+
+        # float -> int
+        h['TEST'] = 0
+        assert isinstance(h['TEST'], int)
+        assert str(h).startswith('TEST    =                    0')
+
+        # int -> complex
+        h['TEST'] = 0.0+0.0j
+        assert isinstance(h['TEST'], complex)
+        assert str(h).startswith('TEST    =           (0.0, 0.0)')
+
+        # complex -> float
+        h['TEST'] = 0.0
+        assert isinstance(h['TEST'], float)
+        assert str(h).startswith('TEST    =                  0.0')
+
+        # float -> complex
+        h['TEST'] = 0.0+0.0j
+        assert isinstance(h['TEST'], complex)
+        assert str(h).startswith('TEST    =           (0.0, 0.0)')
+
+        # complex -> int
+        h['TEST'] = 0
+        assert isinstance(h['TEST'], int)
+        assert str(h).startswith('TEST    =                    0')
+
 
 class TestRecordValuedKeywordCards(FitsTestCase):
     """
