@@ -1,5 +1,7 @@
 # Licensed under a 3-clause BSD style license - see PYFITS.rst
 
+from __future__ import division, with_statement
+
 import math
 import os
 import time
@@ -8,7 +10,9 @@ import warnings
 import numpy as np
 
 from ....io import fits
-from ....tests.helper import pytest, raises
+from ....tests.helper import pytest, raises, catch_warnings
+from ..hdu.compressed import SUBTRACTIVE_DITHER_1, DITHER_SEED_CHECKSUM
+from .test_table import comparerecords
 
 from . import FitsTestCase
 from .util import ignore_warnings
@@ -74,7 +78,7 @@ class TestImageFunctions(FitsTestCase):
     def test_open_2(self):
         r = fits.open(self.data('test0.fits'))
 
-        info = ([(0, 'PRIMARY', 'PrimaryHDU', 138, (), 'int16', '')] +
+        info = ([(0, 'PRIMARY', 'PrimaryHDU', 138, (), '', '')] +
                 [(x, 'SCI', 'ImageHDU', 61, (40, 40), 'int16', '')
                  for x in range(1, 5)])
 
@@ -98,7 +102,7 @@ class TestImageFunctions(FitsTestCase):
         assert hdul[0].name == 'XPRIMARY'
         assert hdul[0].name == hdul[0].header['EXTNAME']
 
-        info = [(0, 'XPRIMARY', 'PrimaryHDU', 5, (), 'uint8', '')]
+        info = [(0, 'XPRIMARY', 'PrimaryHDU', 5, (), '', '')]
         assert hdul.info(output=False) == info
 
         assert hdul['PRIMARY'] is hdul['XPRIMARY']
@@ -258,18 +262,18 @@ class TestImageFunctions(FitsTestCase):
         # make a defect HDUList first
         x = fits.ImageHDU()
         hdu = fits.HDUList(x)  # HDUList can take a list or one single HDU
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             hdu.verify()
-            text = "HDUList's 0th element is not a primary HDU."
-            assert len(w) == 3
-            assert text in str(w[1].message)
+        text = "HDUList's 0th element is not a primary HDU."
+        assert len(w) == 3
+        assert text in str(w[1].message)
 
-        with warnings.catch_warnings(record=True) as w:
+        with catch_warnings() as w:
             hdu.writeto(self.temp('test_new2.fits'), 'fix')
-            text = ("HDUList's 0th element is not a primary HDU.  "
-                    "Fixed by inserting one as 0th HDU.")
-            assert len(w) == 3
-            assert text in str(w[1].message)
+        text = ("HDUList's 0th element is not a primary HDU.  "
+                "Fixed by inserting one as 0th HDU.")
+        assert len(w) == 3
+        assert text in str(w[1].message)
 
     def test_section(self):
         # section testing
@@ -590,8 +594,33 @@ class TestImageFunctions(FitsTestCase):
         with fits.open(self.temp('test.fits')) as hdul:
             assert (hdul['SCI'].data == cube).all()
 
+    def test_subtractive_dither_seed(self):
+        """
+        Regression test for https://github.com/spacetelescope/PyFITS/issues/32
+
+        Ensure that when floating point data is compressed with the
+        SUBTRACTIVE_DITHER_1 quantization method that the correct ZDITHER0 seed
+        is added to the header, and that the data can be correctly
+        decompressed.
+        """
+
+        array = np.arange(100.0).reshape(10, 10)
+        csum = (array[0].view('uint8').sum() % 10000) + 1
+        hdu = fits.CompImageHDU(data=array,
+                                quantize_method=SUBTRACTIVE_DITHER_1,
+                                dither_seed=DITHER_SEED_CHECKSUM)
+        hdu.writeto(self.temp('test.fits'))
+
+        with fits.open(self.temp('test.fits')) as hdul:
+            assert isinstance(hdul[1], fits.CompImageHDU)
+            assert 'ZQUANTIZ' in hdul[1]._header
+            assert hdul[1]._header['ZQUANTIZ'] == 'SUBTRACTIVE_DITHER_1'
+            assert 'ZDITHER0' in hdul[1]._header
+            assert hdul[1]._header['ZDITHER0'] == csum
+            assert np.all(hdul[1].data == array)
+
     def test_disable_image_compression(self):
-        with warnings.catch_warnings():
+        with catch_warnings():
             # No warnings should be displayed in this case
             warnings.simplefilter('error')
             with fits.open(self.data('comp.fits'),
@@ -624,6 +653,16 @@ class TestImageFunctions(FitsTestCase):
         # opening and closing it.
         assert mtime == os.stat(self.temp('comp.fits')).st_mtime
 
+    def test_write_comp_hdu_direct_from_existing(self):
+        with fits.open(self.data('comp.fits')) as hdul:
+            hdul[1].writeto(self.temp('test.fits'))
+
+        with fits.open(self.data('comp.fits')) as hdul1:
+            with fits.open(self.temp('test.fits')) as hdul2:
+                assert np.all(hdul1[1].data == hdul2[1].data)
+                assert comparerecords(hdul1[1].compressed_data,
+                                      hdul2[1].compressed_data)
+
     def test_do_not_scale_image_data(self):
         hdul = fits.open(self.data('scale.fits'), do_not_scale_image_data=True)
         assert hdul[0].data.dtype == np.dtype('>i2')
@@ -998,8 +1037,8 @@ class TestImageFunctions(FitsTestCase):
             assert h[1].header['TFORM2'] == '1PB(359)'
 
     def test_image_none(self):
-        """Regression test
-        for https://github.com/spacetelescope/PyFITS/issues/27
+        """
+        Regression test for https://github.com/spacetelescope/PyFITS/issues/27
         """
 
         with fits.open(self.data('test0.fits')) as h:
@@ -1012,3 +1051,168 @@ class TestImageFunctions(FitsTestCase):
             assert h[1].header['NAXIS'] == 0
             assert 'NAXIS1' not in h[1].header
             assert 'NAXIS2' not in h[1].header
+
+    def test_compression_update_header(self):
+        """Regression test for
+        https://github.com/spacetelescope/PyFITS/issues/23
+        """
+
+        self.copy_file('comp.fits')
+        with fits.open(self.temp('comp.fits'), mode='update') as hdul:
+            assert isinstance(hdul[1], fits.CompImageHDU)
+            hdul[1].header['test1'] = 'test'
+            hdul[1]._header['test2'] = 'test2'
+
+        with fits.open(self.temp('comp.fits')) as hdul:
+            assert 'test1' in hdul[1].header
+            assert hdul[1].header['test1'] == 'test'
+            assert 'test2' in hdul[1].header
+            assert  hdul[1].header['test2'] == 'test2'
+
+        # Test update via index now:
+        with fits.open(self.temp('comp.fits'), mode='update') as hdul:
+            hdr = hdul[1].header
+            hdr[hdr.index('TEST1')] = 'foo'
+
+        with fits.open(self.temp('comp.fits')) as hdul:
+            assert hdul[1].header['TEST1'] == 'foo'
+
+        # Test slice updates
+        with fits.open(self.temp('comp.fits'), mode='update') as hdul:
+            hdul[1].header['TEST*'] = 'qux'
+
+        with fits.open(self.temp('comp.fits')) as hdul:
+            assert list(hdul[1].header['TEST*'].values()) == ['qux', 'qux']
+
+        with fits.open(self.temp('comp.fits'), mode='update') as hdul:
+            hdr = hdul[1].header
+            idx = hdr.index('TEST1')
+            hdr[idx:idx + 2] = 'bar'
+
+        with fits.open(self.temp('comp.fits')) as hdul:
+            assert list(hdul[1].header['TEST*'].values()) == ['bar', 'bar']
+
+        # Test updating a specific COMMENT card duplicate
+        with fits.open(self.temp('comp.fits'), mode='update') as hdul:
+            hdul[1].header[('COMMENT', 1)] = 'I am fire. I am death!'
+
+        with fits.open(self.temp('comp.fits')) as hdul:
+            assert hdul[1].header['COMMENT'][1] == 'I am fire. I am death!'
+            assert hdul[1]._header['COMMENT'][1] == 'I am fire. I am death!'
+
+        # Test deleting by keyword and by slice
+        with fits.open(self.temp('comp.fits'), mode='update') as hdul:
+            hdr = hdul[1].header
+            del hdr['COMMENT']
+            idx = hdr.index('TEST1')
+            del hdr[idx:idx + 2]
+
+        with fits.open(self.temp('comp.fits')) as hdul:
+            assert 'COMMENT' not in hdul[1].header
+            assert 'COMMENT' not in hdul[1]._header
+            assert 'TEST1' not in hdul[1].header
+            assert 'TEST1' not in hdul[1]._header
+            assert 'TEST2' not in hdul[1].header
+            assert 'TEST2' not in hdul[1]._header
+
+    def test_compression_update_header_with_reserved(self):
+        """
+        Ensure that setting reserved keywords related to the table data
+        structure on CompImageHDU image headers fails.
+        """
+
+        def test_set_keyword(hdr, keyword, value):
+            with catch_warnings() as w:
+                hdr[keyword] = value
+                assert len(w) == 1
+                assert str(w[0].message).startswith(
+                        "Keyword %r is reserved" % keyword)
+                assert keyword not in hdr
+
+        with fits.open(self.data('comp.fits')) as hdul:
+            hdr = hdul[1].header
+            test_set_keyword(hdr, 'TFIELDS', 8)
+            test_set_keyword(hdr, 'TTYPE1', 'Foo')
+            test_set_keyword(hdr, 'ZCMPTYPE', 'ASDF')
+            test_set_keyword(hdr, 'ZVAL1', 'Foo')
+
+    def test_compression_header_append(self):
+        with fits.open(self.data('comp.fits')) as hdul:
+            imghdr = hdul[1].header
+            tblhdr = hdul[1]._header
+            with catch_warnings() as w:
+                imghdr.append('TFIELDS')
+                assert len(w) == 1
+                assert 'TFIELDS' not in imghdr
+
+            imghdr.append(('FOO', 'bar', 'qux'), end=True)
+            assert 'FOO' in imghdr
+            assert imghdr[-1] == 'bar'
+            assert 'FOO' in tblhdr
+            assert tblhdr[-1] == 'bar'
+
+            imghdr.append(('CHECKSUM', 'abcd1234'))
+            assert 'CHECKSUM' in imghdr
+            assert imghdr['CHECKSUM'] == 'abcd1234'
+            assert 'CHECKSUM' not in tblhdr
+            assert 'ZHECKSUM' in tblhdr
+            assert tblhdr['ZHECKSUM'] == 'abcd1234'
+
+    def test_compression_header_insert(self):
+        with fits.open(self.data('comp.fits')) as hdul:
+            imghdr = hdul[1].header
+            tblhdr = hdul[1]._header
+            # First try inserting a restricted keyword
+            with catch_warnings() as w:
+                imghdr.insert(1000, 'TFIELDS')
+                assert len(w) == 1
+                assert 'TFIELDS' not in imghdr
+                assert tblhdr.count('TFIELDS') == 1
+
+            # First try keyword-relative insert
+            imghdr.insert('TELESCOP', ('OBSERVER', 'Phil Plait'))
+            assert 'OBSERVER' in imghdr
+            assert imghdr.index('OBSERVER') == imghdr.index('TELESCOP') - 1
+            assert 'OBSERVER' in tblhdr
+            assert tblhdr.index('OBSERVER') == tblhdr.index('TELESCOP') - 1
+
+            # Next let's see if an index-relative insert winds up being
+            # sensible
+            idx = imghdr.index('OBSERVER')
+            imghdr.insert('OBSERVER', ('FOO',))
+            assert 'FOO' in imghdr
+            assert imghdr.index('FOO') == idx
+            assert 'FOO' in tblhdr
+            assert tblhdr.index('FOO') == tblhdr.index('OBSERVER') - 1
+
+    def test_compression_header_set_before_after(self):
+        with fits.open(self.data('comp.fits')) as hdul:
+            imghdr = hdul[1].header
+            tblhdr = hdul[1]._header
+
+            with catch_warnings() as w:
+                imghdr.set('ZBITPIX', 77, 'asdf', after='XTENSION')
+                assert len(w) == 1
+                assert 'ZBITPIX' not in imghdr
+                assert tblhdr.count('ZBITPIX') == 1
+                assert tblhdr['ZBITPIX'] != 77
+
+            # Move GCOUNT before PCOUNT (not that there's any reason you'd
+            # *want* to do that, but it's just a test...)
+            imghdr.set('GCOUNT', 99, before='PCOUNT')
+            assert imghdr.index('GCOUNT') == imghdr.index('PCOUNT') - 1
+            assert imghdr['GCOUNT'] == 99
+            assert tblhdr.index('ZGCOUNT') == tblhdr.index('ZPCOUNT') - 1
+            assert tblhdr['ZGCOUNT'] == 99
+            assert tblhdr.index('PCOUNT') == 5
+            assert tblhdr.index('GCOUNT') == 6
+            assert tblhdr['GCOUNT'] == 1
+
+            imghdr.set('GCOUNT', 2, after='PCOUNT')
+            assert imghdr.index('GCOUNT') == imghdr.index('PCOUNT') + 1
+            assert imghdr['GCOUNT'] == 2
+            assert tblhdr.index('ZGCOUNT') == tblhdr.index('ZPCOUNT') + 1
+            assert tblhdr['ZGCOUNT'] == 2
+            assert tblhdr.index('PCOUNT') == 5
+            assert tblhdr.index('GCOUNT') == 6
+            assert tblhdr['GCOUNT'] == 1
