@@ -4,12 +4,15 @@ This module handles the conversion of various VOTABLE datatypes
 to/from TABLEDATA_ and BINARY_ formats.
 """
 
-from __future__ import division, absolute_import
+from __future__ import absolute_import, division, print_function, unicode_literals
+from ...extern import six
+from ...extern.six.moves import xrange
 
 # STDLIB
 import re
-from struct import unpack as struct_unpack
-from struct import pack as struct_pack
+import sys
+from struct import unpack as _struct_unpack
+from struct import pack as _struct_pack
 
 # THIRD-PARTY
 import numpy as np
@@ -21,7 +24,6 @@ from ...utils.xml.writer import xml_escape_cdata
 # LOCAL
 from .exceptions import (vo_raise, vo_warn, warn_or_raise, W01,
     W30, W31, W39, W46, W47, W49, W51, E01, E02, E03, E04, E05, E06)
-from .util import IS_PY3K
 
 
 __all__ = ['get_converter', 'Converter', 'table_column_to_votable_datatype']
@@ -41,6 +43,29 @@ _empty_bytes = b''
 _zero_byte = b'\0'
 
 
+if six.PY3:
+    struct_unpack = _struct_unpack
+    struct_pack = _struct_pack
+else:
+    def struct_unpack(format, s):
+        return _struct_unpack(format.encode('ascii'), s)
+
+    def struct_pack(format, *args):
+        return _struct_pack(format.encode('ascii'), *args)
+
+
+if sys.byteorder == 'little':
+    def _ensure_bigendian(x):
+        if x.dtype.byteorder != '>':
+            return x.byteswap()
+        return x
+else:
+    def _ensure_bigendian(x):
+        if x.dtype.byteorder == '<':
+            return x.byteswap()
+        return x
+
+
 def _make_masked_array(data, mask):
     """
     Masked arrays of zero length that also have a mask of zero length
@@ -57,6 +82,75 @@ def _make_masked_array(data, mask):
         return ma.array(np.array(data))
 
 
+def bitarray_to_bool(data, length):
+    """
+    Converts a bit array (a string of bits in a bytes object) to a
+    boolean Numpy array.
+
+    Parameters
+    ----------
+    data : bytes
+        The bit array.  The most significant byte is read first.
+
+    length : int
+        The number of bits to read.  The least significant bits in the
+        data bytes beyond length will be ignored.
+
+    Returns
+    -------
+    array : numpy bool array
+    """
+    results = []
+    for byte in data:
+        if not six.PY3:
+            byte = ord(byte)
+        for bit_no in range(7, -1, -1):
+            bit = byte & (1 << bit_no)
+            bit = (bit != 0)
+            results.append(bit)
+            if len(results) == length:
+                break
+        if len(results) == length:
+            break
+
+    return np.array(results, dtype='b1')
+
+
+def bool_to_bitarray(value):
+    """
+    Converts a numpy boolean array to a bit array (a string of bits in
+    a bytes object).
+
+    Parameters
+    ----------
+    value : numpy bool array
+
+    Returns
+    -------
+    bit_array : bytes
+        The first value in the input array will be the most
+        significant bit in the result.  The length will be `floor((N +
+        7) / 8)` where `N` is the length of `value`.
+    """
+    value = value.flat
+    bit_no = 7
+    byte = 0
+    bytes = []
+    for v in value:
+        if v:
+            byte |= 1 << bit_no
+        if bit_no == 0:
+            bytes.append(byte)
+            bit_no = 7
+            byte = 0
+        else:
+            bit_no -= 1
+    if bit_no != 7:
+        bytes.append(byte)
+
+    return struct_pack("%sB" % len(bytes), *bytes)
+
+
 class Converter(object):
     """
     The base class for all converters.  Each subclass handles
@@ -76,7 +170,7 @@ class Converter(object):
         found.  Used for error messages.
 
     """
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         pass
 
     @staticmethod
@@ -87,7 +181,13 @@ class Converter(object):
     def _write_length(length):
         return struct_pack(">I", int(length))
 
-    def parse(self, value, config={}, pos=None):
+    def supports_empty_values(self, config):
+        """
+        Returns True when the field can be completely empty.
+        """
+        return config.get('version_1_3_or_later')
+
+    def parse(self, value, config=None, pos=None):
         """
         Convert the string *value* from the TABLEDATA_ format into an
         object with the correct native in-memory datatype and mask flag.
@@ -106,7 +206,7 @@ class Converter(object):
         raise NotImplementedError(
             "This datatype must implement a 'parse' method.")
 
-    def parse_scalar(self, value, config={}, pos=None):
+    def parse_scalar(self, value, config=None, pos=None):
         """
         Parse a single scalar of the underlying type of the converter.
         For non-array converters, this is equivalent to parse.  For
@@ -203,7 +303,10 @@ class Char(Converter):
     """
     default = _empty_bytes
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
+        if config is None:
+            config = {}
+
         Converter.__init__(self, field, config, pos)
 
         if field.arraysize is None:
@@ -232,19 +335,22 @@ class Char(Converter):
         else:
             self.parse = self._str_parse
 
-    def _ascii_parse(self, value, config={}, pos=None):
+    def supports_empty_values(self, config):
+        return True
+
+    def _ascii_parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
             vo_warn(W46, ('char', self.arraysize), config, pos)
         return value.encode('ascii'), False
 
-    def _str_parse(self, value, config={}, pos=None):
+    def _str_parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
             vo_warn(W46, ('char', self.arraysize), config, pos)
         return value.encode('utf-8'), False
 
     def output(self, value, mask):
         if mask:
-            return u''
+            return ''
         if not isinstance(value, str):
             value = value.decode('ascii')
         return xml_escape_cdata(value)
@@ -261,7 +367,7 @@ class Char(Converter):
         return s, False
 
     def _binoutput_var(self, value, mask):
-        if mask or value is None or value == u'':
+        if mask or value is None or value == '':
             return _zero_int
         return self._write_length(len(value)) + value
 
@@ -277,9 +383,9 @@ class UnicodeChar(Converter):
 
     Missing values are not handled for string or unicode types.
     """
-    default = u''
+    default = ''
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         Converter.__init__(self, field, config, pos)
 
         if field.arraysize is None:
@@ -301,15 +407,15 @@ class UnicodeChar(Converter):
             self.binoutput = self._binoutput_fixed
             self._struct_format = ">%ds" % (self.arraysize * 2)
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
             vo_warn(W46, ('unicodeChar', self.arraysize), config, pos)
         return value, False
 
     def output(self, value, mask):
         if mask:
-            return u''
-        return xml_escape_cdata(unicode(value))
+            return ''
+        return xml_escape_cdata(six.text_type(value))
 
     def _binparse_var(self, read):
         length = self._parse_length(read)
@@ -331,7 +437,7 @@ class UnicodeChar(Converter):
 
     def _binoutput_fixed(self, value, mask):
         if mask:
-            value = u''
+            value = ''
         return struct_pack(self._struct_format, value.encode('utf_16_be'))
 
 
@@ -339,23 +445,24 @@ class Array(Converter):
     """
     Handles both fixed and variable-lengths arrays.
     """
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
+        if config is None:
+            config = {}
         Converter.__init__(self, field, config, pos)
-
         if config.get('pedantic'):
             self._splitter = self._splitter_pedantic
         else:
             self._splitter = self._splitter_lax
 
-    def parse_scalar(self, value, config={}, pos=0):
+    def parse_scalar(self, value, config=None, pos=0):
         return self._base.parse_scalar(value, config, pos)
 
     @staticmethod
-    def _splitter_pedantic(value, config={}, pos=None):
+    def _splitter_pedantic(value, config=None, pos=None):
         return pedantic_array_splitter.split(value)
 
     @staticmethod
-    def _splitter_lax(value, config={}, pos=None):
+    def _splitter_lax(value, config=None, pos=None):
         if ',' in value:
             vo_warn(W01, (), config, pos)
         return array_splitter.split(value)
@@ -367,7 +474,7 @@ class VarArray(Array):
     """
     format = 'O'
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         Array.__init__(self, field, config)
 
         self._base = base
@@ -376,7 +483,7 @@ class VarArray(Array):
     def output(self, value, mask):
         output = self._base.output
         result = [output(x, m) for x, m in np.broadcast(value, value.mask)]
-        return u' '.join(result)
+        return ' '.join(result)
 
     def binparse(self, read):
         length = self._parse_length(read)
@@ -408,7 +515,7 @@ class ArrayVarArray(VarArray):
     Handles an array of variable-length arrays, i.e. where *arraysize*
     ends in '*'.
     """
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
             return ma.array([]), False
 
@@ -431,7 +538,7 @@ class ScalarVarArray(VarArray):
     """
     Handles a variable-length array of numeric scalars.
     """
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
             return ma.array([]), False
 
@@ -454,7 +561,7 @@ class NumericArray(Array):
     """
     vararray_type = ArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         Array.__init__(self, field, config, pos)
 
         self._base = base
@@ -468,11 +575,14 @@ class NumericArray(Array):
         self._memsize = np.dtype(self.format).itemsize
         self._bigendian_format = '>' + self.format
 
-        self.default = (
-            np.ones(arraysize, dtype=self._base.format) *
-            self._base.default)
+        self.default = np.empty(arraysize, dtype=self._base.format)
+        self.default[...] = self._base.default
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
+        if config is None:
+            config = {}
+        if config['version_1_3_or_later'] and value == '':
+            return np.zeros(self._arraysize, dtype=self._base.format), True
         parts = self._splitter(value, config, pos)
         if len(parts) != self._items:
             warn_or_raise(E02, E02, (self._items, len(parts)), config, pos)
@@ -488,7 +598,7 @@ class NumericArray(Array):
                          ([self._base.default] * (self._items - len(parts))))
             return self.parse_parts(parts, config, pos)
 
-    def parse_parts(self, parts, config={}, pos=None):
+    def parse_parts(self, parts, config=None, pos=None):
         base_parse = self._base.parse
         result = []
         result_mask = []
@@ -506,7 +616,7 @@ class NumericArray(Array):
         base_output = self._base.output
         value = np.asarray(value)
         mask = np.asarray(mask)
-        return u' '.join(base_output(x, m) for x, m in
+        return ' '.join(base_output(x, m) for x, m in
                          zip(value.flat, mask.flat))
 
     def binparse(self, read):
@@ -517,8 +627,7 @@ class NumericArray(Array):
 
     def binoutput(self, value, mask):
         filtered = self._base.filter_array(value, mask)
-        if filtered.dtype.byteorder != '>':
-            filtered = filtered.byteswap()
+        filtered = _ensure_bigendian(filtered)
         return filtered.tostring()
 
 
@@ -530,7 +639,7 @@ class Numeric(Converter):
     vararray_type = ScalarVarArray
     null = None
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         Converter.__init__(self, field, config, pos)
 
         self._memsize = np.dtype(self.format).itemsize
@@ -557,31 +666,34 @@ class FloatingPoint(Numeric):
     """
     default = np.nan
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
+        if config is None:
+            config = {}
+
         Numeric.__init__(self, field, config, pos)
 
         precision = field.precision
         width = field.width
-        format_parts = [u'%']
+        format_parts = ['%']
 
         if width is not None:
-            format_parts.append(unicode(width))
+            format_parts.append(six.text_type(width))
 
         if precision is None:
-            format_parts.append(u's')
+            format_parts.append('s')
         elif precision.startswith("E"):
-            format_parts.append(u'.%dg' % int(precision[1:]))
+            format_parts.append('.%dg' % int(precision[1:]))
         elif precision.startswith("F"):
-            format_parts.append(u'.%df' % int(precision[1:]))
+            format_parts.append('.%df' % int(precision[1:]))
         else:
-            format_parts.append(u'.%df' % int(precision))
+            format_parts.append('.%df' % int(precision))
 
-        self._output_format = u''.join(format_parts)
+        self._output_format = ''.join(format_parts)
 
         self.nan = np.array(np.nan, self.format)
 
         if self.null is None:
-            self._null_output = u'NaN'
+            self._null_output = 'NaN'
             self._null_binoutput = self.binoutput(self.nan, False)
             self.filter_array = self._filter_nan
         else:
@@ -594,13 +706,16 @@ class FloatingPoint(Numeric):
         else:
             self.parse = self._parse_permissive
 
-    def _parse_pedantic(self, value, config={}, pos=None):
+    def supports_empty_values(self, config):
+        return True
+
+    def _parse_pedantic(self, value, config=None, pos=None):
         if value.strip() == '':
             return self.null, True
         f = float(value)
         return f, self.is_null(f)
 
-    def _parse_permissive(self, value, config={}, pos=None):
+    def _parse_permissive(self, value, config=None, pos=None):
         try:
             f = float(value)
             return f, self.is_null(f)
@@ -620,16 +735,16 @@ class FloatingPoint(Numeric):
             return self._null_output
         if np.isfinite(value):
             result = self._output_format % value
-            if (self._output_format[-1] == u's' and
-                result.endswith(u'.0')):
+            if (self._output_format[-1] == 's' and
+                result.endswith('.0')):
                 result = result[:-2]
             return result
         elif np.isnan(value):
-            return u'NaN'
+            return 'NaN'
         elif np.isposinf(value):
-            return u'+InF'
+            return '+InF'
         elif np.isneginf(value):
-            return u'-InF'
+            return '-InF'
         # Should never raise
         vo_raise("Invalid floating point value '%s'" % value)
 
@@ -637,8 +752,7 @@ class FloatingPoint(Numeric):
         if mask:
             return self._null_binoutput
 
-        if value.dtype.byteorder != '>':
-            value = value.byteswap()
+        value = _ensure_bigendian(value)
         return value.tostring()
 
     def _filter_nan(self, value, mask):
@@ -669,15 +783,20 @@ class Integer(Numeric):
     """
     default = 0
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         Numeric.__init__(self, field, config, pos)
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
+        if config is None:
+            config = {}
         mask = False
-        if isinstance(value, basestring):
+        if isinstance(value, six.string_types):
             value = value.lower()
             if value == '':
-                warn_or_raise(W49, W49, (), config, pos)
+                if config['version_1_3_or_later']:
+                    mask = True
+                else:
+                    warn_or_raise(W49, W49, (), config, pos)
                 if self.null is not None:
                     value = self.null
                 else:
@@ -711,9 +830,9 @@ class Integer(Numeric):
         if mask:
             if self.null is None:
                 warn_or_raise(W31, W31)
-                return u'NaN'
-            return unicode(self.null)
-        return unicode(value)
+                return 'NaN'
+            return six.text_type(self.null)
+        return six.text_type(value)
 
     def binoutput(self, value, mask):
         if mask:
@@ -721,8 +840,8 @@ class Integer(Numeric):
                 vo_raise(W31)
             else:
                 value = self.null
-        if value.dtype.byteorder != '>':
-            value = value.byteswap()
+
+        value = _ensure_bigendian(value)
         return value.tostring()
 
     def filter_array(self, value, mask):
@@ -774,12 +893,9 @@ class ComplexArrayVarArray(VarArray):
     """
     Handles an array of variable-length arrays of complex numbers.
     """
-    def __init__(self, field, base, arraysize, config={}, pos=None):
-        VarArray.__init__(self, field, base, arraysize, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
-            return ma.array([]), False
+            return ma.array([]), True
 
         parts = self._splitter(value, config, pos)
         items = self._base._items
@@ -800,12 +916,9 @@ class ComplexVarArray(VarArray):
     """
     Handles a variable-length array of complex numbers.
     """
-    def __init__(self, field, base, arraysize, config={}, pos=None):
-        VarArray.__init__(self, field, base, arraysize, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
-            return ma.array([]), False
+            return ma.array([]), True
 
         parts = self._splitter(value, config, pos)
         parse_parts = self._base.parse_parts
@@ -827,17 +940,17 @@ class ComplexArray(NumericArray):
     """
     vararray_type = ComplexArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         NumericArray.__init__(self, field, base, arraysize, config, pos)
         self._items *= 2
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         parts = self._splitter(value, config, pos)
         if parts == ['']:
             parts = []
         return self.parse_parts(parts, config, pos)
 
-    def parse_parts(self, parts, config={}, pos=None):
+    def parse_parts(self, parts, config=None, pos=None):
         if len(parts) != self._items:
             vo_raise(E02, (self._items, len(parts)), config, pos)
         base_parse = self._base.parse_parts
@@ -863,12 +976,13 @@ class Complex(FloatingPoint, Array):
     vararray_type = ComplexVarArray
     default = np.nan
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         FloatingPoint.__init__(self, field, config, pos)
         Array.__init__(self, field, config, pos)
 
-    def parse(self, value, config={}, pos=None):
-        if value.strip() == '':
+    def parse(self, value, config=None, pos=None):
+        stripped = value.strip()
+        if stripped == '' or stripped.lower() == 'nan':
             return np.nan, True
         splitter = self._splitter
         parts = [float(x) for x in splitter(value, config, pos)]
@@ -878,24 +992,24 @@ class Complex(FloatingPoint, Array):
     _parse_permissive = parse
     _parse_pedantic = parse
 
-    def parse_parts(self, parts, config={}, pos=None):
+    def parse_parts(self, parts, config=None, pos=None):
         value = complex(*parts)
         return value, self.is_null(value)
 
     def output(self, value, mask):
         if mask:
             if self.null is None:
-                return u'NaN'
+                return 'NaN'
             else:
                 value = self.null
         real = self._output_format % value.real
         imag = self._output_format % value.imag
-        if self._output_format[-1] == u's':
-            if real.endswith(u'.0'):
+        if self._output_format[-1] == 's':
+            if real.endswith('.0'):
                 real = real[:-2]
-            if imag.endswith(u'.0'):
+            if imag.endswith('.0'):
                 imag = imag[:-2]
-        return real + u' ' + imag
+        return real + ' ' + imag
 
 
 class FloatComplex(Complex):
@@ -920,42 +1034,32 @@ class BitArray(NumericArray):
     """
     vararray_type = ArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         NumericArray.__init__(self, field, base, arraysize, config, pos)
 
         self._bytes = ((self._items - 1) // 8) + 1
 
     @staticmethod
-    def _splitter_pedantic(value, config={}, pos=None):
+    def _splitter_pedantic(value, config=None, pos=None):
         return list(re.sub('\s', '', value))
 
     @staticmethod
-    def _splitter_lax(value, config={}, pos=None):
+    def _splitter_lax(value, config=None, pos=None):
         if ',' in value:
             vo_warn(W01, (), config, pos)
         return list(re.sub('\s|,', '', value))
 
     def output(self, value, mask):
+        if np.any(mask):
+            vo_warn(W39)
         value = np.asarray(value)
-        mapping = {False: u'0', True: u'1'}
-        return u''.join(mapping[x] for x in value.flat)
+        mapping = {False: '0', True: '1'}
+        return ''.join(mapping[x] for x in value.flat)
 
     def binparse(self, read):
         data = read(self._bytes)
-        results = []
-        for byte in data:
-            if not IS_PY3K:
-                byte = ord(byte)
-            for bit_no in range(7, -1, -1):
-                bit = byte & (1 << bit_no)
-                bit = (bit != 0)
-                results.append(bit)
-                if len(results) == self._items:
-                    break
-            if len(results) == self._items:
-                break
-
-        result = np.array(results, dtype='b1').reshape(self._arraysize)
+        result = bitarray_to_bool(data, self._items)
+        result = result.reshape(self._arraysize)
         result_mask = np.zeros(self._arraysize, dtype='b1')
         return result, result_mask
 
@@ -963,25 +1067,7 @@ class BitArray(NumericArray):
         if np.any(mask):
             vo_warn(W39)
 
-        value = value.flat
-        bit_no = 7
-        byte = 0
-        bytes = []
-        for v in value:
-            if v:
-                byte |= 1 << bit_no
-            if bit_no == 0:
-                bytes.append(byte)
-                bit_no = 7
-                byte = 0
-            else:
-                bit_no -= 1
-        if bit_no != 7:
-            bytes.append(byte)
-
-        assert len(bytes) == self._bytes
-
-        return struct_pack("%sB" % len(bytes), *bytes)
+        return bool_to_bitarray(value)
 
 
 class Bit(Converter):
@@ -995,13 +1081,13 @@ class Bit(Converter):
     binary_one = b'\x08'
     binary_zero = b'\0'
 
-    def __init__(self, field, config={}, pos=None):
-        Converter.__init__(self, field, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
+        if config is None:
+            config = {}
         mapping = {'1': True, '0': False}
         if value is False or value.strip() == '':
-            warn_or_raise(W49, W49, (), config, pos)
+            if not config['version_1_3_or_later']:
+                warn_or_raise(W49, W49, (), config, pos)
             return False, True
         else:
             try:
@@ -1014,9 +1100,9 @@ class Bit(Converter):
             vo_warn(W39)
 
         if value:
-            return u'1'
+            return '1'
         else:
-            return u'0'
+            return '0'
 
     def binparse(self, read):
         data = read(1)
@@ -1037,16 +1123,13 @@ class BooleanArray(NumericArray):
     """
     vararray_type = ArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
-        NumericArray.__init__(self, field, base, arraysize, config, pos)
-
     def binparse(self, read):
         data = read(self._items)
         binparse = self._base.binparse_value
         result = []
         result_mask = []
         for char in data:
-            if not IS_PY3K:
+            if not six.PY3:
                 char = ord(char)
             value, mask = binparse(char)
             result.append(value)
@@ -1078,10 +1161,9 @@ class Boolean(Converter):
     binary_true = b'T'
     binary_false = b'F'
 
-    def __init__(self, field, config={}, pos=None):
-        Converter.__init__(self, field, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
+        if value == '':
+            return False, True
         if value is False:
             return False, True
         mapping = {'TRUE'  : (True, False),
@@ -1101,10 +1183,10 @@ class Boolean(Converter):
 
     def output(self, value, mask):
         if mask:
-            return u'?'
+            return '?'
         if value:
-            return u'T'
-        return u'F'
+            return 'T'
+        return 'F'
 
     def binparse(self, read):
         value = ord(read(1))
@@ -1150,7 +1232,7 @@ converter_mapping = {
     'unicodeChar'   : UnicodeChar }
 
 
-def get_converter(field, config={}, pos=None):
+def get_converter(field, config=None, pos=None):
     """
     Get an appropriate converter instance for a given field.
 
@@ -1168,6 +1250,9 @@ def get_converter(field, config={}, pos=None):
     -------
     converter : astropy.io.votable.converters.Converter
     """
+    if config is None:
+        config = {}
+
     if field.datatype not in converter_mapping:
         vo_raise(E06, (field.datatype, field.ID), config)
 
@@ -1222,11 +1307,7 @@ numpy_dtype_to_field_mapping = {
 }
 
 
-# numpy 1.4.1 doesn't have a "bytes_" type
-if hasattr(np, 'bytes_'):
-    numpy_dtype_to_field_mapping[np.bytes_().dtype.num] = 'char'
-else:
-    numpy_dtype_to_field_mapping[np.str_().dtype.num] = 'char'
+numpy_dtype_to_field_mapping[np.bytes_().dtype.num] = 'char'
 
 
 def _all_bytes(column):
@@ -1238,7 +1319,7 @@ def _all_bytes(column):
 
 def _all_unicode(column):
     for x in column:
-        if not isinstance(x, unicode):
+        if not isinstance(x, six.text_type):
             return False
     return True
 
@@ -1335,7 +1416,7 @@ def table_column_to_votable_datatype(column):
         if isinstance(column[0], bytes):
             if _all_bytes(column[1:]):
                 return {'datatype': 'char', 'arraysize': '*'}
-        elif isinstance(column[0], unicode):
+        elif isinstance(column[0], six.text_type):
             if _all_unicode(column[1:]):
                 return {'datatype': 'unicodeChar', 'arraysize': '*'}
         elif isinstance(column[0], np.ndarray):
