@@ -1,20 +1,39 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from ..extern import six
+from ..extern.six import text_type
+from ..extern.six.moves import zip as izip
+from ..extern.six.moves import xrange
+
 import os
 import sys
-from itertools import izip
+import inspect
+
+import numpy as np
 
 from .. import log
 from ..utils.console import Getch, color_print
 from ..config import ConfigurationItem
 
-_format_funcs = {None: lambda format_, val: str(val)}
+
+if six.PY3:
+    def default_format_func(format_, val):
+        if isinstance(val, bytes):
+            return val.decode('utf-8')
+        else:
+            return str(val)
+    _format_funcs = {None: default_format_func}
+else:
+    _format_funcs = {None: lambda format_, val: text_type(val)}
 
 MAX_LINES = ConfigurationItem('max_lines', 25, 'Maximum number of lines for '
-    'the pretty-printer to use if it cannot determine the terminal size. '
-    'Negative numbers mean no limit.')
+                              'the pretty-printer to use if it cannot determine the terminal size. '
+                              'Negative numbers mean no limit.')
 MAX_WIDTH = ConfigurationItem('max_width', 80, 'Maximum number of characters '
-    'for the pretty-printer to use per line if it cannot determine the '
-    'terminal size.  Negative numbers mean no limit.')
+                              'for the pretty-printer to use per line if it cannot determine the '
+                              'terminal size.  Negative numbers mean no limit.')
+
 
 def _get_pprint_size(max_lines=None, max_width=None):
     """Get the output size (number of lines and character width) for Column and
@@ -61,14 +80,14 @@ def _get_pprint_size(max_lines=None, max_width=None):
     if max_lines is None:
         max_lines = lines
     elif max_lines < 0:
-        max_lines = sys.maxint
+        max_lines = sys.maxsize
     if max_lines < 6:
         max_lines = 6
 
     if max_width is None:
         max_width = width
     elif max_width < 0:
-        max_width = sys.maxint
+        max_width = sys.maxsize
     if max_width < 10:
         max_width = 10
 
@@ -77,34 +96,46 @@ def _get_pprint_size(max_lines=None, max_width=None):
 
 def _auto_format_func(format_, val):
     """Format ``val`` according to ``format_`` for both old- and new-
-    style format specifications.  More importantly, determine and cache
-    (in _format_funcs) a function that will do this subsequently.  In
-    this way this complicated logic is only done for the first value.
+    style format specifications or using a user supplied function.
+    More importantly, determine and cache (in _format_funcs) a function
+    that will do this subsequently.  In this way this complicated logic is
+    only done for the first value.
 
     Returns the formatted value.
     """
-    try:
-        # Convert val to Python object with tolist().  See
-        # https://github.com/astropy/astropy/issues/148#issuecomment-3930809
-        out = format_.format(val.tolist())
-        # Require that the format statement actually did something
-        if out == format_:
-            raise ValueError
-        format_func = lambda format_, val: format_.format(val.tolist())
-    except:  # Not sure what exceptions might be raised
+    if inspect.isfunction(format_):
+        format_func = lambda format_, val: format_(val.tolist())
         try:
-            out = format_ % val
+            out = format_func(format_, val)
+            if not isinstance(out, six.string_types):
+                raise ValueError('Format function for value {0} returned {1} instead of string type'
+                                 .format(val, type(val)))
+        except Exception as err:
+            raise ValueError('Format function for value {0} failed: {1}'
+                             .format(val, err))
+    else:
+        try:
+            # Convert val to Python object with tolist().  See
+            # https://github.com/astropy/astropy/issues/148#issuecomment-3930809
+            out = format_.format(val.tolist())
+            # Require that the format statement actually did something
             if out == format_:
                 raise ValueError
-            format_func = lambda format_, val: format_ % val
-        except:
-            raise ValueError('Unable to parse format string {0}'
-                             .format(format_))
+            format_func = lambda format_, val: format_.format(val.tolist())
+        except:  # Not sure what exceptions might be raised
+            try:
+                out = format_ % val
+                if out == format_:
+                    raise ValueError
+                format_func = lambda format_, val: format_ % val
+            except:
+                raise ValueError('Unable to parse format string {0}'
+                                 .format(format_))
     _format_funcs[format_] = format_func
     return out
 
 
-def _pformat_col(col, max_lines=None, show_name=True, show_units=False):
+def _pformat_col(col, max_lines=None, show_name=True, show_unit=False):
     """Return a list of formatted string representation of column values.
 
     Parameters
@@ -115,8 +146,8 @@ def _pformat_col(col, max_lines=None, show_name=True, show_units=False):
     show_name : bool
         Include column name (default=True)
 
-    show_units : bool
-        Include a header row for units (default=False)
+    show_unit : bool
+        Include a header row for unit (default=False)
 
     Returns
     -------
@@ -128,7 +159,7 @@ def _pformat_col(col, max_lines=None, show_name=True, show_units=False):
 
     """
     outs = {}  # Some values from _pformat_col_iter iterator that are needed here
-    col_strs = list(_pformat_col_iter(col, max_lines, show_name, show_units, outs))
+    col_strs = list(_pformat_col_iter(col, max_lines, show_name, show_unit, outs))
     col_width = max(len(x) for x in col_strs)
 
     # Center line content and generate dashed headerline
@@ -144,7 +175,7 @@ def _pformat_col(col, max_lines=None, show_name=True, show_units=False):
     return col_strs, outs['n_header']
 
 
-def _pformat_col_iter(col, max_lines, show_name, show_units, outs):
+def _pformat_col_iter(col, max_lines, show_name, show_unit, outs):
     """Iterator which yields formatted string representation of column values.
 
     Parameters
@@ -155,8 +186,8 @@ def _pformat_col_iter(col, max_lines, show_name, show_units, outs):
     show_name : bool
         Include column name (default=True)
 
-    show_units : bool
-        Include a header row for units (default=False)
+    show_unit : bool
+        Include a header row for unit (default=False)
 
     out : dict
         Must be a dict which is used to pass back additional values
@@ -168,6 +199,7 @@ def _pformat_col_iter(col, max_lines, show_name, show_units, outs):
     if multidims:
         multidim0 = tuple(0 for n in multidims)
         multidim1 = tuple(n - 1 for n in multidims)
+        trivial_multidims = np.prod(multidims) == 1
 
     col_strs = []  # List of formatted column values
     i_dashes = None
@@ -177,16 +209,16 @@ def _pformat_col_iter(col, max_lines, show_name, show_units, outs):
         i_centers.append(n_header)
         if multidims:
             col_name = col.name + ' [{0}]'.format(
-                ','.join(str(n) for n in multidims))
+                ','.join(six.text_type(n) for n in multidims))
         else:
             col_name = col.name
         n_header += 1
         yield col_name
-    if show_units:
+    if show_unit:
         i_centers.append(n_header)
         n_header += 1
-        yield str(col.units or '')
-    if show_units or show_name:
+        yield six.text_type(col.unit or '')
+    if show_unit or show_name:
         i_dashes = n_header
         n_header += 1
         yield '---'
@@ -207,9 +239,15 @@ def _pformat_col_iter(col, max_lines, show_name, show_units, outs):
     for i in xrange(n_rows):
         if i < i0 or i > i1:
             if multidims:
-                col_str = (format_func(col.format, col[(i,) + multidim0]) +
-                           ' .. ' +
-                           format_func(col.format, col[(i,) + multidim1]))
+                # Prevents colums like Column(data=[[(1,)],[(2,)]], name='a')
+                # with shape (n,1,...,1) from being printed as if there was
+                # more than one element in a row
+                if trivial_multidims:
+                    col_str = format_func(col.format, col[(i,) + multidim0])   
+                else:
+                    col_str = (format_func(col.format, col[(i,) + multidim0]) +
+                              ' .. ' +
+                              format_func(col.format, col[(i,) + multidim1]))
             else:
                 col_str = format_func(col.format, col[i])
             yield col_str
@@ -220,8 +258,9 @@ def _pformat_col_iter(col, max_lines, show_name, show_units, outs):
     outs['i_centers'] = i_centers
     outs['i_dashes'] = i_dashes
 
+
 def _pformat_table(table, max_lines=None, max_width=None, show_name=True,
-                   show_units=False, html=False):
+                   show_unit=False, html=False, tableid=None):
     """Return a list of lines for the formatted string representation of
     the table.
 
@@ -236,12 +275,17 @@ def _pformat_table(table, max_lines=None, max_width=None, show_name=True,
     show_name : bool
         Include a header row for column names (default=True)
 
-    show_units : bool
-        Include a header row for units (default=False)
+    show_unit : bool
+        Include a header row for unit (default=False)
 
     html : bool
         Format the output as an HTML table (default=False)
 
+    tableid : str or None
+        An ID tag for the table; only used if html is set.  Default is
+        "table{id}", where id is the unique integer id of the table object,
+        id(table)
+
     Returns
     -------
     out : str
@@ -254,9 +298,9 @@ def _pformat_table(table, max_lines=None, max_width=None, show_name=True,
     # use and to determine the width
     max_lines, max_width = _get_pprint_size(max_lines, max_width)
     cols = []
-    for col in table.columns.values():
+    for col in six.itervalues(table.columns):
         lines, n_header = _pformat_col(col, max_lines, show_name,
-                                       show_units)
+                                       show_unit)
         cols.append(lines)
 
     if not cols:
@@ -283,7 +327,9 @@ def _pformat_table(table, max_lines=None, max_width=None, show_name=True,
     if html:
         from ..utils.xml.writer import xml_escape
 
-        rows.append('<table>')
+        if tableid is None:
+            tableid = 'table{id}'.format(id=id(table))
+        rows.append('<table id="{tid}">'.format(tid=tableid))
         for i in range(n_rows):
             # _pformat_col output has a header line '----' which is not needed here
             if i == n_header - 1:
@@ -292,6 +338,8 @@ def _pformat_table(table, max_lines=None, max_width=None, show_name=True,
             vals = ('<{0}>{1}</{2}>'.format(td, xml_escape(col[i].strip()), td)
                     for col in cols)
             row = ('<tr>' + ''.join(vals) + '</tr>')
+            if i < n_header:
+                row = ('<thead>' + row + '</thead>')
             rows.append(row)
         rows.append('</table>')
     else:
@@ -303,7 +351,7 @@ def _pformat_table(table, max_lines=None, max_width=None, show_name=True,
 
 
 def _more_tabcol(tabcol, max_lines=None, max_width=None, show_name=True,
-                show_units=False):
+                 show_unit=False):
     """Interactive "more" of a table or column.
 
     Parameters
@@ -317,8 +365,8 @@ def _more_tabcol(tabcol, max_lines=None, max_width=None, show_name=True,
     show_name : bool
         Include a header row for column names (default=True)
 
-    show_units : bool
-        Include a header row for units (default=False)
+    show_unit : bool
+        Include a header row for unit (default=False)
     """
     allowed_keys = 'f br<>qhpn'
 
@@ -326,13 +374,13 @@ def _more_tabcol(tabcol, max_lines=None, max_width=None, show_name=True,
     n_header = 0
     if show_name:
         n_header += 1
-    if show_units:
+    if show_unit:
         n_header += 1
-    if show_name or show_units:
+    if show_name or show_unit:
         n_header += 1
 
     # Set up kwargs for pformat call.  Only Table gets max_width.
-    kwargs = dict(max_lines=-1, show_name=show_name, show_units=show_units)
+    kwargs = dict(max_lines=-1, show_name=show_name, show_unit=show_unit)
     if hasattr(tabcol, 'columns'):  # tabcol is a table
         kwargs['max_width'] = max_width
 
@@ -340,7 +388,7 @@ def _more_tabcol(tabcol, max_lines=None, max_width=None, show_name=True,
     # This is because get_pprint_size leaves 6 extra lines so that in
     # ipython you normally see the last input line.
     max_lines1, max_width = _get_pprint_size(max_lines, max_width)
-    if max_lines == None:
+    if max_lines is None:
         max_lines1 += 2
     delta_lines = max_lines1 - n_header
 
@@ -362,20 +410,20 @@ def _more_tabcol(tabcol, max_lines=None, max_width=None, show_name=True,
             for color, line in izip(colors, lines):
                 color_print(line, color)
         showlines = True
-        print
-        print "-- f, <space>, b, r, p, n, <, >, q h (help) --",
+        print()
+        print("-- f, <space>, b, r, p, n, <, >, q h (help) --", end=' ')
         # Get a valid key
         while True:
             try:
                 key = inkey().lower()
             except:
-                print "\n"
+                print("\n")
                 log.error('Console does not support getting a character'
                           ' as required by more().  Use pprint() instead.')
                 return
             if key in allowed_keys:
                 break
-        print key
+        print(key)
 
         if key.lower() == 'q':
             break
@@ -395,7 +443,7 @@ def _more_tabcol(tabcol, max_lines=None, max_width=None, show_name=True,
             i0 += 1
         elif key == 'h':
             showlines = False
-            print """
+            print("""
 Browsing keys:
    f, <space> : forward one page
    b : back one page
@@ -405,9 +453,9 @@ Browsing keys:
    < : go to beginning
    > : go to end
    q : quit browsing
-   h : print this help""",
+   h : print this help""", end=' ')
         if i0 < 0:
             i0 = 0
         if i0 >= len(tabcol) - delta_lines:
             i0 = len(tabcol) - delta_lines
-        print "\n"
+        print("\n")
