@@ -5,15 +5,11 @@ import numpy as np
 from ...tests.helper import pytest
 from ... import table
 from ...table import pprint
+from ...extern.six import PY3
 
 BIG_WIDE_ARR = np.arange(2000, dtype=np.float).reshape(100, 20)
 SMALL_ARR = np.arange(12, dtype=np.int).reshape(4, 3)
 
-numpy_lt_1p5 = version.LooseVersion(np.__version__) < version.LooseVersion('1.5')
-
-# Dummy init of Table for pyflakes and to be sure test fixture is working
-Table = None
-
 
 class MaskedTable(table.Table):
     def __init__(self, *args, **kwargs):
@@ -21,17 +17,22 @@ class MaskedTable(table.Table):
         table.Table.__init__(self, *args, **kwargs)
 
 
-# Fixture to run all tests for both an unmasked (ndarray) and masked (MaskedArray) column.
-@pytest.fixture(params=[False] if numpy_lt_1p5 else [False, True])
-def set_global_Table(request):
-    global Table
-    Table = MaskedTable if request.param else table.Table
+# Fixture to run all tests for both an unmasked (ndarray) and masked
+# (MaskedArray) column.
+@pytest.fixture(params=[False, True])
+def table_type(request):
+    # return MaskedTable if request.param else table.Table
+    try:
+        request.param
+        return MaskedTable
+    except AttributeError:
+        return table.Table
 
 
-@pytest.mark.usefixtures('set_global_Table')
+@pytest.mark.usefixtures('table_type')
 class TestMultiD():
 
-    def test_multidim(self):
+    def test_multidim(self, table_type):
         """Test printing with multidimensional column"""
         arr = [np.array([[1, 2],
                          [10, 20]]),
@@ -39,67 +40,105 @@ class TestMultiD():
                          [30, 40]]),
                np.array([[5, 6],
                          [50, 60]])]
-        t = Table(arr)
+        t = table_type(arr)
         lines = t.pformat()
-        print lines
+        print(lines)
         assert lines == ['col0 [2] col1 [2] col2 [2]',
                          '-------- -------- --------',
                          '  1 .. 2   3 .. 4   5 .. 6',
                          '10 .. 20 30 .. 40 50 .. 60']
 
         lines = t.pformat(html=True)
-        assert lines == ['<table>',
-                         '<tr><th>col0 [2]</th><th>col1 [2]</th><th>col2 [2]</th></tr>',
+        assert lines == ['<table id="table{id}">'.format(id=id(t)),
+                         '<thead><tr><th>col0 [2]</th><th>col1 [2]</th><th>col2 [2]</th></tr></thead>',
                          '<tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td></tr>',
                          '<tr><td>10 .. 20</td><td>30 .. 40</td><td>50 .. 60</td></tr>',
                          '</table>']
-        assert t._repr_html_() == ('<table><tr><th>col0 [2]</th><th>col1 [2]</th><th>col2 [2]'
-                                   '</th></tr><tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td>'
+        assert t._repr_html_() == ('<table id="table{tid}"><thead><tr><th>col0 [2]</th><th>col1 [2]</th><th>col2 [2]'.format(tid=id(t)) +
+                                   '</th></tr></thead><tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td>'
                                    '</tr><tr><td>10 .. 20</td><td>30 .. 40</td><td>50 .. 60</td>'
                                    '</tr></table>')
 
-        t = Table([arr])
+        t = table_type([arr])
         lines = t.pformat()
-        print lines
+        print(lines)
         assert lines == ['col0 [2,2]',
                          '----------',
                          '   1 .. 20',
                          '   3 .. 40',
                          '   5 .. 60']
+                         
+    def test_fake_multidim(self, table_type):
+        """Test printing with 'fake' multidimensional column"""
+        arr = [np.array([[(1,)],
+                         [(10,)]]),
+               np.array([[(3,)],
+                         [(30,)]]),
+               np.array([[(5,)],
+                         [(50,)]])]
+        t = table_type(arr)
+        lines = t.pformat()
+        print(lines)
+        assert lines == ['col0 [1,1] col1 [1,1] col2 [1,1]',
+                         '---------- ---------- ----------',
+                         '         1          3          5',
+                         '        10         30         50']
+
+        lines = t.pformat(html=True)
+        assert lines == ['<table id="table{id}">'.format(id=id(t)),
+                         '<thead><tr><th>col0 [1,1]</th><th>col1 [1,1]</th><th>col2 [1,1]</th></tr></thead>',
+                         '<tr><td>1</td><td>3</td><td>5</td></tr>',
+                         '<tr><td>10</td><td>30</td><td>50</td></tr>',
+                         '</table>']
+        assert t._repr_html_() == ('<table id="table{tid}"><thead><tr><th>col0 [1,1]</th><th>col1 [1,1]</th><th>col2'.format(tid=id(t)) +
+                                   ' [1,1]</th></tr></thead><tr><td>1</td><td>3</td><td>5</td>'
+                                   '</tr><tr><td>10</td><td>30</td><td>50</td></tr></table>')
+
+        t = table_type([arr])
+        lines = t.pformat()
+        print(lines)
+        assert lines == ['col0 [2,1,1]',
+                         '------------',
+                         '     1 .. 10',
+                         '     3 .. 30',
+                         '     5 .. 50']
 
 
 def test_html_escaping():
     t = table.Table([('<script>alert("gotcha");</script>', 2, 3)])
     assert t._repr_html_() == (
-        '<table><tr><th>col0</th></tr><tr>'
-        '<td>&lt;script&gt;alert(&quot;gotcha&quot;);&lt;/script&gt;</td>'
+        '<table id="table{id}"><thead><tr><th>col0</th></tr></thead>'.format(id=id(t)) +
+        '<tr><td>&lt;script&gt;alert(&quot;gotcha&quot;);&lt;/script&gt;</td>'
         '</tr><tr><td>2</td></tr><tr><td>3</td></tr></table>')
 
 
-@pytest.mark.usefixtures('set_global_Table')
+@pytest.mark.usefixtures('table_type')
 class TestPprint():
 
-    def setup_method(self, method):
-        self.tb = Table(BIG_WIDE_ARR)
+    def _setup(self, table_type):
+        self.tb = table_type(BIG_WIDE_ARR)
         self.tb['col0'].format = '%e'
         self.tb['col1'].format = '%.6f'
-        self.tb['col0'].units = 'km**2'
-        self.tb['col19'].units = 'kg s m**-2'
-        self.ts = Table(SMALL_ARR)
 
-    def test_format0(self):
+        self.tb['col0'].unit = 'km**2'
+        self.tb['col19'].unit = 'kg s m**-2'
+        self.ts = table_type(SMALL_ARR)
+
+    def test_format0(self, table_type):
         """Try getting screen size but fail to defaults because testing doesn't
         have access to screen (fcntl.ioctl fails).
         """
+        self._setup(table_type)
         arr = np.arange(4000, dtype=np.float).reshape(100, 40)
-        lines = Table(arr).pformat()
+        lines = table_type(arr).pformat()
         assert len(lines) == pprint.MAX_LINES()
         for line in lines:
             assert (len(line) > pprint.MAX_WIDTH() - 10 and
                     len(line) <= pprint.MAX_WIDTH())
 
-    def test_format1(self):
+    def test_format1(self, table_type):
         """Basic test of formatting"""
+        self._setup(table_type)
         lines = self.tb.pformat(max_lines=8, max_width=40)
         assert lines == ['    col0         col1    ... col19 ',
                          '------------ ----------- ... ------',
@@ -110,21 +149,24 @@ class TestPprint():
                          '1.960000e+03 1961.000000 ... 1979.0',
                          '1.980000e+03 1981.000000 ... 1999.0']
 
-    def test_format2(self):
-        """Include the units header row"""
-        lines = self.tb.pformat(max_lines=8, max_width=40, show_units=True)
-        print(lines)
-        assert lines == ['    col0         col1    ...    col19   ',
-                         '    km2                  ... kg s / (m2)',
-                         '------------ ----------- ... -----------',
-                         '0.000000e+00    1.000000 ...        19.0',
-                         '2.000000e+01   21.000000 ...        39.0',
-                         '         ...         ... ...         ...',
-                         '1.960000e+03 1961.000000 ...      1979.0',
-                         '1.980000e+03 1981.000000 ...      1999.0']
+    def test_format2(self, table_type):
+        """Include the unit header row"""
+        self._setup(table_type)
+        lines = self.tb.pformat(max_lines=8, max_width=40, show_unit=True)
 
-    def test_format3(self):
+        print(lines)
+        assert lines == ['    col0         col1    ...   col19  ',
+                         '    km2                  ... kg s / m2',
+                         '------------ ----------- ... ---------',
+                         '0.000000e+00    1.000000 ...      19.0',
+                         '2.000000e+01   21.000000 ...      39.0',
+                         '         ...         ... ...       ...',
+                         '1.960000e+03 1961.000000 ...    1979.0',
+                         '1.980000e+03 1981.000000 ...    1999.0']
+
+    def test_format3(self, table_type):
         """Do not include the name header row"""
+        self._setup(table_type)
         lines = self.tb.pformat(max_lines=8, max_width=40, show_name=False)
         assert lines == ['0.000000e+00    1.000000 ...   19.0',
                          '2.000000e+01   21.000000 ...   39.0',
@@ -135,8 +177,9 @@ class TestPprint():
                          '1.960000e+03 1961.000000 ... 1979.0',
                          '1.980000e+03 1981.000000 ... 1999.0']
 
-    def test_noclip(self):
+    def test_noclip(self, table_type):
         """Basic table print"""
+        self._setup(table_type)
         lines = self.ts.pformat(max_lines=-1, max_width=-1)
         assert lines == ['col0 col1 col2',
                          '---- ---- ----',
@@ -145,9 +188,10 @@ class TestPprint():
                          '   6    7    8',
                          '   9   10   11']
 
-    def test_clip1(self):
+    def test_clip1(self, table_type):
         """max lines below hard limit of 6
         """
+        self._setup(table_type)
         lines = self.ts.pformat(max_lines=3, max_width=-1)
         assert lines == ['col0 col1 col2',
                          '---- ---- ----',
@@ -156,10 +200,11 @@ class TestPprint():
                          '   6    7    8',
                          '   9   10   11']
 
-    def test_clip2(self):
+    def test_clip2(self, table_type):
         """max lines below hard limit of 6 and output longer than 6
         """
-        lines = self.ts.pformat(max_lines=3, max_width=-1, show_units=True)
+        self._setup(table_type)
+        lines = self.ts.pformat(max_lines=3, max_width=-1, show_unit=True)
         assert lines == ['col0 col1 col2',
                          '              ',
                          '---- ---- ----',
@@ -167,11 +212,12 @@ class TestPprint():
                          ' ...  ...  ...',
                          '   9   10   11']
 
-    def test_clip3(self):
+    def test_clip3(self, table_type):
         """Max lines below hard limit of 6 and max width below hard limit
         of 10
         """
-        lines = self.ts.pformat(max_lines=3, max_width=1, show_units=True)
+        self._setup(table_type)
+        lines = self.ts.pformat(max_lines=3, max_width=1, show_unit=True)
         assert lines == ['col0 ...',
                          '     ...',
                          '---- ...',
@@ -179,18 +225,19 @@ class TestPprint():
                          ' ... ...',
                          '   9 ...']
 
-    def test_clip4(self):
+    def test_clip4(self, table_type):
         """Test a range of max_lines"""
+        self._setup(table_type)
         for max_lines in range(130):
             lines = self.tb.pformat(max_lines=max_lines)
             assert len(lines) == max(6, min(102, max_lines))
 
 
-@pytest.mark.usefixtures('set_global_Table')
+@pytest.mark.usefixtures('table_type')
 class TestFormat():
 
-    def test_column_format(self):
-        t = Table([[1, 2], [3, 4]], names=('a', 'b'))
+    def test_column_format(self, table_type):
+        t = table_type([[1, 2], [3, 4]], names=('a', 'b'))
         # default (format=None)
         assert str(t['a']) == ' a \n---\n  1\n  2'
 
@@ -215,13 +262,66 @@ class TestFormat():
         with pytest.raises(ValueError):
             str(t['a'])
 
-    def test_column_format_with_threshold(self):
+    def test_column_format_with_threshold(self, table_type):
         MAX_LINES_val = pprint.MAX_LINES()
         pprint.MAX_LINES.set(6)
-        t = Table([np.arange(20)], names=['a'])
+        t = table_type([np.arange(20)], names=['a'])
         t['a'].format = '%{0:}'
         assert str(t['a']) == ' a \n---\n %0\n %1\n...\n%19'
         t['a'].format = '{ %4.2f }'
         assert str(t['a']) == '    a    \n---------\n { 0.00 }\n' \
                               ' { 1.00 }\n      ...\n{ 19.00 }'
         pprint.MAX_LINES.set(MAX_LINES_val)
+
+    def test_column_format_func(self, table_type):
+        # run most of functions twice
+        # 1) astropy.table.pprint._format_funcs gets populated
+        # 2) astropy.table.pprint._format_funcs gets used
+
+        t = table_type([[1., 2.], [3, 4]], names=('a', 'b'))
+
+        # mathematical function
+        t['a'].format = lambda x: str(x * 3.)
+        assert str(t['a']) == ' a \n---\n3.0\n6.0'
+        assert str(t['a']) == ' a \n---\n3.0\n6.0'
+
+    def test_column_format_func_wrong_number_args(self, table_type):
+        t = table_type([[1., 2.], [3, 4]], names=('a', 'b'))
+
+        # function that expects wrong number of arguments
+        def func(a, b):
+            pass
+
+        t['a'].format = func
+        with pytest.raises(ValueError):
+            str(t['a'])
+
+    def test_column_format_func_multiD(self, table_type):
+        arr = [np.array([[1, 2],
+                         [10, 20]])]
+        t = table_type(arr, names=['a'])
+
+        # mathematical function
+        t['a'].format = lambda x: str(x * 3.)
+        outstr = '   a [2]    \n------------\n  3.0 .. 6.0\n30.0 .. 60.0'
+        assert str(t['a']) == outstr
+        assert str(t['a']) == outstr
+
+    def test_column_format_func_not_str(self, table_type):
+        t = table_type([[1., 2.], [3, 4]], names=('a', 'b'))
+
+        # mathematical function
+        t['a'].format = lambda x: x * 3
+        with pytest.raises(ValueError):
+            str(t['a'])
+
+
+def test_pprint_py3_bytes():
+    """
+    Test for #1346.  Make sure a bytestring (dtype=S<N>) in Python 3 is printed
+    correctly (without the "b" prefix like b'string').
+    """
+    val = bytes('val', encoding='utf-8') if PY3 else 'val'
+    dat = np.array([(val,)], dtype=[('col', 'S3')])
+    t = table.Table(dat)
+    assert t['col'].pformat() == ['col', '---', 'val']
