@@ -3,7 +3,8 @@
 """
 Utilities for console input and output.
 """
-from __future__ import division, print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
 import codecs
 import locale
@@ -23,30 +24,37 @@ try:
 except ImportError:
     _CAN_RESIZE_TERMINAL = False
 
-import numpy as np
-
 try:
     get_ipython()
 except NameError:
     OutStream = None
+    IPythonIOStream = None
     stdio = sys
 else:
     try:
         from IPython.zmq.iostream import OutStream
         from IPython.utils import io
+        # On Windows in particular this is necessary, as the io.stdout stream
+        # in IPython gets hooked up to some pyreadline magic to handle colors
         stdio = io
+        IPythonIOStream = io.IOStream
     except ImportError:
         OutStream = None
+        IPythonIOStream = None
         stdio = sys
 
 try:
-    import IPython
+    import IPython  # pylint: disable=W0611
     # This is just to set a flag that IPython is installed at all
     _HAVE_IPYTHON = True
 except ImportError:
     _HAVE_IPYTHON = False
 
 from ..config import ConfigurationItem
+from ..extern import six
+from ..extern.six.moves import range
+
+from .misc import deprecated, isiterable
 
 
 __all__ = [
@@ -58,9 +66,7 @@ __all__ = [
 USE_COLOR = ConfigurationItem(
     'use_color', sys.platform != 'win32' or _HAVE_IPYTHON,
     'When True, use ANSI color escape sequences when writing to the console.')
-USE_UNICODE = ConfigurationItem(
-    'use_unicode', True,
-    'Use Unicode characters when drawing progress bars etc. at the console.')
+from ..import UNICODE_OUTPUT
 
 
 IS_PY3 = sys.version_info[0] == 3
@@ -134,7 +140,7 @@ def _color_text(text, color):
         return text
 
     color_code = color_mapping.get(color, '0;39')
-    return u'\033[{0}m{1}\033[0m'.format(color_code, text)
+    return '\033[{0}m{1}\033[0m'.format(color_code, text)
 
 
 def _decode_preferred_encoding(s):
@@ -162,6 +168,14 @@ def _write_with_fallback(s, write, fileobj):
     'latin-1'.
     """
 
+    if IPythonIOStream is not None and isinstance(fileobj, IPythonIOStream):
+        # If the output stream is an IPython.utils.io.IOStream object that's
+        # not going to be very helpful to us since it doesn't raise any
+        # exceptions when an error occurs writing to its underlying stream.
+        # There's no advantage to us using IOStream.write directly though;
+        # instead just write directly to its underlying stream:
+        write = fileobj.stream.write
+
     try:
         write(s)
         return write
@@ -202,7 +216,7 @@ def color_print(*args, **kwargs):
 
     Parameters
     ----------
-    positional args : strings
+    positional args : str
         The positional arguments come in pairs (*msg*, *color*), where
         *msg* is the string to display and *color* is the color to
         display it in.
@@ -224,11 +238,11 @@ def color_print(*args, **kwargs):
 
     file = kwargs.get('file', stdio.stdout)
 
-    end = kwargs.get('end', u'\n')
+    end = kwargs.get('end', '\n')
 
     write = file.write
     if isatty(file) and USE_COLOR():
-        for i in xrange(0, len(args), 2):
+        for i in range(0, len(args), 2):
             msg = args[i]
             if i + 1 == len(args):
                 color = ''
@@ -248,7 +262,7 @@ def color_print(*args, **kwargs):
 
         write(end)
     else:
-        for i in xrange(0, len(args), 2):
+        for i in range(0, len(args), 2):
             msg = args[i]
             if not IS_PY3 and isinstance(msg, bytes):
                 # Support decoding bytes to unicode on Python 2; use the
@@ -293,26 +307,26 @@ def human_time(seconds):
         that is always exactly 6 characters.
     """
     units = [
-        (u'y', 60 * 60 * 24 * 7 * 52),
-        (u'w', 60 * 60 * 24 * 7),
-        (u'd', 60 * 60 * 24),
-        (u'h', 60 * 60),
-        (u'm', 60),
-        (u's', 1),
+        ('y', 60 * 60 * 24 * 7 * 52),
+        ('w', 60 * 60 * 24 * 7),
+        ('d', 60 * 60 * 24),
+        ('h', 60 * 60),
+        ('m', 60),
+        ('s', 1),
     ]
 
     seconds = int(seconds)
 
     if seconds < 60:
-        return u'   {0:02d}s'.format(seconds)
+        return '   {0:02d}s'.format(seconds)
     for i in xrange(len(units) - 1):
         unit1, limit1 = units[i]
         unit2, limit2 = units[i + 1]
         if seconds >= limit1:
-            return u'{0:02d}{1}{2:02d}{3}'.format(
+            return '{0:02d}{1}{2:02d}{3}'.format(
                 seconds // limit1, unit1,
                 (seconds % limit1) // limit2, unit2)
-    return u'  ~inf'
+    return '  ~inf'
 
 
 def human_file_size(size):
@@ -337,7 +351,7 @@ def human_file_size(size):
     size : str
         A human-friendly representation of the size of the file
     """
-    suffixes = u' kMGTPEH'
+    suffixes = ' kMGTPEH'
     if size == 0:
         num_scale = 0
     else:
@@ -356,22 +370,28 @@ def human_file_size(size):
     return "{0:>3s}{1}".format(str_value, suffix)
 
 
-class ProgressBar(object):
+class ProgressBar(six.Iterator):
     """
     A class to display a progress bar in the terminal.
 
-    It is designed to be used with the `with` statement::
+    It is designed to be used either with the `with` statement::
 
         with ProgressBar(len(items)) as bar:
             for item in enumerate(items):
                 bar.update()
+
+    or as a generator::
+
+        for item in ProgressBar(items):
+            item.process()
     """
-    def __init__(self, total, file=None):
+    def __init__(self, total_or_items, file=None):
         """
         Parameters
         ----------
-        total : int
-            The number of increments in the process being tracked.
+        total_or_items : int or sequence
+            If an int, the number of increments in the process being
+            tracked.  If a sequence, the items to iterate over.
 
         file : writable file-like object, optional
             The file to write the progress bar to.  Defaults to
@@ -388,7 +408,17 @@ class ProgressBar(object):
         else:
             self._silent = False
 
-        self._total = total
+        if isiterable(total_or_items):
+            self._items = iter(total_or_items)
+            self._total = len(total_or_items)
+        else:
+            try:
+                self._total = int(total_or_items)
+            except TypeError:
+                raise TypeError("First argument must be int or sequence")
+            else:
+                self._items = iter(range(self._total))
+
         self._file = file
         self._start_time = time.time()
 
@@ -406,6 +436,9 @@ class ProgressBar(object):
 
     def _handle_resize(self, signum=None, frame=None):
         if self._should_handle_resize:
+            # Don't import numpy at module level since it may not be
+            # available in all contexts that this module is imported
+            import numpy as np
             data = fcntl.ioctl(self._file, termios.TIOCGWINSZ, '\0' * 8)
             arr = np.fromstring(data, dtype=np.int16)
             terminal_width = arr[1]
@@ -428,6 +461,20 @@ class ProgressBar(object):
             if self._signal_set:
                 signal.signal(signal.SIGWINCH, signal.SIG_DFL)
 
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        try:
+            rv = next(self._items)
+        except StopIteration:
+            self.__exit__(None, None, None)
+            raise
+        else:
+            self.update()
+            return rv
+
+
     def update(self, value=None):
         """
         Update the progress bar to the given value (out of the total
@@ -446,26 +493,26 @@ class ProgressBar(object):
         write = file.write
 
         bar_fill = int(float(self._bar_length) * frac)
-        write(u'\r|')
-        color_print(u'=' * bar_fill, 'blue', file=file, end=u'')
+        write('\r|')
+        color_print('=' * bar_fill, 'blue', file=file, end='')
         if bar_fill < self._bar_length:
-            color_print(u'>', 'green', file=file, end=u'')
-            write(u'-' * (self._bar_length - bar_fill - 1))
-        write(u'|')
+            color_print('>', 'green', file=file, end='')
+            write('-' * (self._bar_length - bar_fill - 1))
+        write('|')
 
         if value >= self._total:
             t = time.time() - self._start_time
-            prefix = u'     '
+            prefix = '     '
         elif value <= 0:
             t = None
-            prefix = u''
+            prefix = ''
         else:
             t = ((time.time() - self._start_time) * (1.0 - frac)) / frac
-            prefix = u' ETA '
-        write(u' {0:>4s}/{1:>4s}'.format(
+            prefix = ' ETA '
+        write(' {0:>4s}/{1:>4s}'.format(
             human_file_size(value),
             self._human_total))
-        write(u' ({0:>6s}%)'.format(u'{0:.2f}'.format(frac * 100.0)))
+        write(' ({0:>6s}%)'.format('{0:.2f}'.format(frac * 100.0)))
         write(prefix)
         if t is not None:
             write(human_time(t))
@@ -520,7 +567,6 @@ class ProgressBar(object):
                     if (i % steps) == 0:
                         bar.update(i)
             else:
-                import multiprocessing
                 p = multiprocessing.Pool()
                 for i, result in enumerate(
                     p.imap_unordered(function, items, steps)):
@@ -531,6 +577,7 @@ class ProgressBar(object):
 
         return results
 
+    @deprecated('0.3', alternative='ProgressBar')
     @classmethod
     def iterate(cls, items, file=None):
         """
@@ -558,17 +605,12 @@ class ProgressBar(object):
         generator :
             A generator over `items`
         """
-
         if file is None:
             file = stdio.stdout
-
-        with cls(len(items), file=file) as bar:
-            for item in items:
-                yield item
-                bar.update()
+        return cls(items, file=file)
 
 
-class Spinner():
+class Spinner(object):
     """
     A class to display a spinner in the terminal.
 
@@ -578,8 +620,8 @@ class Spinner():
             for item in enumerate(items):
                 s.next()
     """
-    _default_unicode_chars = u"◓◑◒◐"
-    _default_ascii_chars = u"-/|\\"
+    _default_unicode_chars = "◓◑◒◐"
+    _default_ascii_chars = "-/|\\"
 
     def __init__(self, msg, color='default', file=None, step=1,
                  chars=None):
@@ -615,7 +657,7 @@ class Spinner():
         self._file = file
         self._step = step
         if chars is None:
-            if USE_UNICODE:
+            if UNICODE_OUTPUT():
                 chars = self._default_unicode_chars
             else:
                 chars = self._default_ascii_chars
@@ -629,21 +671,30 @@ class Spinner():
         file = self._file
         write = file.write
         flush = file.flush
+        try_fallback = True
 
         while True:
-            write(u'\r')
-            color_print(self._msg, self._color, file=file, end=u'')
-            write(u' ')
-            write(chars[index])
+            write('\r')
+            color_print(self._msg, self._color, file=file, end='')
+            write(' ')
+            try:
+                if try_fallback:
+                    write = _write_with_fallback(chars[index], write, file)
+                else:
+                    write(chars[index])
+            except UnicodeError:
+                # If even _write_with_fallback failed for any reason just give
+                # up on trying to use the unicode characters
+                chars = self._default_ascii_chars
+                write(chars[index])
+                try_fallback = False  # No good will come of using this again
             flush()
             yield
 
-            for i in xrange(self._step):
+            for i in range(self._step):
                 yield
 
-            index += 1
-            if index == len(chars):
-                index = 0
+            index = (index + 1) % len(chars)
 
     def __enter__(self):
         if self._silent:
@@ -657,23 +708,23 @@ class Spinner():
         flush = file.flush
 
         if not self._silent:
-            write(u'\r')
-            color_print(self._msg, self._color, file=file, end=u'')
+            write('\r')
+            color_print(self._msg, self._color, file=file, end='')
         if exc_type is None:
-            color_print(u' [Done]', 'green', file=file)
+            color_print(' [Done]', 'green', file=file)
         else:
-            color_print(u' [Failed]', 'red', file=file)
+            color_print(' [Failed]', 'red', file=file)
         flush()
 
     def _silent_iterator(self):
-        color_print(self._msg, self._color, file=self._file, end=u'')
+        color_print(self._msg, self._color, file=self._file, end='')
         self._file.flush()
 
         while True:
             yield
 
 
-class ProgressBarOrSpinner:
+class ProgressBarOrSpinner(object):
     """
     A class that displays either a `ProgressBar` or `Spinner`
     depending on whether the total size of the operation is
@@ -743,13 +794,13 @@ class ProgressBarOrSpinner:
         given to the constructor.
         """
         if self._is_spinner:
-            self._iter.next()
+            next(self._iter)
         else:
             self._obj.update(value)
 
 
 def print_code_line(line, col=None, file=None, tabwidth=8, width=70):
-    u"""
+    """
     Prints a line of source code, highlighting a particular character
     position in the line.  Useful for displaying the context of error
     messages.
@@ -789,35 +840,40 @@ def print_code_line(line, col=None, file=None, tabwidth=8, width=70):
     if file is None:
         file = stdio.stdout
 
+    if UNICODE_OUTPUT():
+        ellipsis = '…'
+    else:
+        ellipsis = '...'
+
     write = file.write
 
     if col is not None:
         assert col < len(line)
-        ntabs = line[:col].count(u'\t')
+        ntabs = line[:col].count('\t')
         col += ntabs * (tabwidth - 1)
 
-    line = line.rstrip(u'\n')
-    line = line.replace(u'\t', u' ' * tabwidth)
+    line = line.rstrip('\n')
+    line = line.replace('\t', ' ' * tabwidth)
 
     if col is not None and col > width:
         new_col = min(width // 2, len(line) - col)
         offset = col - new_col
-        line = line[offset + 1:]
+        line = line[offset + len(ellipsis):]
+        width -= len(ellipsis)
         new_col = col
         col -= offset
-        width = width - 3
-        color_print(u'…', 'darkgrey', file=file, end=u'')
+        color_print(ellipsis, 'darkgrey', file=file, end='')
 
     if len(line) > width:
-        write(line[:width - 1])
-        color_print(u'…', 'darkgrey', file=file)
+        write(line[:width - len(ellipsis)])
+        color_print(ellipsis, 'darkgrey', file=file)
     else:
         write(line)
-        write(u'\n')
+        write('\n')
 
     if col is not None:
-        write(u' ' * col)
-        color_print(u'^', 'red', file=file)
+        write(' ' * col)
+        color_print('^', 'red', file=file)
 
 
 # The following four Getch* classes implement unbuffered character reading from
@@ -876,7 +932,7 @@ class _GetchWindows(object):
         return msvcrt.getch()
 
 
-class _GetchMacCarbon:
+class _GetchMacCarbon(object):
     """
     A function which returns the current ASCII key that is down;
     if no ASCII key is down, the null string is returned.  The
