@@ -1,18 +1,32 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
 """ This module contains helper functions for accessing, downloading, and
 caching data files.
 """
 
-from __future__ import division
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from ..extern import six
+from ..extern.six.moves import urllib
 
-import os
-import io
-import sys
 import atexit
 import contextlib
-import urllib2
+import fnmatch
+import hashlib
+import os
+import io
+import shutil
+import socket
+import sys
+import time
+
+from tempfile import NamedTemporaryFile, gettempdir
+from warnings import warn
 
 from ..config.configuration import ConfigurationItem
+from ..utils.exceptions import AstropyWarning
+
 
 __all__ = ['get_readable_fileobj', 'get_file_contents', 'get_pkg_data_fileobj',
            'get_pkg_data_filename', 'get_pkg_data_contents',
@@ -21,6 +35,7 @@ __all__ = ['get_readable_fileobj', 'get_file_contents', 'get_pkg_data_fileobj',
            'get_free_space_in_dir', 'check_free_space_in_dir', 'download_file',
            'download_files_in_parallel']
 
+
 DATAURL = ConfigurationItem(
     'dataurl', 'http://data.astropy.org/', 'URL for astropy remote data site.')
 REMOTE_TIMEOUT = ConfigurationItem(
@@ -43,7 +58,7 @@ DELETE_TEMPORARY_DOWNLOADS_AT_EXIT = ConfigurationItem(
 PY3K = sys.version_info[0] >= 3
 
 
-class CacheMissingWarning(Warning):
+class CacheMissingWarning(AstropyWarning):
     """
     This warning indicates the standard cache directory is not accessible, with
     the first argument providing the warning message. If args[1] is present, it
@@ -61,8 +76,7 @@ def _is_url(string):
     string : str
         The string to test
     """
-    from urlparse import urlparse
-    url = urlparse(string)
+    url = urllib.parse.urlparse(string)
     # url[0]==url.scheme, but url[0] is py 2.6-compat
     # we can't just check that url[0] is not an empty string, because
     # file paths in windows would return a non-empty scheme (e.g. e:\\
@@ -80,7 +94,8 @@ def _is_inside(path, parent_path):
 
 
 @contextlib.contextmanager
-def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
+def get_readable_fileobj(name_or_obj, encoding=None, cache=False,
+                         show_progress=True, remote_timeout=None):
     """
     Given a filename or a readable file-like object, return a context
     manager that yields a readable file-like object.
@@ -122,9 +137,19 @@ def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
 
     cache : bool, optional
         Whether to cache the contents of remote URLs.
-    """
 
-    import tempfile
+    show_progress : bool, optional
+        Whether to display a progress bar if the file is downloaded
+        from a remote server.  Default is `True`.
+
+    remote_timeout : float
+        Timeout for remote requests in seconds (default is the configurable
+        REMOTE_TIMEOUT, which is 3s by default)
+
+    Returns
+    -------
+    file : readable file-like object
+    """
 
     # close_fds is a list of file handles created by this function
     # that need to be closed.  We don't want to always just close the
@@ -135,10 +160,16 @@ def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
     close_fds = []
     delete_fds = []
 
+    if remote_timeout is None:
+        # use configfile default
+        remote_timeout = REMOTE_TIMEOUT()
+
     # Get a file object to the content
-    if isinstance(name_or_obj, basestring):
+    if isinstance(name_or_obj, six.string_types):
         if _is_url(name_or_obj):
-            name_or_obj = download_file(name_or_obj, cache=cache)
+            name_or_obj = download_file(
+                name_or_obj, cache=cache, show_progress=show_progress,
+                timeout=remote_timeout)
         if PY3K:
             fileobj = io.FileIO(name_or_obj, 'r')
         else:
@@ -178,7 +209,7 @@ def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
         try:
             # bz2.BZ2File does not support file objects, only filenames, so we
             # need to write the data to a temporary file
-            tmp = tempfile.NamedTemporaryFile("wb", delete=False)
+            tmp = NamedTemporaryFile("wb", delete=False)
             tmp.write(fileobj.read())
             tmp.close()
             delete_fds.append(tmp)
@@ -210,7 +241,7 @@ def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
         # return a handle to that.
         import bz2
         if isinstance(fileobj, bz2.BZ2File):
-            tmp = tempfile.NamedTemporaryFile("wb", delete=False)
+            tmp = NamedTemporaryFile("wb", delete=False)
             data = fileobj.read()
             tmp.write(data)
             tmp.close()
@@ -313,9 +344,11 @@ def get_pkg_data_fileobj(data_name, encoding=None, cache=True):
 
     cache : bool
         If True, the file will be downloaded and saved locally or the
-        already-cached local copy will be accessed. If False, the file-like
-        object will directly access the resource (e.g. if a remote URL is
-        accessed, an object like that from `urllib2.urlopen` is returned).
+        already-cached local copy will be accessed. If False, the
+        file-like object will directly access the resource (e.g. if a
+        remote URL is accessed, an object like that from
+        `urllib2.urlopen` on Python 2 or `urllib.request.urlopen` on
+        Python 3 is returned).
 
     Returns
     -------
@@ -326,7 +359,7 @@ def get_pkg_data_fileobj(data_name, encoding=None, cache=True):
 
     Raises
     ------
-    urllib2.URLError
+    urllib2.URLError, urllib.error.URLError
         If a remote file cannot be found.
     IOError
         If problems occur writing or reading a local file.
@@ -373,7 +406,7 @@ def get_pkg_data_fileobj(data_name, encoding=None, cache=True):
                                     cache=cache)
 
 
-def get_pkg_data_filename(data_name):
+def get_pkg_data_filename(data_name, show_progress=True, remote_timeout=None):
     """
     Retrieves a data file from the standard locations for the package and
     provides a local filename for the data.
@@ -403,9 +436,17 @@ def get_pkg_data_filename(data_name):
               will first be searched for locally, and if not found,
               the Astropy data server will be queried.
 
+    show_progress : bool, optional
+        Whether to display a progress bar if the file is downloaded
+        from a remote server.  Default is `True`.
+
+    timeout : float
+        Timeout for the requests in seconds (default is the configurable
+        REMOTE_TIMEOUT, which is 3s by default)
+
     Raises
     ------
-    urllib2.URLError
+    urllib2.URLError, urllib.error.URLError
         If a remote file cannot be found.
     IOError
         If problems occur writing or reading a local file.
@@ -446,11 +487,18 @@ def get_pkg_data_filename(data_name):
 
     data_name = os.path.normpath(data_name)
 
+    if remote_timeout is None:
+        # use configfile default
+        remote_timeout = REMOTE_TIMEOUT()
+
     if data_name.startswith('hash/'):
         # first try looking for a local version if a hash is specified
         hashfn = _find_hash_fn(data_name[5:])
         if hashfn is None:
-            return download_file(DATAURL() + data_name, cache=True)
+            return download_file(
+                DATAURL() + data_name, cache=True,
+                show_progress=show_progress,
+                timeout=remote_timeout)
         else:
             return hashfn
     else:
@@ -461,7 +509,10 @@ def get_pkg_data_filename(data_name):
         elif os.path.isfile(datafn):  # local file
             return datafn
         else:  # remote file
-            return download_file(DATAURL() + data_name, cache=True)
+            return download_file(
+                DATAURL() + data_name, cache=True,
+                show_progress=show_progress,
+                timeout=remote_timeout)
 
 
 def get_pkg_data_contents(data_name, encoding=None, cache=True):
@@ -507,9 +558,11 @@ def get_pkg_data_contents(data_name, encoding=None, cache=True):
 
     cache : bool
         If True, the file will be downloaded and saved locally or the
-        already-cached local copy will be accessed. If False, the file-like
-        object will directly access the resource (e.g. if a remote URL is
-        accessed, an object like that from `urllib2.urlopen` is returned).
+        already-cached local copy will be accessed. If False, the
+        file-like object will directly access the resource (e.g. if a
+        remote URL is accessed, an object like that from
+        `urllib2.urlopen` on Python 2 or `urllib.request.urlopen` on
+        Python 3 is returned).
 
     Returns
     -------
@@ -518,7 +571,7 @@ def get_pkg_data_contents(data_name, encoding=None, cache=True):
 
     Raises
     ------
-    urllib2.URLError
+    urllib2.URLError, urllib.error.URLError
         If a remote file cannot be found.
     IOError
         If problems occur writing or reading a local file.
@@ -571,19 +624,16 @@ def get_pkg_data_filenames(datadir, pattern='*'):
             with open(fn) as f:
                 fcontents = f.read()
     """
-    import fnmatch
-    from os.path import isdir, isfile, join
-    from os import listdir
 
     path = _find_pkg_data_path(datadir)
-    if isfile(path):
+    if os.path.isfile(path):
         raise IOError(
             "Tried to access a data directory that's actually "
             "a package data file")
-    elif isdir(path):
-        for filename in listdir(path):
+    elif os.path.isdir(path):
+        for filename in os.listdir(path):
             if fnmatch.fnmatch(filename, pattern):
-                yield join(path, filename)
+                yield os.path.join(path, filename)
     else:
         raise IOError("Path not found")
 
@@ -670,7 +720,6 @@ def compute_hash(localfn):
         The hex digest of the MD5 hash for the contents of the `localfn` file.
 
     """
-    import hashlib
 
     with open(localfn, 'rb') as f:
         h = hashlib.md5()
@@ -687,7 +736,7 @@ def _find_pkg_data_path(data_name):
     Look for data in the source-included data directories and return the
     path.
     """
-    from os.path import dirname, join
+
     from ..utils.misc import find_current_module
 
     module = find_current_module(1, True)
@@ -709,10 +758,10 @@ def _find_pkg_data_path(data_name):
 
     rootpkg = __import__(rootpkgname)
 
-    module_path = dirname(module.__file__)
-    path = join(module_path, data_name)
+    module_path = os.path.dirname(module.__file__)
+    path = os.path.join(module_path, data_name)
 
-    root_dir = dirname(rootpkg.__file__)
+    root_dir = os.path.dirname(rootpkg.__file__)
     assert _is_inside(path, root_dir), \
            ("attempted to get a local data file outside "
             "of the " + rootpkgname + " tree")
@@ -725,8 +774,6 @@ def _find_hash_fn(hash):
     Looks for a local file by hash - returns file name if found and a valid
     file, otherwise returns None.
     """
-    from os.path import isfile, join
-    from warnings import warn
 
     try:
         dldir, urlmapfn = _get_download_cache_locs()
@@ -734,8 +781,8 @@ def _find_hash_fn(hash):
         msg = 'Could not access cache directory to search for data file: '
         warn(CacheMissingWarning(msg + str(e)))
         return None
-    hashfn = join(dldir, hash)
-    if isfile(hashfn):
+    hashfn = os.path.join(dldir, hash)
+    if os.path.isfile(hashfn):
         return hashfn
     else:
         return None
@@ -757,8 +804,19 @@ def get_free_space_in_dir(path):
         The amount of free space on the partition that the directory
         is on.
     """
-    stat = os.statvfs(path)
-    return stat.f_bavail * stat.f_frsize
+
+    if sys.platform.startswith('win'):
+        import ctypes
+        free_bytes = ctypes.c_ulonglong(0)
+        retval = ctypes.windll.kernel32.GetDiskFreeSpaceExW(
+                ctypes.c_wchar_p(path), None, None, ctypes.pointer(free_bytes))
+        if retval == 0:
+            raise IOError('Checking free space on %r failed unexpectedly.' %
+                          path)
+        return free_bytes.value
+    else:
+        stat = os.statvfs(path)
+        return stat.f_bavail * stat.f_frsize
 
 
 def check_free_space_in_dir(path, size):
@@ -788,7 +846,7 @@ def check_free_space_in_dir(path, size):
                 path, human_file_size(size)))
 
 
-def download_file(remote_url, cache=False):
+def download_file(remote_url, cache=False, show_progress=True, timeout=REMOTE_TIMEOUT()):
     """
     Accepts a URL, downloads and optionally caches the result
     returning the filename, with a name determined by the file's MD5
@@ -799,9 +857,14 @@ def download_file(remote_url, cache=False):
     ----------
     remote_url : str
         The URL of the file to download
+
     cache : bool, optional
         Whether to use the cache
 
+    show_progress : bool, optional
+        Whether to display a progress bar during the download (default
+        is `True`)
+
     Returns
     -------
     local_path : str
@@ -809,17 +872,10 @@ def download_file(remote_url, cache=False):
 
     Raises
     ------
-    `urllib2.URLError`
+    urllib2.URLError, urllib.error.URLError
         Whenever there's a problem getting the remote file.
     """
 
-    import hashlib
-    import socket
-    from contextlib import closing
-    from tempfile import NamedTemporaryFile, gettempdir
-    from shutil import move
-    from warnings import warn
-
     from ..utils.console import ProgressBarOrSpinner
 
     missing_cache = False
@@ -840,7 +896,8 @@ def download_file(remote_url, cache=False):
                 if str(remote_url) in url2hash:
                     return url2hash[str(remote_url)]
 
-        with closing(urllib2.urlopen(remote_url, timeout=REMOTE_TIMEOUT())) as remote:
+        with contextlib.closing(urllib.request.urlopen(
+                remote_url, timeout=timeout)) as remote:
             #keep a hash to rename the local file to the hashed name
             hash = hashlib.md5()
 
@@ -858,10 +915,15 @@ def download_file(remote_url, cache=False):
                 if cache:
                     check_free_space_in_dir(dldir, size)
 
+            if show_progress:
+                progress_stream = sys.stdout
+            else:
+                progress_stream = io.StringIO()
+
             dlmsg = "Downloading {0}".format(remote_url)
-            with ProgressBarOrSpinner(size, dlmsg) as p:
-                try:
-                    with NamedTemporaryFile(delete=False) as f:
+            with ProgressBarOrSpinner(size, dlmsg, file=progress_stream) as p:
+                with NamedTemporaryFile(delete=False) as f:
+                    try:
                         bytes_read = 0
                         block = remote.read(DOWNLOAD_CACHE_BLOCK_SIZE())
                         while block:
@@ -870,10 +932,10 @@ def download_file(remote_url, cache=False):
                             bytes_read += len(block)
                             p.update(bytes_read)
                             block = remote.read(DOWNLOAD_CACHE_BLOCK_SIZE())
-                except:
-                    if os.path.exists(f.name):
-                        os.remove(f.name)
-                    raise
+                    except:
+                        if os.path.exists(f.name):
+                            os.remove(f.name)
+                        raise
 
         if cache:
             _acquire_download_cache_lock()
@@ -885,7 +947,7 @@ def download_file(remote_url, cache=False):
                     if str(remote_url) in url2hash:
                         return url2hash[str(remote_url)]
                     local_path = os.path.join(dldir, hash.hexdigest())
-                    move(f.name, local_path)
+                    shutil.move(f.name, local_path)
                     url2hash[str(remote_url)] = local_path
             finally:
                 _release_download_cache_lock()
@@ -898,7 +960,7 @@ def download_file(remote_url, cache=False):
             if DELETE_TEMPORARY_DOWNLOADS_AT_EXIT():
                 global _tempfilestodel
                 _tempfilestodel.append(local_path)
-    except urllib2.URLError as e:
+    except urllib.error.URLError as e:
         if hasattr(e, 'reason') and hasattr(e.reason, 'errno') and e.reason.errno == 8:
             e.reason.strerror = e.reason.strerror + '. requested URL: ' + remote_url
             e.reason.args = (e.reason.errno, e.reason.strerror)
@@ -908,21 +970,17 @@ def download_file(remote_url, cache=False):
         # through.  It's supposed to be caught in `urrlib2` and raised in this
         # way, but for some reason in mysterious circumstances it doesn't. So
         # we'll just re-raise it here instead
-        raise urllib2.URLError(e)
+        raise urllib.error.URLError(e)
 
     return local_path
 
 
 def _do_download_files_in_parallel(args):
-    orig_stdout = sys.stdout
-    sys.stdout = io.BytesIO()
-    try:
-        return download_file(*args)
-    finally:
-        sys.stdout = orig_stdout
+    return download_file(*args, show_progress=False)
 
 
-def download_files_in_parallel(urls, cache=False):
+def download_files_in_parallel(urls, cache=False, show_progress=True,
+                               timeout=REMOTE_TIMEOUT()):
     """
     Downloads multiple files in parallel from the given URLs.  Blocks until
     all files have downloaded.  The result is a list of local file paths
@@ -936,6 +994,14 @@ def download_files_in_parallel(urls, cache=False):
     cache : bool, optional
         Whether to use the cache
 
+    show_progress : bool, optional
+        Whether to display a progress bar during the download (default
+        is `True`)
+
+    timeout : float
+        Timeout for the requests in seconds (default is the configurable
+        REMOTE_TIMEOUT, which is 3s by default)
+
     Returns
     -------
     paths : list of str
@@ -943,11 +1009,17 @@ def download_files_in_parallel(urls, cache=False):
     """
     from .console import ProgressBar
 
+    if show_progress:
+        progress = sys.stdout
+    else:
+        progress = io.BytesIO()
+
     # Combine duplicate URLs
     combined_urls = list(set(urls))
     combined_paths = ProgressBar.map(
         _do_download_files_in_parallel,
         [(x, cache) for x in combined_urls],
+        file=progress,
         multiprocess=True)
     paths = []
     for url in urls:
@@ -987,10 +1059,6 @@ def clear_download_cache(hashorurl=None):
         If the requested filename is not present in the data directory.
 
     """
-    from os import unlink
-    from os.path import join, exists
-    from shutil import rmtree
-    from warnings import warn
 
     try:
         dldir, urlmapfn = _get_download_cache_locs()
@@ -1003,34 +1071,34 @@ def clear_download_cache(hashorurl=None):
     _acquire_download_cache_lock()
     try:
         if hashorurl is None:
-            if exists(dldir):
-                rmtree(dldir)
-            if exists(urlmapfn):
-                unlink(urlmapfn)
+            if os.path.exists(dldir):
+                shutil.rmtree(dldir)
+            if os.path.exists(urlmapfn):
+                os.unlink(urlmapfn)
         else:
             with _open_shelve(urlmapfn, True) as url2hash:
-                filepath = join(dldir, hashorurl)
+                filepath = os.path.join(dldir, hashorurl)
                 assert _is_inside(filepath, dldir), \
                        ("attempted to use clear_download_cache on a location" +
                         " that's not inside the data cache directory")
 
-                if exists(filepath):
-                    for k, v in url2hash.items():
+                if os.path.exists(filepath):
+                    for k, v in list(six.iteritems(url2hash)):
                         if v == filepath:
                             del url2hash[k]
-                    unlink(filepath)
+                    os.unlink(filepath)
 
                 elif hashorurl in url2hash:
                     filepath = url2hash[hashorurl]
                     del url2hash[hashorurl]
-                    unlink(filepath)
+                    os.unlink(filepath)
 
                 else:
                     msg = 'Could not find file or url {0}'
                     raise OSError(msg.format(hashorurl))
     finally:
         # the lock will be gone if rmtree was used above, but release otherwise
-        if exists(join(_get_download_cache_locs()[0], 'lock')):
+        if os.path.exists(os.path.join(_get_download_cache_locs()[0], 'lock')):
             _release_download_cache_lock()
 
 
@@ -1046,23 +1114,21 @@ def _get_download_cache_locs():
         The path to the shelve object that stores the cache info.
     """
     from ..config.paths import get_cache_dir
-    from os.path import exists, isdir, join
-    from os import mkdir
 
-    datadir = join(get_cache_dir(), 'download')
-    shelveloc = join(get_cache_dir(), 'download_urlmap')
+    datadir = os.path.join(get_cache_dir(), 'download')
+    shelveloc = os.path.join(get_cache_dir(), 'download_urlmap')
 
-    if not exists(datadir):
+    if not os.path.exists(datadir):
         try:
-            mkdir(datadir)
+            os.mkdir(datadir)
         except OSError as e:
-            if not exists(datadir):
+            if not os.path.exists(datadir):
                 raise
-    elif not isdir(datadir):
+    elif not os.path.isdir(datadir):
         msg = 'Data cache directory {0} is not a directory'
         raise IOError(msg.format(datadir))
 
-    if isdir(shelveloc):
+    if os.path.isdir(shelveloc):
         msg = 'Data cache shelve object location {0} is a directory'
         raise IOError(msg.format(shelveloc))
 
@@ -1078,15 +1144,14 @@ def _open_shelve(shelffn, withclosing=False):
             ...
     """
     import shelve
-    from contextlib import closing
 
-    if not PY3K:  # pragma: py3
+    if not PY3K:  # pragma: py2
         shelf = shelve.open(shelffn, protocol=2)
-    else:  # pragma: py2
+    else:  # pragma: py3
         shelf = shelve.open(shelffn + '.db', protocol=2)
 
     if withclosing:
-        return closing(shelf)
+        return contextlib.closing(shelf)
     else:
         return shelf
 
@@ -1098,20 +1163,17 @@ def _acquire_download_cache_lock():
     Uses the lock directory method.  This is good because `mkdir` is
     atomic at the system call level, so it's thread-safe.
     """
-    from os.path import join
-    from os import mkdir
-    from time import sleep
 
     lockdir = os.path.join(_get_download_cache_locs()[0], 'lock')
     for i in range(DOWNLOAD_CACHE_LOCK_ATTEMPTS()):
         try:
-            mkdir(lockdir)
+            os.mkdir(lockdir)
             #write the pid of this process for informational purposes
-            with open(join(lockdir, 'pid'), 'w') as f:
+            with open(os.path.join(lockdir, 'pid'), 'w') as f:
                 f.write(str(os.getpid()))
 
         except OSError:
-            sleep(1)
+            time.sleep(1)
         else:
             return
     msg = 'Unable to acquire lock for cache directory ({0} exists)'
@@ -1119,17 +1181,14 @@ def _acquire_download_cache_lock():
 
 
 def _release_download_cache_lock():
-    from os.path import join, exists, isdir
-    from os import rmdir
+    lockdir = os.path.join(_get_download_cache_locs()[0], 'lock')
 
-    lockdir = join(_get_download_cache_locs()[0], 'lock')
-
-    if isdir(lockdir):
+    if os.path.isdir(lockdir):
         #if the pid file is present, be sure to remove it
-        pidfn = join(lockdir, 'pid')
-        if exists(pidfn):
+        pidfn = os.path.join(lockdir, 'pid')
+        if os.path.exists(pidfn):
             os.remove(pidfn)
-        rmdir(lockdir)
+        os.rmdir(lockdir)
     else:
         msg = 'Error releasing lock. "{0}" either does not exist or is not ' +\
               'a directory.'
