@@ -22,7 +22,17 @@ a standard pipeline.
 
 from __future__ import division  # confidence high
 
-if not _ASTROPY_SETUP_:
+try:
+    # Not guaranteed available at setup time
     from .wcs import *
+except ImportError:
+    if not _ASTROPY_SETUP_:
+        raise
 
-    pass
+
+def get_include():
+    """
+    Get the path to astropy.wcs's C header files.
+    """
+    import os
+    return os.path.join(os.path.dirname(__file__), "include")
