@@ -6,7 +6,9 @@
 #define NO_IMPORT_ARRAY
 
 /* util.h must be imported first */
-#include "pyutil.h"
+#include "astropy_wcs/pyutil.h"
+
+#include "astropy_wcs/docstrings.h"
 
 #include "wcsfix.h"
 #include "wcsprintf.h"
@@ -159,7 +161,7 @@ wcsprm_fix_values(
 
   unsigned int naxis = (unsigned int)x->naxis;
 
-  value_fixer(x->cd, 4);
+  value_fixer(x->cd, naxis * naxis);
   value_fixer(x->cdelt, naxis);
   value_fixer(x->crder, naxis);
   value_fixer(x->crota, naxis);
@@ -203,6 +205,7 @@ wcsprm_python2c(
  * Exceptions                                                              *
  ***************************************************************************/
 
+PyObject* WcsExc_Wcs;
 PyObject* WcsExc_SingularMatrix;
 PyObject* WcsExc_InconsistentAxisTypes;
 PyObject* WcsExc_InvalidTransform;
@@ -219,8 +222,47 @@ PyObject* WcsExc_InvalidTabularParameters;
  */
 PyObject** wcs_errexc[14];
 
+static PyObject*
+_new_exception_with_doc(char *name, char *doc, PyObject *base)
+{
+#if ((PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION >= 7) || \
+     (PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION >= 2))
+  return PyErr_NewExceptionWithDoc(name, doc, base, NULL);
+#else
+  /* Python 2.6 and 3.1 don't have PyErr_NewExceptionWithDoc */
+  PyObject *dict;
+  PyObject *docobj;
+  int result;
+
+  dict = PyDict_New();
+  if (dict == NULL) {
+    return NULL;
+  }
+
+  if (doc != NULL) {
+    docobj = PyUnicode_FromString(doc);
+    if (docobj == NULL) {
+      Py_DECREF(dict);
+      return NULL;
+    }
+
+    result = PyDict_SetItemString(dict, "__doc__", docobj);
+    Py_DECREF(docobj);
+    if (result < 0) {
+      Py_DECREF(dict);
+      return NULL;
+    }
+
+    return PyErr_NewException(name, base, dict);
+  }
+#endif
+}
+
 #define DEFINE_EXCEPTION(exc) \
-  WcsExc_##exc = PyErr_NewException("astropy.wcs._wcs." #exc "Error", PyExc_ValueError, NULL); \
+  WcsExc_##exc = _new_exception_with_doc(                             \
+      "astropy.wcs._wcs." #exc "Error",                                 \
+      doc_##exc,                                                        \
+      WcsExc_Wcs);                                                      \
   if (WcsExc_##exc == NULL) \
     return 1; \
   PyModule_AddObject(m, #exc "Error", WcsExc_##exc); \
@@ -229,6 +271,15 @@ int
 _define_exceptions(
     PyObject* m) {
 
+  WcsExc_Wcs = _new_exception_with_doc(
+      "astropy.wcs._wcs.WcsError",
+      doc_WcsError,
+      PyExc_ValueError);
+  if (WcsExc_Wcs == NULL) {
+    return 1;
+  }
+  PyModule_AddObject(m, "WcsError", WcsExc_Wcs);
+
   DEFINE_EXCEPTION(SingularMatrix);
   DEFINE_EXCEPTION(InconsistentAxisTypes);
   DEFINE_EXCEPTION(InvalidTransform);
@@ -363,20 +414,29 @@ set_string(
 
   char*      buffer;
   Py_ssize_t len;
+  PyObject*  ascii_obj = NULL;
+  int        result = -1;
 
   if (check_delete(propname, value)) {
     return -1;
   }
 
-  #if PY3K
-  if (PyBytes_AsStringAndSize(value, &buffer, &len) == -1) {
-    return -1;
+  if (PyUnicode_Check(value)) {
+    ascii_obj = PyUnicode_AsASCIIString(value);
+    if (ascii_obj == NULL) {
+      goto end;
+    }
+    if (PyBytes_AsStringAndSize(ascii_obj, &buffer, &len) == -1) {
+      goto end;
+    }
+  } else if (PyBytes_Check(value)) {
+    if (PyBytes_AsStringAndSize(value, &buffer, &len) == -1) {
+      goto end;
+    }
+  } else {
+    PyErr_SetString(PyExc_TypeError, "value must be bytes or unicode");
+    goto end;
   }
-  #else
-  if (PyString_AsStringAndSize(value, &buffer, &len) == -1) {
-    return -1;
-  }
-  #endif
 
   if (len > maxlen) {
     PyErr_Format(
@@ -384,12 +444,17 @@ set_string(
         "'%s' must be less than %u characters",
         propname,
         (unsigned int)maxlen);
-    return -1;
+    goto end;
   }
 
   strncpy(dest, buffer, (size_t)maxlen);
 
-  return 0;
+  result = 0;
+
+ end:
+  Py_XDECREF(ascii_obj);
+
+  return result;
 }
 
 /* get_bool is inlined */
@@ -550,8 +615,6 @@ set_int_array(
 
 /* get_str_list is inlined */
 
-/* set_str_list is inlined */
-
 int
 set_str_list(
     const char* propname,
@@ -561,8 +624,7 @@ set_str_list(
     char (*dest)[72]) {
 
   PyObject*  str      = NULL;
-  char*      str_char = NULL;
-  Py_ssize_t str_len  = 0;
+  Py_ssize_t input_len;
   Py_ssize_t i        = 0;
 
   if (check_delete(propname, value)) {
@@ -600,38 +662,26 @@ set_str_list(
       return -1;
     }
 
-    #if PY3K
-    if (!PyBytes_CheckExact(str)) {
-    #else
-    if (!PyString_CheckExact(str)) {
-    #endif
+    if (!(PyBytes_CheckExact(str) || PyUnicode_CheckExact(str))) {
       PyErr_Format(
           PyExc_TypeError,
-          #if PY3K
-          "'%s' must be a sequence of bytes",
-          #else
-          "'%s' must be a sequence of strings",
-          #endif
+          "'%s' must be a sequence of bytes or strings",
           propname);
       Py_DECREF(str);
       return -1;
     }
 
-    #if PY3K
-    if (PyBytes_Size(str) > maxlen) {
-    #else
-    if (PyString_Size(str) > maxlen) {
-    #endif
+    input_len = PySequence_Size(str);
+    if (input_len > maxlen) {
       PyErr_Format(
           PyExc_TypeError,
-          #if PY3K
-          "Each bytes in '%s' must be less than %u characters",
-          #else
-          "Each string in '%s' must be less than %u characters",
-          #endif
+          "Each entry in '%s' must be less than %u characters",
           propname, (unsigned int)maxlen);
       Py_DECREF(str);
       return -1;
+    } else if (input_len == -1) {
+      Py_DECREF(str);
+      return -1;
     }
 
     Py_DECREF(str);
@@ -642,20 +692,15 @@ set_str_list(
     if (str == NULL) {
       /* Theoretically, something has gone really wrong here, since
          we've already verified the list. */
+      PyErr_Clear();
       PyErr_Format(
           PyExc_RuntimeError,
           "Input values have changed underneath us.  Something is seriously wrong.");
       return -1;
     }
 
-    /* We already know its a string of the correct length */
-    #if PY3K
-    if (PyBytes_AsStringAndSize(str, &str_char, &str_len)) {
-    #else
-    if (PyString_AsStringAndSize(str, &str_char, &str_len)) {
-    #endif
-      /* Theoretically, something has gone really wrong here, since
-         we've already verified the list. */
+    if (set_string(propname, str, dest[i], maxlen)) {
+      PyErr_Clear();
       PyErr_Format(
           PyExc_RuntimeError,
           "Input values have changed underneath us.  Something is seriously wrong.");
@@ -663,8 +708,6 @@ set_str_list(
       return -1;
     }
 
-    strncpy(dest[i], str_char, (size_t)maxlen);
-
     Py_DECREF(str);
   }
 
@@ -683,8 +726,7 @@ get_pscards(
   Py_ssize_t i         = 0;
 
   if (nps < 0) {
-    PyErr_SetString(PyExc_ValueError, "Object has no pscards");
-    return NULL;
+    nps = 0;
   }
 
   result = PyList_New((Py_ssize_t)nps);
@@ -692,6 +734,11 @@ get_pscards(
     return NULL;
   }
 
+  if (nps && ps == NULL) {
+    PyErr_SetString(PyExc_MemoryError, "NULL pointer");
+    return NULL;
+  }
+
   for (i = 0; i < (Py_ssize_t)nps; ++i) {
     subresult = Py_BuildValue("iis", ps[i].i, ps[i].m, ps[i].value);
     if (subresult == NULL) {
@@ -790,8 +837,7 @@ get_pvcards(
   Py_ssize_t i         = 0;
 
   if (npv < 0) {
-    PyErr_SetString(PyExc_ValueError, "Object has no pvcards");
-    return NULL;
+    npv = 0;
   }
 
   result = PyList_New((Py_ssize_t)npv);
@@ -799,6 +845,11 @@ get_pvcards(
     return NULL;
   }
 
+  if (npv && pv == NULL) {
+    PyErr_SetString(PyExc_MemoryError, "NULL pointer");
+    return NULL;
+  }
+
   for (i = 0; i < (Py_ssize_t)npv; ++i) {
     subresult = Py_BuildValue("iid", pv[i].i, pv[i].m, pv[i].value);
     if (subresult == NULL) {
@@ -824,65 +875,50 @@ set_pvcards(
     int *npv,
     int *npvmax) {
 
-  PyObject*  subvalue  = NULL;
-  int        i         = 0;
-  Py_ssize_t size      = 0;
-  int        ival      = 0;
-  int        mval      = 0;
-  double     dblvalue  = 0.0;
-  void*      newmem    = NULL;
+  PyObject* fastseq = NULL;
+  struct pvcard* newmem = NULL;
+  Py_ssize_t size;
+  int ret = -1;
+  int i;
 
-  if (!PySequence_Check(value)) {
-    return -1;
-  }
-  size = PySequence_Size(value);
-  if (size > 0x7fffffff) {
+  fastseq = PySequence_Fast(value, "Expected sequence type");
+  if (!fastseq)
+    goto done;
+
+  size = PySequence_Fast_GET_SIZE(value);
+  newmem = malloc(sizeof(struct pvcard) * size);
+
+  /* Raise exception if size is nonzero but newmem
+   * could not be allocated. */
+  if (size && !newmem) {
+    PyErr_SetString(PyExc_MemoryError, "Could not allocate memory.");
     return -1;
   }
 
-  if (size > (Py_ssize_t)*npvmax) {
-    newmem = malloc(sizeof(struct pvcard) * size);
-    if (newmem == NULL) {
-      PyErr_SetString(PyExc_MemoryError, "Could not allocate memory.");
-      return -1;
+  for (i = 0; i < size; ++i)
+  {
+    if (!PyArg_ParseTuple(PySequence_Fast_GET_ITEM(value, i), "iid",
+        &newmem[i].i, &newmem[i].m, &newmem[i].value))
+    {
+      goto done;
     }
+  }
+
+  if (size <= (Py_ssize_t)*npvmax) {
+    memcpy(*pv, newmem, sizeof(struct pvcard) * size);
+  } else { /* (size > (Py_ssize_t)*npvmax) */
     free(*pv);
+    *npv = (int)size;
     *pv = newmem;
-    *npvmax = (int)size;
+    newmem = NULL;
   }
+  *npv = (int)size;
 
-  /* Verify the entire list for correct types first, so we don't have
-     to undo anything copied into the canonical array. */
-  for (i = 0; i < size; ++i) {
-    subvalue = PySequence_GetItem(value, i);
-    if (subvalue == NULL) {
-      return -1;
-    }
-    if (!PyArg_ParseTuple(subvalue, "iid", &ival, &mval, &dblvalue)) {
-      Py_DECREF(subvalue);
-      return -1;
-    }
-    Py_DECREF(subvalue);
-  }
-
-  for (i = 0; i < size; ++i) {
-    subvalue = PySequence_GetItem(value, i);
-    if (subvalue == NULL) {
-      return -1;
-    }
-    if (!PyArg_ParseTuple(subvalue, "iid", &ival, &mval, &dblvalue)) {
-      Py_DECREF(subvalue);
-      return -1;
-    }
-    Py_DECREF(subvalue);
-
-    (*pv)[i].i = ival;
-    (*pv)[i].m = mval;
-    (*pv)[i].value = dblvalue;
-    (*npv) = i + 1;
-  }
-
-  return 0;
+  ret = 0;
+done:
+  Py_XDECREF(fastseq);
+  free(newmem);
+  return ret;
 }
 
 PyObject*
