@@ -5,10 +5,11 @@
 
 #define NO_IMPORT_ARRAY
 
-#include "wcslib_wrap.h"
-#include "wcslib_tabprm_wrap.h"
-#include "wcslib_wtbarr_wrap.h"
-#include "wcslib_units_wrap.h"
+#include "astropy_wcs/wcslib_wrap.h"
+#include "astropy_wcs/wcslib_tabprm_wrap.h"
+#include "astropy_wcs/wcslib_wtbarr_wrap.h"
+#include "astropy_wcs/wcslib_units_wrap.h"
+#include "astropy_wcs/unit_list_proxy.h"
 #include <structmember.h> /* from Python */
 
 #include <wcs.h>
@@ -18,8 +19,8 @@
 #include <wcsprintf.h>
 #include <wcsunits.h>
 
-#include "isnan.h"
-#include "distortion.h"
+#include "astropy_wcs/isnan.h"
+#include "astropy_wcs/distortion.h"
 
 /*
  It gets to be really tedious to type long docstrings in ANSI C syntax
@@ -27,7 +28,7 @@
  docstrings are written in doc/docstrings.py, which are then converted
  by setup.py into docstrings.h, which we include here.
 */
-#include "docstrings.h"
+#include "astropy_wcs/docstrings.h"
 
 /***************************************************************************
  * Helper functions                                                        *
@@ -53,6 +54,101 @@ is_valid_alt_key(
   return 1;
 }
 
+static int
+convert_rejections_to_warnings() {
+  char buf[1024];
+  const char *src;
+  char *dst;
+  int last_was_space;
+  PyObject *wcs_module = NULL;
+  PyObject *FITSFixedWarning = NULL;
+  int status = -1;
+
+  if (wcsprintf_buf()[0] == 0) {
+    return 0;
+  }
+
+  wcs_module = PyImport_ImportModule("astropy.wcs");
+  if (wcs_module == NULL) {
+    goto exit;
+  }
+
+  FITSFixedWarning = PyObject_GetAttrString(
+      wcs_module, "FITSFixedWarning");
+  if (FITSFixedWarning == NULL) {
+    goto exit;
+  }
+
+  src = wcsprintf_buf();
+  while (*src != 0) {
+    dst = buf;
+
+    /* Read the first line, removing any repeated spaces */
+    last_was_space = 0;
+    for (; *src != 0; ++src) {
+      if (*src == ' ') {
+        if (!last_was_space) {
+          *(dst++) = *src;
+          last_was_space = 1;
+        }
+      } else if (*src == '\n') {
+        ++src;
+        break;
+      } else {
+        *(dst++) = *src;
+        last_was_space = 0;
+      }
+    }
+
+    *(dst++) = '\n';
+
+    /* For the second line, remove everything up to and including the
+       first colon */
+    for (; *src != 0; ++src) {
+      if (*src == ':') {
+        ++src;
+        break;
+      }
+    }
+
+    /* Read to the end of the second line, removing any repeated
+       spaces */
+    last_was_space = 1;
+    for (; *src != 0; ++src) {
+      if (*src == ' ') {
+        if (!last_was_space) {
+          *(dst++) = *src;
+          last_was_space = 1;
+        }
+      } else if (*src == '\n') {
+        ++src;
+        break;
+      } else {
+        *(dst++) = *src;
+        last_was_space = 0;
+      }
+    }
+
+    /* NULL terminate the string */
+    *dst = 0;
+
+    /* Raise the warning.  Depending on the user's configuration, this
+       may raise an exception, and PyErr_WarnEx returns -1. */
+    if (PyErr_WarnEx(FITSFixedWarning, buf, 1)) {
+      goto exit;
+    }
+  }
+
+  status = 0;
+
+ exit:
+
+  Py_XDECREF(wcs_module);
+  Py_XDECREF(FITSFixedWarning);
+
+  return status;
+}
+
 /***************************************************************************
  * PyWcsprm methods
  */
@@ -110,7 +206,6 @@ PyWcsprm_init(
   PyObject*      colsel        = Py_None;
   PyArrayObject* colsel_array  = NULL;
   int*           colsel_ints   = NULL;
-  int            ctrl          = 0;
   int            nreject       = 0;
   int            nwcs          = 0;
   struct wcsprm* wcs           = NULL;
@@ -126,13 +221,6 @@ PyWcsprm_init(
   }
 
   if (header_obj == NULL || header_obj == Py_None) {
-    if (relax_obj != NULL && relax_obj != Py_False) {
-      PyErr_SetString(
-          PyExc_ValueError,
-          "If no header is provided, relax may not be provided either.");
-      return -1;
-    }
-
     if (keysel > 0) {
       PyErr_SetString(
           PyExc_ValueError,
@@ -203,6 +291,8 @@ PyWcsprm_init(
             "relax must be True, False or an integer.");
         return -1;
       }
+      /* Mask out any invalid flags */
+      relax &= WCSHDR_all;
     }
 
     if (!is_valid_alt_key(key)) {
@@ -248,12 +338,55 @@ PyWcsprm_init(
       Py_DECREF(colsel_array);
     }
 
+    wcsprintf_set(NULL);
+
+    /* Call the header parser twice, the first time to get warnings
+       out about "rejected" keywords (which we can then send to Python
+       as warnings), and the second time to get a corrected wcsprm
+       object. */
+
+    if (keysel < 0) {
+      status = wcspih(
+          header,
+          (int)nkeyrec,
+          WCSHDR_reject,
+          2,
+          &nreject,
+          &nwcs,
+          &wcs);
+    } else {
+      status = wcsbth(
+          header,
+          (int)nkeyrec,
+          WCSHDR_reject,
+          2,
+          keysel,
+          colsel_ints,
+          &nreject,
+          &nwcs,
+          &wcs);
+    }
+
+    if (status != 0) {
+      free(colsel_ints);
+      PyErr_SetString(
+          PyExc_MemoryError,
+          "Memory allocation error.");
+      return -1;
+    }
+
+    if (convert_rejections_to_warnings(wcsprintf_buf())) {
+      free(colsel_ints);
+      wcsvfree(&nwcs, &wcs);
+      return -1;
+    }
+
     if (keysel < 0) {
       status = wcspih(
           header,
           (int)nkeyrec,
           relax,
-          ctrl,
+          0,
           &nreject,
           &nwcs,
           &wcs);
@@ -262,7 +395,7 @@ PyWcsprm_init(
           header,
           (int)nkeyrec,
           relax,
-          ctrl,
+          0,
           keysel,
           colsel_ints,
           &nreject,
@@ -360,7 +493,6 @@ PyWcsprm_find_all_wcs(
   PyObject*      relax_obj     = NULL;
   int            relax         = 0;
   int            keysel        = 0;
-  int            ctrl          = 0;
   int            nreject       = 0;
   int            nwcs          = 0;
   struct wcsprm* wcs           = NULL;
@@ -408,6 +540,50 @@ PyWcsprm_find_all_wcs(
           "relax must be True, False or an integer.");
       return NULL;
     }
+
+    /* Mask out any invalid flags */
+    relax &= WCSHDR_all;
+  }
+
+  /* Call the header parser twice, the first time to get warnings
+     out about "rejected" keywords (which we can then send to Python
+     as warnings), and the second time to get a corrected wcsprm
+     object. */
+
+  Py_BEGIN_ALLOW_THREADS
+  if (keysel < 0) {
+    status = wcspih(
+        header,
+        (int)nkeyrec,
+        WCSHDR_reject,
+        2,
+        &nreject,
+        &nwcs,
+        &wcs);
+  } else {
+    status = wcsbth(
+        header,
+        (int)nkeyrec,
+        WCSHDR_reject,
+        2,
+        keysel,
+        NULL,
+        &nreject,
+        &nwcs,
+        &wcs);
+  }
+  Py_END_ALLOW_THREADS
+
+  if (status != 0) {
+    PyErr_SetString(
+        PyExc_MemoryError,
+        "Memory allocation error.");
+    return NULL;
+  }
+
+  if (convert_rejections_to_warnings(wcsprintf_buf())) {
+    wcsvfree(&nwcs, &wcs);
+    return NULL;
   }
 
   Py_BEGIN_ALLOW_THREADS
@@ -416,7 +592,7 @@ PyWcsprm_find_all_wcs(
         header,
         (int)nkeyrec,
         relax,
-        ctrl,
+        0,
         &nreject,
         &nwcs,
         &wcs);
@@ -425,7 +601,7 @@ PyWcsprm_find_all_wcs(
         header,
         (int)nkeyrec,
         relax,
-        ctrl,
+        0,
         keysel,
         NULL,
         &nreject,
@@ -466,12 +642,6 @@ PyWcsprm_find_all_wcs(
     }
 
     subresult->x.flag = 0;
-    if (PyWcsprm_cset(subresult, 0)) {
-      Py_DECREF(subresult);
-      Py_DECREF(result);
-      wcsvfree(&nwcs, &wcs);
-      return NULL;
-    }
     wcsprm_c2python(&subresult->x);
   }
 
@@ -541,7 +711,7 @@ PyWcsprm_cylfix(
     return NULL;
   }
 
-  if (naxis_obj != NULL) {
+  if (naxis_obj != NULL && naxis_obj != Py_None) {
     naxis_array = (PyArrayObject*)PyArray_ContiguousFromAny(
         naxis_obj, 1, 1, PyArray_INT);
     if (naxis_array == NULL) {
@@ -644,7 +814,7 @@ PyWcsprm_fix(
     }
   }
 
-  if (naxis_obj != NULL) {
+  if (naxis_obj != NULL && naxis_obj != Py_None) {
     naxis_array = (PyArrayObject*)PyArray_ContiguousFromAny(
         naxis_obj, 1, 1, PyArray_INT);
     if (naxis_array == NULL) {
@@ -753,13 +923,6 @@ PyWcsprm_get_ps(
     /*@unused@*/ PyObject* args,
     /*@unused@*/ PyObject* kwds) {
 
-  if (self->x.ps == NULL) {
-    PyErr_SetString(
-        PyExc_AssertionError,
-        "No PSi_ma records present.");
-    return NULL;
-  }
-
   return get_pscards("ps", self->x.ps, self->x.nps);
 }
 
@@ -769,13 +932,6 @@ PyWcsprm_get_pv(
     /*@unused@*/ PyObject* args,
     /*@unused@*/ PyObject* kwds) {
 
-  if (self->x.pv == NULL) {
-    PyErr_SetString(
-        PyExc_AssertionError,
-        "No PVi_ma records present.");
-    return NULL;
-  }
-
   return get_pvcards("pv", self->x.pv, self->x.npv);
 }
 
@@ -1333,18 +1489,14 @@ PyWcsprm_set_pv(
 
   if (is_null(self->x.pv)) {
     return NULL;
-  }
-
-  if (set_pvcards("pv", arg, &self->x.pv, &self->x.npv, &self->x.npvmax)) {
-    self->x.m_pv = self->x.pv;
+  } else if (set_pvcards("pv", arg, &self->x.pv, &self->x.npv, &self->x.npvmax)) {
     return NULL;
+  } else {
+    self->x.m_pv = self->x.pv;
+    note_change(self);
+    Py_INCREF(Py_None);
+    return Py_None;
   }
-  self->x.m_pv = self->x.pv;
-
-  note_change(self);
-
-  Py_INCREF(Py_None);
-  return Py_None;
 }
 
 /* TODO: This is convenient for debugging for now -- but it's not very
@@ -1585,11 +1737,7 @@ PyWcsprm_sub(
     goto exit;
   }
 
-  if (nsub == 0) {
-    alloc_size = self->x.naxis;
-  } else {
-    alloc_size = nsub;
-  }
+  alloc_size = self->x.naxis;
 
   py_dest_wcs = (PyWcsprm*)PyWcsprm_cnew();
   py_dest_wcs->x.flag = -1;
@@ -1653,7 +1801,7 @@ PyWcsprm_to_header(
   if (relax_obj == Py_True) {
     relax = WCSHDO_all;
   } else if (relax_obj == NULL || relax_obj == Py_False) {
-    relax = WCSHDO_none;
+    relax = WCSHDO_safe;
   } else {
     #if PY3K
     relax = (int)PyLong_AsLong(relax_obj);
@@ -1673,9 +1821,7 @@ PyWcsprm_to_header(
   wcsprm_c2python(&self->x);
 
   if (status != 0) {
-    PyErr_SetString(
-        PyExc_RuntimeError,
-        "Unknown error occurred.  Something is seriously wrong.");
+    wcs_to_python_exc(&(self->x));
     goto exit;
   }
 
@@ -2237,23 +2383,6 @@ PyWcsprm_set_cubeface(
   return set_int("cubeface", value, &self->x.cubeface);
 }
 
-static int
-unit_verify(char* val) {
-
-  int status, func;
-  double scale, units[WCSUNITS_NTYPE];
-  struct wcserr *err = NULL;
-
-  status = wcsulexe(val, &func, &scale, units, &err);
-  if (status == 0) {
-    return 1;
-  } else {
-    wcserr_units_to_python_exc(err);
-    free(err);
-    return 0;
-  }
-}
-
 /*@null@*/ static PyObject*
 PyWcsprm_get_cunit(
     PyWcsprm* self,
@@ -2263,9 +2392,8 @@ PyWcsprm_get_cunit(
     return NULL;
   }
 
-  return get_str_list_verified(
-    "cunit", self->x.cunit, (Py_ssize_t)self->x.naxis, 68, (PyObject*)self,
-    unit_verify);
+  return get_unit_list(
+    "cunit", self->x.cunit, (Py_ssize_t)self->x.naxis, (PyObject*)self);
 }
 
 static int
@@ -2280,9 +2408,8 @@ PyWcsprm_set_cunit(
 
   note_change(self);
 
-  return set_str_list_verified(
-    "cunit", value, (Py_ssize_t)self->x.naxis, 0, self->x.cunit,
-    unit_verify);
+  return set_unit_list(
+    (PyObject *)self, "cunit", value, (Py_ssize_t)self->x.naxis, self->x.cunit);
 }
 
 /*@null@*/ static PyObject*
@@ -3155,7 +3282,7 @@ PyTypeObject PyWcsprmType = {
   PyObject_HEAD_INIT(NULL)
   0,                            /*ob_size*/
   #endif
-  "astropy.wcs._Wcsprm",              /*tp_name*/
+  "astropy.wcs.Wcsprm",              /*tp_name*/
   sizeof(PyWcsprm),             /*tp_basicsize*/
   0,                            /*tp_itemsize*/
   (destructor)PyWcsprm_dealloc, /*tp_dealloc*/
@@ -3210,7 +3337,7 @@ _setup_wcsprm_type(
   wcserr_enable(1);
 
   return (
-    PyModule_AddObject(m, "_Wcsprm", (PyObject *)&PyWcsprmType) ||
+    PyModule_AddObject(m, "Wcsprm", (PyObject *)&PyWcsprmType) ||
     CONSTANT(WCSSUB_LONGITUDE) ||
     CONSTANT(WCSSUB_LATITUDE)  ||
     CONSTANT(WCSSUB_CUBEFACE)  ||
