@@ -1,20 +1,31 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+from ...extern import six
+
 import os
 import sys
 import warnings
 
 import numpy as np
 from numpy.testing import (
-    assert_array_almost_equal, assert_array_almost_equal_nulp)
+    assert_allclose, assert_array_almost_equal, assert_array_almost_equal_nulp)
 
-from ...tests.helper import raises
+from ...tests.helper import raises, catch_warnings, pytest
 from ... import wcs
 from ...utils.data import (
     get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
-from ...utils.misc import NumpyRNGContext
 from ...tests.helper import pytest
+from ...utils.misc import NumpyRNGContext
 
 
+try:
+    import scipy  # pylint: disable=W0611
+except ImportError:
+    HAS_SCIPY = False
+else:
+    HAS_SCIPY = True
+
 # test_maps() is a generator
 def test_maps():
 
@@ -216,14 +227,15 @@ def test_fixes():
             'data/nonstandard_units.hdr', encoding='binary')
         w = wcs.WCS(header)
 
-    with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter("always")
+    with catch_warnings(wcs.FITSFixedWarning) as w:
         run()
-        assert len(w) == 2
-        for item in w:
-            assert issubclass(item.category, wcs.FITSFixedWarning)
-            if 'unitfix' in str(item.message):
-                assert 'Hz' in str(item.message)
+
+    assert len(w) == 2
+    for item in w:
+        if 'unitfix' in str(item.message):
+            assert 'Hz' in str(item.message)
+            assert 'M/S' in str(item.message)
+            assert 'm/s' in str(item.message)
 
 
 def test_outside_sky():
@@ -239,6 +251,40 @@ def test_outside_sky():
     assert not np.any(np.isnan(w.wcs_pix2world([[1000., 1000.]], 0)))
 
 
+def test_pix2world():
+    """
+    From github issue #1463
+    """
+    # TODO: write this to test the expected output behavior of pix2world,
+    # currently this just makes sure it doesn't error out in unexpected ways
+    filename = get_pkg_data_filename('data/sip2.fits')
+    with catch_warnings(wcs.wcs.FITSFixedWarning) as caught_warnings:
+        # this raises a warning unimportant for this testing the pix2world
+        #   FITSFixedWarning(u'The WCS transformation has more axes (2) than the
+        #        image it is associated with (0)')
+        ww = wcs.WCS(filename)
+
+        # might as well monitor for changing behavior
+        assert len(caught_warnings) == 1
+
+    n = 3
+    pixels = (np.arange(n)*np.ones((2, n))).T
+    result = ww.wcs_pix2world(pixels, 0, ra_dec_order=True)
+
+    close_enough = 1e-8
+    # assuming that the data of sip2.fits doesn't change
+    answer = np.array([[0.00024976, 0.00023018],
+                       [0.00023043, -0.00024997]])
+
+    assert np.all(np.abs(ww.wcs.pc-answer) < close_enough)
+
+    answer = np.array([[ 202.39265216,   47.17756518],
+                       [ 202.39335826,   47.17754619],
+                       [ 202.39406436,   47.1775272 ]])
+
+    assert  np.all(np.abs(result-answer) < close_enough)
+
+
 def test_load_fits_path():
     fits = get_pkg_data_filename('data/sip.fits')
     w = wcs.WCS(fits)
@@ -379,15 +425,30 @@ def test_invalid_shape():
         xy2 = w.wcs_pix2world(xy, 1)
 
 
-@raises(wcs._wcs.InvalidTransformError)
-def test_find_all_wcs_crash():
-    """
-    Causes a double free without a recent fix in wcslib_wrap.C
-    """
-    from ...io import fits
+def test_warning_about_defunct_keywords():
+    def run():
+        header = get_pkg_data_contents(
+            'data/defunct_keywords.hdr', encoding='binary')
+        w = wcs.WCS(header)
 
-    with fits.open(get_pkg_data_filename("data/too_many_pv.hdr")) as hdulist:
-        wcses = wcs.find_all_wcs(hdulist[0].header)
+    with catch_warnings(wcs.FITSFixedWarning) as w:
+        run()
+
+    assert len(w) == 4
+    for item in w:
+        assert 'PCi_ja' in str(item.message)
+
+
+@raises(wcs.FITSFixedWarning)
+def test_warning_about_defunct_keywords_exception():
+    def run():
+        header = get_pkg_data_contents(
+            'data/defunct_keywords.hdr', encoding='binary')
+        w = wcs.WCS(header)
+
+    with catch_warnings(wcs.FITSFixedWarning) as w:
+        warnings.simplefilter("error", wcs.FITSFixedWarning)
+        run()
 
 
 def test_to_header_string():
@@ -401,3 +462,116 @@ def test_to_header_string():
 def test_to_fits():
     w = wcs.WCS()
     w.to_fits()
+
+
+@raises(wcs.InvalidTransformError)
+def test_find_all_wcs_crash():
+    """
+    Causes a double free without a recent fix in wcslib_wrap.C
+    """
+    with open(get_pkg_data_filename("data/too_many_pv.hdr")) as fd:
+        header = fd.read()
+    # We have to set fix=False here, because one of the fixing tasks is to
+    # remove redundant SCAMP distortion parameters when SIP distortion
+    # parameters are also present.
+    wcses = wcs.find_all_wcs(header, fix=False)
+
+
+def test_validate():
+    results = wcs.validate(get_pkg_data_filename("data/validate.fits"))
+    results_txt = repr(results)
+    with open(get_pkg_data_filename("data/validate.txt"), "r") as fd:
+        assert set([x.strip() for x in fd.readlines()]) == set([
+            x.strip() for x in results_txt.splitlines()])
+
+
+def test_validate_with_2_wcses():
+    # From Issue #2053
+    results = wcs.validate(get_pkg_data_filename("data/2wcses.hdr"))
+
+    assert "WCS key 'A':" in six.text_type(results)
+
+
+@pytest.mark.skipif(str('not HAS_SCIPY'))
+def test_all_world2pix():
+    """Test all_world2pix, iterative inverse of all_pix2world"""
+    fits = get_pkg_data_filename('data/sip.fits')
+    w = wcs.WCS(fits)
+
+    tolerance = 1e-6
+    with NumpyRNGContext(123456789):
+        world = 0.1 * np.random.randn(100, 2)
+        for i in range(len(w.wcs.crval)):
+            world[:, i] += w.wcs.crval[i]
+        all_pix = w.all_world2pix(world, 0, tolerance=tolerance)
+        wcs_pix = w.wcs_world2pix(world, 0)
+        all_world = w.all_pix2world(all_pix, 0)
+
+        # First, check that the SIP distortion correction at least produces
+        # some different answers from the WCS-only transform.
+        assert np.any(all_pix != wcs_pix)
+
+        assert_allclose(all_world, world, rtol=0, atol=tolerance)
+
+
+def test_scamp_sip_distortion_parameters():
+    """
+    Test parsing of WCS parameters with redundant SIP and SCAMP distortion
+    parameters.
+    """
+    header = get_pkg_data_contents('data/validate.fits', encoding='binary')
+    w = wcs.WCS(header)
+    # Just check that this doesn't raise an exception.
+    w.all_pix2world(0, 0, 0)
+
+
+def test_fixes():
+    """
+    From github issue #1854
+    """
+    header = get_pkg_data_contents(
+        'data/nonstandard_units.hdr', encoding='binary')
+    w = wcs.WCS(header, fix=False)
+
+    with pytest.raises(wcs.InvalidTransformError):
+        w.to_header()
+
+
+def test_footprint_to_file(tmpdir):
+    """
+    From github issue #1912
+    """
+    # Arbitrary keywords from real data
+    w = wcs.WCS({'CTYPE1': 'RA---ZPN', 'CRUNIT1': 'deg',
+                 'CRPIX1': -3.3495999e+02, 'CRVAL1': 3.185790700000e+02,
+                 'CTYPE2': 'DEC--ZPN', 'CRUNIT2': 'deg',
+                 'CRPIX2': 3.0453999e+03, 'CRVAL2': 4.388538000000e+01,
+                 'PV2_1': 1., 'PV2_3': 220.})
+    # Just check that this doesn't raise an exception:
+    w.footprint_to_file(str(tmpdir.join('test.txt')))
+
+
+def test_validate_faulty_wcs():
+    """
+    From github issue #2053
+    """
+    from ...io import fits
+    h = fits.Header()
+    # Illegal WCS:
+    h['RADESYSA'] = 'ICRS'
+    h['PV2_1'] = 1.0
+    hdu = fits.PrimaryHDU([[0]], header=h)
+    hdulist = fits.HDUList([hdu])
+    # Check that this doesn't raise a NameError exception:
+    wcs.validate(hdulist)
+
+
+def test_error_message():
+    header = get_pkg_data_contents(
+        'data/invalid_header.hdr', encoding='binary')
+
+    with pytest.raises(wcs.InvalidTransformError):
+        # Both lines are in here, because 0.4 calls .set within WCS.__init__,
+        # whereas 0.3 and earlier did not.
+        w = wcs.WCS(header)
+        c = w.all_pix2world([[536.0, 894.0]], 0)
